package com.bcxin.sp.config.aspect;

import com.alibaba.fastjson.JSON;
import com.bcxin.sp.config.annotation.SysLog;
import com.bcxin.sp.work.dao.log.SystemLogMapper;
import com.bcxin.sp.work.entity.domain.log.SystemLog;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.util.Const.GlobalConst;
import com.bcxin.sp.work.util.HttpContextUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;


/**
 * 系统日志，切面处理类
 */
@Aspect
@Component
public class SysLogAspect {

	@Resource
	private SystemLogMapper systemLogMapper;
	
	@Pointcut("@annotation(com.bcxin.sp.config.annotation.SysLog)")
	public void logPointCut() {
	}

	@Around("logPointCut()")
	public Object around(ProceedingJoinPoint point) throws Throwable {
		long beginTime = System.currentTimeMillis();
		//执行方法
		Object result = point.proceed();
		//执行时长(毫秒)
		long time = System.currentTimeMillis() - beginTime;
		//保存日志
		saveSysLog(point, time);
		return result;
	}

	private void saveSysLog(ProceedingJoinPoint joinPoint, long time) {
		try {
			MethodSignature signature = (MethodSignature) joinPoint.getSignature();
			Method method = signature.getMethod();

			SystemLog sysLog = new SystemLog();
			SysLog syslog = method.getAnnotation(SysLog.class);
			if (syslog != null) {
				//注解上的描述
				sysLog.setOperation(syslog.value());
			}

			//请求的方法名
			String className = joinPoint.getTarget().getClass().getName();
			String methodName = signature.getName();
			sysLog.setMethod(className + "." + methodName + "()");

			//请求的参数
			Object[] args = joinPoint.getArgs();
			try {
				StringBuffer params = new StringBuffer();
				if (args != null && args.length > 0) {
					for (Object obj : args) {
						params.append(JSON.toJSONString(obj)).append("&");
					}
				}
				sysLog.setParams(params.toString());
			} catch (Exception e) {

			}

			//获取request
			HttpServletRequest request = HttpContextUtils.getHttpServletRequest();

			//用户名
			Object obj = request.getSession().getAttribute(GlobalConst.LOGIN_USER);
			if (obj != null) {
				User user = (User) obj;
				sysLog.setUserId(user.getId());
			}
			sysLog.setTime(time);
			//保存系统日志
			systemLogMapper.insert(sysLog);
		} catch (Exception e){
			e.printStackTrace();
			/* 如果保存异常就算了，不要影响正常业务 */
		}
	}
}
