package com.bcxin.sp.manager.factory;

import java.util.TimerTask;

import com.bcxin.sp.shiro.session.OnlineSession;
import com.bcxin.sp.work.entity.domain.user.LoginUserDetail;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.domain.user.UserOnline;
import com.bcxin.sp.work.service.log.AuditLogService;
import com.bcxin.sp.work.service.user.UserOnlineService;
import com.bcxin.sp.work.service.user.UserService;
import com.bcxin.sp.work.util.ServletUtils;
import com.bcxin.sp.work.util.ShiroUtils;
import com.bcxin.sp.work.util.SpringContextUtil;
import com.mysql.cj.core.util.LogUtils;
import eu.bitwalker.useragentutils.UserAgent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 异步工厂（产生任务用）
 * 
 * @author liuhulu
 *
 */
public class AsyncFactory
{
    private static final Logger sys_user_logger = LoggerFactory.getLogger("sys-user");

    /**
     * 同步session到数据库
     * 
     * @param session 在线用户会话
     * @return 任务task
     */
    public static TimerTask syncSessionToDb(final OnlineSession session)
    {
        return new TimerTask()
        {
            @Override
            public void run()
            {
                UserOnline online = new UserOnline();
                online.setSessionId(String.valueOf(session.getId()));
                online.setDeptName(session.getDeptName());
                online.setLoginName(session.getLoginName());
                online.setStartTimestamp(session.getStartTimestamp());
                online.setLastAccessTime(session.getLastAccessTime());
                online.setExpireTime(session.getTimeout());
                online.setIpaddr(session.getHost());
                online.setLoginLocation("内网IP");
                online.setBrowser(session.getBrowser());
                online.setOs(session.getOs());
                online.setStatus(session.getStatus());
                SpringContextUtil.getBean(UserOnlineService.class).saveOnline(online);

            }
        };
    }

    /**
     * 审计日志记录
     *
     * @return 任务task
     */
    public static TimerTask recordAuditLog(final int opType,final String opCondition,final String opResult,final LoginUserDetail detail)
    {
        return new TimerTask()
        {
            @Override
            public void run()
            {
                SpringContextUtil.getBean(AuditLogService.class).auditLog1(opType,opCondition,opResult,detail);
            }
        };
    }

    /**
     * 审计日志记录
     *
     * @return 任务task
     */
    public static TimerTask recordLoginLog(final User user)
    {
        return new TimerTask()
        {
            @Override
            public void run()
            {
                SpringContextUtil.getBean(UserService.class).logLogin(user);
            }
        };
    }

    /**
     * 记录登录信息
     * 
     * @param username 用户名
     * @param status 状态
     * @param message 消息
     * @param args 列表
     * @return 任务task
     */
//    public static TimerTask recordLogininfor(final String username, final String status, final String message, final Object... args)
//    {
//        final UserAgent userAgent = UserAgent.parseUserAgentString(ServletUtils.getRequest().getHeader("User-Agent"));
//        final String ip = ShiroUtils.getIp();
//        return new TimerTask()
//        {
//            @Override
//            public void run()
//            {
//                String address = "内网IP";
//                StringBuilder s = new StringBuilder();
//                s.append(LogUtils.getBlock(ip));
//                s.append(address);
//                s.append(LogUtils.getBlock(username));
//                s.append(LogUtils.getBlock(status));
//                s.append(LogUtils.getBlock(message));
//                // 打印信息到日志
//                sys_user_logger.info(s.toString(), args);
//                // 获取客户端操作系统
//                String os = userAgent.getOperatingSystem().getName();
//                // 获取客户端浏览器
//                String browser = userAgent.getBrowser().getName();
//                // 封装对象
//                SysLogininfor logininfor = new SysLogininfor();
//                logininfor.setLoginName(username);
//                logininfor.setIpaddr(ip);
//                logininfor.setLoginLocation(address);
//                logininfor.setBrowser(browser);
//                logininfor.setOs(os);
//                logininfor.setMsg(message);
//                // 日志状态
//                if (StringUtils.equalsAny(status, Constants.LOGIN_SUCCESS, Constants.LOGOUT, Constants.REGISTER))
//                {
//                    logininfor.setStatus(Constants.SUCCESS);
//                }
//                else if (Constants.LOGIN_FAIL.equals(status))
//                {
//                    logininfor.setStatus(Constants.FAIL);
//                }
//                // 插入数据
//                SpringUtils.getBean(SysLogininforServiceImpl.class).insertLogininfor(logininfor);
//            }
//        };
//    }
}
