package com.bcxin.sp.work.controller.activity.action;

import com.bcxin.sp.config.annotation.AuditLogAn;
import com.bcxin.sp.config.annotation.OperLog;
import com.bcxin.sp.work.controller.sys.BaseController;
import com.bcxin.sp.work.entity.PageResult;
import com.bcxin.sp.work.entity.domain.activity.base.Activity;
import com.bcxin.sp.work.entity.domain.activity.history.ActivityHistory;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.ActivityHistoryDTO;
import com.bcxin.sp.work.entity.dto.AjaxResult;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.service.activity.action.ActivityActionService;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.MenuConst;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 上报活动审批
 */
@Controller
@RequestMapping("/business/activity/action")
public class ActivityActionController extends BaseController {

    @Resource
    private ActivityActionService activityActionService;

    @RequestMapping("/list")
    public ModelAndView listInit(HttpServletRequest request) {
        ModelAndView view = new ModelAndView("/activity/action/list");
        //隐藏左边菜单树标记
        view.addObject("hiddenLeft",request.getParameter("hiddenLeft"));
        //市级编码
        view.addObject("city",request.getParameter("city"));
        //区县级编码
        view.addObject("area",request.getParameter("area"));
        //活动开始起始日期
        view.addObject("startDate",request.getParameter("startDate"));
        //活动开始截至日期
        view.addObject("endDate",request.getParameter("endDate"));

        return view;
    }

    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.HDGL_GABA)
    @RequestMapping("findData")
    @ResponseBody
    public SearchDto<ActivityHistoryDTO> findData(HttpServletRequest request, SearchDto<ActivityHistory> dto) throws Exception {
        /* 查出所有的记录 */
        PageResult pageResult = activityActionService.findActivity(dto);
        List<ActivityHistoryDTO> resultList = (List<ActivityHistoryDTO>)pageResult.getRows();
        SearchDto<ActivityHistoryDTO> result = new SearchDto<ActivityHistoryDTO>();
        result.setData(resultList);
        result.setIDisplayStart(dto.getIDisplayStart());
        result.setIDisplayLength(dto.getIDisplayLength());
        result.setITotalRecords(pageResult.getTotal());
        result.setITotalDisplayRecords(pageResult.getTotal());
        return result;
    }

    /**
     * 上报信息
     * @return
     */
    @RequestMapping("/detail/{actionId}")
    public ModelAndView detail(@PathVariable("actionId") Long actionId){
        ModelAndView view = new ModelAndView("/activity/action/sessionList");
        Activity active = activityActionService.detail(actionId);
        view.addObject("activity", active);
        return view;
    }
    @AuditLogAn(operateType = DictConst.OP_UPDATE,menuName = MenuConst.HDGL_GABA_SCHD)
    @OperLog(menuName = "活动上报确认",operateType = "活动上报",operateObj = "#id")
    @RequestMapping("submit/{actionId}")
    public @ResponseBody AjaxResult delete(@PathVariable("actionId") Long actionId,HttpServletRequest request) throws Exception {
        User user = getCurrentUser(request);
        return activityActionService.uploadaction(actionId);
    }

}
