package com.bcxin.sp.work.controller.activity.base;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.bcxin.sp.config.annotation.AuditLogAn;
import com.bcxin.sp.config.annotation.SysLog;
import com.bcxin.sp.config.annotation.SysManageAuth;
import com.bcxin.sp.work.controller.sys.BaseController;
import com.bcxin.sp.work.dao.activity.extend.ActivityDetailMapper;
import com.bcxin.sp.work.dao.activity.extend.ActivitySessionMapper;
import com.bcxin.sp.work.entity.PageResult;
import com.bcxin.sp.work.entity.domain.activity.approve.Approve;
import com.bcxin.sp.work.entity.domain.activity.approve.ApproveHistory;
import com.bcxin.sp.work.entity.domain.activity.base.Activity;
import com.bcxin.sp.work.entity.domain.activity.expand.*;
import com.bcxin.sp.work.entity.domain.activity.session.ActivitySession;
import com.bcxin.sp.work.entity.domain.activity.survey.Survey;
import com.bcxin.sp.work.entity.domain.activity.survey.SurveyDetail;
import com.bcxin.sp.work.entity.domain.log.ChangeLog;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.*;
import com.bcxin.sp.work.service.activity.approve.ApproveService;
import com.bcxin.sp.work.service.activity.base.ActivityService;
import com.bcxin.sp.work.service.activity.extend.ActivityContractorService;
import com.bcxin.sp.work.service.activity.extend.WorkflowService;
import com.bcxin.sp.work.service.activity.survey.SurveyService;
import com.bcxin.sp.work.service.log.ChangeLogService;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.GlobalConst;
import com.bcxin.sp.work.util.Const.MenuConst;
import com.bcxin.sp.work.util.DESUtil;
import com.bcxin.sp.work.util.OperateLogUtil;
import com.bcxin.sp.work.util.StringUtil;
import com.bcxin.sp.work.util.WebUtil;
import com.bcxin.sp.work.util.sys.ConfigUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.io.Files;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 活动管理
 */
@Controller
@RequestMapping("/business/activity")
@Slf4j
public class ActivityController extends BaseController {

    @Resource
    private ActivityService activityService;

    @Resource
    private ApproveService approveService;

    @Resource
    private SurveyService surveyService;

    @Resource
    private WorkflowService workflowService;

    @Resource
    private ChangeLogService changeLogService;

    @Resource
    private ActivityContractorService activityContractorService;

    @Resource
    private ActivityDetailMapper activityDetailMapper;
    @Resource
    private ActivitySessionMapper activitySessionMapper;

    /**
     * 报备列表
     * @return
     */
    @RequestMapping("/list")
    public ModelAndView list() {
        return new ModelAndView("/activity/gov/list");
    }

    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.HDGL_BAHD)
    @RequestMapping("findData")
    public @ResponseBody SearchDto<Activity> findData(SearchDto<Activity> dto, HttpServletRequest request) throws Exception {
        User user = super.getCurrentUser(request);
        dto.setGov(true);
        List<Activity> activityList = activityService.findActivity(dto,user);
        long count = activityService.findCountOfActivity(dto,user);
        SearchDto<Activity> result = new SearchDto<Activity>();
        result.setData(activityList);
        result.setIDisplayStart(dto.getIDisplayStart());
        result.setIDisplayLength(dto.getIDisplayLength());
        result.setITotalRecords(count);
        result.setITotalDisplayRecords(count);
        return result;
    }

    @AuditLogAn(operateType = DictConst.OP_SAVE,menuName = MenuConst.HDGL_BAHD_TJHD)
    @RequestMapping("createOrUpdate")
    public @ResponseBody
    AjaxResult createOrUpdate(Activity activity, HttpServletRequest request) {
        User user = super.getCurrentUser(request);
        activity.setUpdater(user.getUsername());
        activityService.createOrUpdate(activity);
        AjaxResult result = new AjaxResult();
        result.setSuccessful(true);
        result.setObj(activity);
        return result;
    }

    @AuditLogAn(operateType = DictConst.OP_DELETE,menuName = MenuConst.HDGL_BAHD_SCHD)
    @RequestMapping("delete/{id}")
    public @ResponseBody
    AjaxResult delete(@PathVariable("id") long id) {
        AjaxResult result = new AjaxResult();
        Activity activity = activityService.findById(id);
        if (activity.getApproveStatus().equals(DictConst.STATUS_NONE)) {
            activityService.delete(id);
            result.setSuccessful(true);
        } else {
            result.setSuccessful(false);
            result.setMsg("当前记录无法删除.");
        }
        return result;
    }

    @RequestMapping("downloadStuff/{activityId}")
    public ResponseEntity<byte[]> downloadStuff(@PathVariable("activityId") Long activityId) throws IOException {
        Activity activity = activityService.findById(activityId);
        ActivityMaterial material = activityService.findMaterialByActivity(activityId);
        if (activity != null && !StrUtil.isEmpty(material.getBpcl())) {
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
            String stuffPath = ConfigUtil.getValue("stuff-path") + material.getBpcl();
            String zipFileName = material.getBpcl();
            String fileType = zipFileName.substring(zipFileName.lastIndexOf("."), zipFileName.length());
            File file = new File(stuffPath);
            if (file != null) {
                String fileName = new String((activity.getFullName() + "审批材料" + fileType).getBytes("GBK"), "iso-8859-1");
                headers.setContentDispositionFormData("attachment", fileName);
                try {
                    return new ResponseEntity<byte[]>(FileUtils.readFileToByteArray(file), headers, HttpStatus.OK);
                } catch (IOException e) {
                    log.error(e.getMessage());
                    e.printStackTrace();
                    return null;
                }
            }
        }
        return null;
    }


    /**
     * 获取审批详情（初审，受理，审批 三个阶段都包括）
     *
     * @param activityId
     * @return
     * @throws Exception
     */
    @RequestMapping("approve/{activityId}")
    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.HDGL_XKHD_HDXQ)
    public @ResponseBody List<Approve> approveInfo(@PathVariable("activityId") Long activityId) throws Exception {
        return approveService.findByActivityId(activityId);
    }

    /**
     * 获取审批历史记录
     *
     * @param activityId
     * @return
     * @throws Exception
     */
    @RequestMapping("approve/history/{activityId}")
    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.HDGL_XKHD_SPLS)
    public @ResponseBody List<ApproveHistory> approveHistoryInfo(@PathVariable("activityId") Long activityId) throws Exception {
        return approveService.findHistoryByActivityId(activityId);
    }

    @RequestMapping("changeLogList/{activityId}")
    @ResponseBody
    public List<ChangeLog> selectChangeLog(@PathVariable("activityId") Long activityId){
        List<ChangeLog> changLogList=changeLogService.selectChangeLog(activityId);
        transferChangeLog(changLogList);
        return changLogList;
    }


    /**
     * 获取活动数据
     * @param activityId
     * @return
     */
    @RequestMapping("activityBase/{activityId}")
    @ResponseBody
    public Activity selectActivityBase(@PathVariable("activityId") Long activityId){
        return activityService.findFullById(activityId);
    }

    /**
     * 获取活动数据
     * @param activityId
     * @return
     */
    @RequestMapping("activityDetail/{activityId}")
    @ResponseBody
    public ActivityDetail selectActivityDetail(@PathVariable("activityId") Long activityId){
        return activityDetailMapper.findByActivityId(activityId);
    }

    /**
     * 获取活动上报数据
     * @param activityId
     * @return
     */
    @RequestMapping("activityReportDetail/{activityId}")
    @ResponseBody
    public ActivityReportDetail selectActivityReportDetail(@PathVariable("activityId") Long activityId){
        return activityService.findReportDetailById(activityId);
    }

    @RequestMapping("mainContractor/{activityId}")
    public @ResponseBody ActivityContractor mainContractor(@PathVariable("activityId") Long activityId, HttpServletRequest request) {
        List<ActivityContractor> list = activityContractorService.selectByActivityId(activityId);
        if (CollUtil.isEmpty(list)) {
            return new ActivityContractor();
        }
        return list.get(0);
    }

    /**
     * 转换显示
     * @param changeLogList
     */
    private void transferChangeLog(List<ChangeLog> changeLogList){
        if(changeLogList!=null){
            for(ChangeLog log : changeLogList){
                String sourceVal="";
                String newVal="";
                if(log.getSourceVal().contains("&&") || log.getSourceVal().contains("||")){
                    String [] tmp=log.getSourceVal().split("&&");
                    for(String s : tmp){
                        String [] sub=s.split("\\|\\|");
                        if(sub.length>2){
                            sourceVal+=sub[2]+",";
                        }
                    }
                    if(StringUtil.isNotEmpty(sourceVal)){
                        log.setSourceVal(sourceVal.substring(0,sourceVal.length()-1));
                    }
                }
                // 新值
                if(log.getNowVal().contains("&&") || log.getNowVal().contains("||")){
                    String [] tmp=log.getNowVal().split("&&");
                    for(String s : tmp){
                        String [] sub=s.split("\\|\\|");
                        if(sub.length>2){
                            newVal+=sub[2]+",";
                        }
                    }
                    if(StringUtil.isNotEmpty(newVal)){
                        log.setNowVal(newVal.substring(0,newVal.length()-1));
                    }
                }
            }
        }
    }


    /**
     * 获取材料详情
     *
     * @param activityId
     * @return
     * @throws Exception
     */
    @RequestMapping("material/{activityId}")
    @ResponseBody
    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.HDGL_XKHD_HDXQ)
    public ActivityMaterial materialInfo(@PathVariable("activityId") Long activityId) throws Exception {
        return activityService.findMaterialByActivity(activityId);
    }



    /**
     * 获取勘查详情
     *
     * @param activityId
     * @return
     * @throws Exception
     */
    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.HDGL_XKHD_KCXQ)
    @RequestMapping("surveyInfo/{activityId}")
    public ModelAndView surveyInfo(@PathVariable("activityId") Long activityId) throws Exception {
        ModelAndView view = new ModelAndView("/activity/survey/surveyInfo");
        SurveyDto dto = new SurveyDto();
        Activity activity = activityService.findFullById(activityId);
        List<Survey> surveyList = surveyService.findByActivity(activityId);
        Map<Long, List<SurveyDetail>> map = Maps.newHashMap();
        for (Survey survey : surveyList) {
            List<SurveyDetail> temp = surveyService.findBySurveyId(survey);
            map.put(survey.getId(), temp);
        }
        dto.setSurveyList(surveyList);
        dto.setSurveyInfoMap(map);
        view.addObject("surveyDto", dto);
        activity.setTypeCode(ActivityUtil.getTypeByCode(activity.getTypeCode()));
        view.addObject("activity", activity);
        return view;
    }


    /**
     * 获取审批用户详情
     * @param activityId
     * @return
     * @throws Exception
     */
    @RequestMapping("approveUserInfo")
    @ResponseBody
    public List approveUserInfo(@RequestParam("activityId") Long activityId) throws Exception {
        return activityService.findActivityApproveUser(activityId);
    }

    @RequestMapping("export")
    @SysManageAuth(limitRole = DictConst.ROLE_APPROVE)
    public ModelAndView export(HttpServletRequest request) {
        User user = super.getCurrentUser(request);
        /* 额外增加因审批步骤跳过的同步任务 */
        int taskSize = 0;
        if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_redirectsl"))
                || GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_redirectsp"))) {
            List<SyncTaskDTO> activityTaskList = activityService.findNeedSyncTask(user);
            taskSize = activityTaskList.size();
        }

        List<Activity> activityList = activityService.findNeedSyncActivity(user);
        ModelAndView view = new ModelAndView("/sys/export");
        view.addObject("needSyncActivity",activityList.size()+taskSize);
        return view;
    }


    @RequestMapping("syncData")
    public @ResponseBody
    SearchDto<Activity> syncData(HttpServletRequest request) {
        User user = super.getCurrentUser(request);
        List<Activity> activityList = Lists.newArrayList();
        if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_redirectsl"))
                || GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_redirectsp"))) {
            List<SyncTaskDTO> activityTaskList = activityService.findNeedSyncTask(user);
            for (SyncTaskDTO dto:activityTaskList) {
                Activity activity = new Activity();
                activity.setFullName(dto.getActivityName());
                activity.setApproveStatus(dto.getApproveStatus());
                activity.setTypeCode("1");
                activityList.add(activity);
            }
        }
        List<Activity> activityList2 = activityService.findNeedSyncActivity(user);
        activityList.addAll(activityList2);
        long count = activityList.size();
        SearchDto<Activity> result = new SearchDto<Activity>();
        result.setData(activityList);
        result.setIDisplayStart(1);
        result.setIDisplayLength(100);
        result.setITotalRecords(count);
        result.setITotalDisplayRecords(count);
        return result;
    }

    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.XTPZ_SPJGDC)
    @RequestMapping("/exportactivity")
    @SysLog("导出已经审批的活动")
    public void exportActivity(HttpServletRequest request, HttpServletResponse response) throws Exception {
        StringBuilder sb=new StringBuilder();
        User user = super.getCurrentUser(request);
        List<Map<String, String>> reMapList = new ArrayList<>();

        /* 额外增加因审批步骤跳过的同步任务 */
        if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_redirectsl"))
                || GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_redirectsp"))) {
            List<SyncTaskDTO> activityTaskList = activityService.findNeedSyncTask(user);
            for (SyncTaskDTO syncTaskDTO : activityTaskList) {
                reMapList.add(workflowService.generateApproveFileMap(syncTaskDTO));
                sb.append(syncTaskDTO.getActivityName()).append(",");
            }
        }

        List<Activity> activityList = activityService.findNeedSyncActivity(user);
        for (Activity activity : activityList) {
            reMapList.add(workflowService.generateApproveFileMap(activity));
            sb.append(activity.getFullName()).append(",");
        }

        String result = JSON.toJSONString(reMapList);
        byte[] buf = DESUtil.encrypt(result.getBytes("UTF-8"), GlobalConst.KEY);

        File fileDir = Files.createTempDir();
        File file = new File(fileDir.getPath()+"/" + StringUtil.randomString(10) + ".bcx");
        Files.write(buf,file);

        // 记录操作日志
        OperateLogUtil.saveOperate(user,"结果导出","导出审批结果文件",sb.toString());

        ServletOutputStream out = response.getOutputStream();
        String downLoadName = new String("完成审批活动-未同步".getBytes("gbk"), "iso8859-1");
        response.addHeader("Content-Disposition", "attachment;filename=" + downLoadName + ".bcx");
        InputStream is = new FileInputStream(file);
        int read = 0;
        byte[] buffer = new byte[8192];
        while ((read = is.read(buffer)) != -1) {
            out.write(buffer, 0, read);
        }
        is.close();
        out.flush();
        out.close();
    }

    /**
     * 活动上报
     * @param request
     * @return
     * @throws Exception
     */
    @AuditLogAn(operateType = DictConst.OP_SAVE,menuName = MenuConst.HDGL_XKHD_HDSB)
    @RequestMapping("report/save")
    public @ResponseBody AjaxResult reportSave(HttpServletRequest request){
        Map<String, String> parameterMap = WebUtil.getParameterMap(request);
        String msg = activityService.reportSave(parameterMap);
        AjaxResult result = new AjaxResult();
        if (StringUtil.isEmpty(msg)) {
            result.setSuccessful(true);
        } else {
            result.setSuccessful(false);
            result.setMsg(msg);
        }
        return result;
    }

    /**
     * 政府报备列表
     * @return
     */
    @RequestMapping("/govreport/{activityId}")
    public ModelAndView govreport(@PathVariable("activityId") Long activityId){
        ModelAndView view = new ModelAndView("/activity/gov/sessionList");
        Activity active = activityService.findFullById(activityId);
        view.addObject("activity", active);
        return view;
    }
    /**
     * 报备列表
     * @return
     */
    @RequestMapping("/report/{activityId}")
    public ModelAndView report(@PathVariable("activityId") Long activityId){
        ModelAndView view = new ModelAndView("/activity/session/list");
        Activity active = activityService.findFullById(activityId);
        view.addObject("activity", active);
        return view;
    }

    @AuditLogAn(operateType = DictConst.OP_SAVE,menuName = MenuConst.HDGL_BAHD_TJHD)
    @RequestMapping("/submitaction/{activityId}")
    public @ResponseBody
    AjaxResult submitaction(@PathVariable("activityId") Long activityId, HttpServletRequest request) {
        User user = super.getCurrentUser(request);
        return activityService.submitaction(user,activityId);
    }
}
