package com.bcxin.sp.work.controller.activity.extend;

import com.bcxin.sp.config.annotation.AuditLogAn;
import com.bcxin.sp.config.annotation.OperLog;
import com.bcxin.sp.config.annotation.SysManageAuth;
import com.bcxin.sp.work.controller.sys.BaseController;
import com.bcxin.sp.work.entity.domain.activity.approve.ApproveRole;
import com.bcxin.sp.work.entity.domain.activity.expand.Workflow;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.AjaxResult;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.service.activity.approve.ApproveRoleService;
import com.bcxin.sp.work.service.activity.extend.WorkflowService;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.MenuConst;
import com.bcxin.sp.work.util.StringUtil;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 流程管理
 */
@Controller
@RequestMapping("/business/workflow")
public class WorkFlowController extends BaseController {

    @Resource
    private WorkflowService workflowService;

    @Resource
    private ApproveRoleService approveRoleService;

    @RequestMapping("/list")
    @SysManageAuth(limitRole = DictConst.ROLE_APPROVE)
    public ModelAndView list(HttpServletRequest request) {
        User cuser = super.getCurrentUser(request);
        ModelAndView view = new ModelAndView("/sys/workflow");
        List<ApproveRole> approveRoles = approveRoleService.findByRegion(cuser.getProvince(), cuser.getCity(), cuser.getArea());
        view.addObject("approveRoles", approveRoles);

        Integer maxseq = workflowService.findMaxSeq(cuser.getProvince(), cuser.getCity(), cuser.getArea());
        view.addObject("nextSeq", (maxseq == null ? 0 : maxseq) + 1);
        return view;
    }

    @AuditLogAn(operateType = DictConst.OP_SAVE,menuName = MenuConst.XTPZ_LCPZ_TJLC)
    @OperLog(menuName = "流程配置", operateType = "#id", operateTypeMethod = "transferWorkFlow", operateObj = "#fullName")
    @RequestMapping("createOrUpdate")
    public
    @ResponseBody
    AjaxResult createOrUpdate(Workflow flow, HttpServletRequest request) {
        User cuser = super.getCurrentUser(request);
        flow.setUpdater(cuser.getUsername());
        flow.setProvince(cuser.getProvince());
        flow.setCity(cuser.getCity());
        flow.setArea(cuser.getArea());
        if (StringUtil.isEmpty(cuser.getProvince())) {
            flow.setProvince(null);
        }
        workflowService.createOrUpdate(flow);
        AjaxResult result = new AjaxResult();
        result.setSuccessful(true);
        result.setObj(flow);
        return result;
    }

    @AuditLogAn(operateType = DictConst.OP_DELETE,menuName = MenuConst.XTPZ_LCPZ_SCLC)
    @RequestMapping("delete")
    public
    @ResponseBody
    AjaxResult delete(Workflow workflow) {
        AjaxResult result = new AjaxResult();
        Workflow msg = workflowService.delete(workflow);
        result.setSuccessful(true);
        return result;
    }

    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.XTPZ_LCPZ)
    @RequestMapping("findData")
    @ResponseBody
    public SearchDto<Workflow> findData(SearchDto<Workflow> dto, HttpServletRequest request) throws Exception {
        User cuser = super.getCurrentUser(request);

        String province = "";
        String city = "";
        String area = "";
        String areaCode = request.getParameter("areaCode");
        if (!StringUtils.isEmpty(areaCode)) {
            if (areaCode.endsWith("0000")) {//省
                province = areaCode;
            } else if (areaCode.endsWith("00")) {//市
                province = areaCode.substring(0, 2) + "0000";
                city = areaCode;
            } else {//区
                province = areaCode.substring(0, 2) + "0000";
                city = areaCode.substring(0, 4) + "00";
                area = areaCode;
            }
        } else {
            province = cuser.getProvince();
            city = cuser.getCity();
            area = cuser.getArea();
        }

        List<Workflow> flowList = workflowService.findActive(province, city, area);
        long count = workflowService.findCountOfWork(province, city, area);
        SearchDto<Workflow> result = new SearchDto<Workflow>();
        result.setData(flowList);
        result.setIDisplayStart(dto.getIDisplayStart());
        result.setIDisplayLength(dto.getIDisplayLength());
        result.setITotalRecords(count);
        result.setITotalDisplayRecords(count);
        return result;
    }

    @RequestMapping(value = "getWorkflow/{seq}/{activityId}")
    public
    @ResponseBody
    Workflow getWorkflow(@PathVariable("seq") int seq
            , @PathVariable("activityId") Long activityId) {
        return workflowService.findWorkFlow(seq, activityId);
    }


    @RequestMapping("/workflowModal")
    public ModelAndView workflowModal(HttpServletRequest request) {
        ModelAndView view = new ModelAndView("/sys/dialog/workflowModal");
        view.addObject("areaCode", request.getParameter("areaCode"));
        return view;
    }
}
