package com.bcxin.sp.work.controller.activity.history;

import com.bcxin.sp.config.annotation.AuditLogAn;
import com.bcxin.sp.config.annotation.OperLog;
import com.bcxin.sp.work.controller.sys.BaseController;
import com.bcxin.sp.work.entity.PageResult;
import com.bcxin.sp.work.entity.domain.activity.base.Activity;
import com.bcxin.sp.work.entity.domain.activity.history.ActivityHistory;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.ActivityHistoryDTO;
import com.bcxin.sp.work.entity.dto.AjaxResult;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.service.activity.history.ActivityHistoryService;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.MenuConst;
import com.bcxin.sp.work.util.StringUtil;
import com.bcxin.sp.work.util.WebUtil;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * 历史数据录入
 */
@Controller
@RequestMapping("/business/activity/history")
public class ActivityHistoryController extends BaseController {

    @Resource
    private ActivityHistoryService activityHistoryService;

    @RequestMapping("/list")
    public ModelAndView listInit(HttpServletRequest request) {
        ModelAndView view = new ModelAndView("/activity/history/approve/list");
        //隐藏左边菜单树标记
        view.addObject("hiddenLeft",request.getParameter("hiddenLeft"));
        //市级编码
        view.addObject("city",request.getParameter("city"));
        //区县级编码
        view.addObject("area",request.getParameter("area"));
        //数据源
        view.addObject("dataSource",request.getParameter("dataSource"));
        //活动开始起始日期
        view.addObject("startDate",request.getParameter("startDate"));
        //活动开始截至日期
        view.addObject("endDate",request.getParameter("endDate"));

        return view;
    }

    @RequestMapping("/approveList")
    public ModelAndView approveListInit(HttpServletRequest request) {
        ModelAndView view = new ModelAndView("/activity/history/approve/list");
        //隐藏左边菜单树标记
        view.addObject("hiddenLeft",request.getParameter("hiddenLeft"));
        //市级编码
        view.addObject("city",request.getParameter("city"));
        //区县级编码
        view.addObject("area",request.getParameter("area"));
        //数据源
        view.addObject("dataSource",request.getParameter("dataSource"));
        //活动开始起始日期
        view.addObject("startDate",request.getParameter("startDate"));
        //活动开始截至日期
        view.addObject("endDate",request.getParameter("endDate"));

        return view;
    }

    @RequestMapping("/reportList")
    public ModelAndView recordListInit(HttpServletRequest request) {
        ModelAndView view = new ModelAndView("/activity/history/report/list");
        //隐藏左边菜单树标记
        view.addObject("hiddenLeft",request.getParameter("hiddenLeft"));
        //市级编码
        view.addObject("city",request.getParameter("city"));
        //区县级编码
        view.addObject("area",request.getParameter("area"));
        //数据源
        view.addObject("dataSource",request.getParameter("dataSource"));
        //活动开始起始日期
        view.addObject("startDate",request.getParameter("startDate"));
        //活动开始截至日期
        view.addObject("endDate",request.getParameter("endDate"));

        return view;
    }

    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.HDGL_GABA)
    @RequestMapping("findData")
    @ResponseBody
    public SearchDto<ActivityHistoryDTO> findData(HttpServletRequest request, SearchDto<ActivityHistory> dto) throws Exception {
        /* 查出所有的记录 */
        PageResult pageResult = activityHistoryService.findActivity(dto);
        List<ActivityHistoryDTO> resultList = (List<ActivityHistoryDTO>)pageResult.getRows();
        SearchDto<ActivityHistoryDTO> result = new SearchDto<ActivityHistoryDTO>();
        result.setData(resultList);
        result.setIDisplayStart(dto.getIDisplayStart());
        result.setIDisplayLength(dto.getIDisplayLength());
        result.setITotalRecords(pageResult.getTotal());
        result.setITotalDisplayRecords(pageResult.getTotal());
        return result;
    }

    /**
     * 现场报备列表
     * @return
     */
    @RequestMapping("/approve/{activityId}")
    public ModelAndView approve(@PathVariable("activityId") Long activityId){
        ModelAndView view = new ModelAndView("/activity/history/approve/sessionList");
        view.addObject("activity", activityHistoryService.findFullById(activityId));
        return view;
    }
    /**
     * 报备列表
     * @return
     */
    @RequestMapping("/report/{activityId}")
    public ModelAndView report(@PathVariable("activityId") Long activityId){
        ModelAndView view = new ModelAndView("/activity/history/report/sessionList");
        view.addObject("activity", activityHistoryService.findFullById(activityId));
        return view;
    }

    @RequestMapping("/enterApprove")
    public ModelAndView enterApproveInit(HttpServletRequest request) {
        return new ModelAndView("/activity/history/approve/enterNew");
    }
    @RequestMapping("/enterApproveTab")
    public ModelAndView enterApproveTab(HttpServletRequest request) {
        return new ModelAndView("/activity/history/approve/enterTab");
    }

    @RequestMapping("/enterReport")
    public ModelAndView enterReportInit(HttpServletRequest request) {
        return new ModelAndView("/activity/history/report/enterNew");
    }
    @RequestMapping("/enterReportTab")
    public ModelAndView enterReportTab(HttpServletRequest request) {
        return new ModelAndView("/activity/history/report/enterTab");
    }


    @AuditLogAn(operateType = DictConst.OP_SAVE,menuName = MenuConst.HDGL_GABA_SJLR)
    @OperLog(menuName = "公安备案",operateType = "数据录入",operateObj = "#fullName")
    @RequestMapping("enter/save")
    public @ResponseBody AjaxResult enter(MultipartHttpServletRequest request) throws Exception {
        User user = getCurrentUser(request);
        MultipartFile pgbg = request.getFile("pgbg");
        MultipartFile bbcl = request.getFile("bbcl");
        Map<String, String> parameterMap = WebUtil.getParameterMap(request);
        String msg = activityHistoryService.save(user,parameterMap,pgbg,bbcl);
        AjaxResult result = new AjaxResult();
        if (StringUtil.isEmpty(msg)) {
            result.setSuccessful(true);
        } else {
            result.setSuccessful(false);
            result.setMsg(msg);
        }
        return result;
    }
    @AuditLogAn(operateType = DictConst.OP_DELETE,menuName = MenuConst.HDGL_GABA_SCHD)
    @OperLog(menuName = "备案记录删除",operateType = "数据录入",operateObj = "#id")
    @RequestMapping("delete/{Id}")
    public @ResponseBody AjaxResult delete(@PathVariable("Id") Long Id,HttpServletRequest request) throws Exception {
        User user = getCurrentUser(request);
        AjaxResult result = new AjaxResult();
        ActivityHistory activityHistory = activityHistoryService.findById(Id);
        if(activityHistory.getReportTime()!=null){
            result.setSuccessful(false);
            result.setMsg("操作失败:活动已提交上报");
            return result;
        }
        if (activityHistoryService.deleteByActivityHistoryId(Id)) {
            result.setSuccessful(true);
        } else {
            result.setSuccessful(false);
            result.setMsg("操作失败");
        }
        return result;
    }

    @AuditLogAn(operateType = DictConst.OP_SAVE,menuName = MenuConst.HDGL_BAHD_TJHD)
    @RequestMapping("/submitaction/{activityId}")
    public @ResponseBody
    AjaxResult submitaction(@PathVariable("activityId") Long activityId, HttpServletRequest request) {
        User user = super.getCurrentUser(request);
        return activityHistoryService.submitaction(user,activityId);
    }

}
