package com.bcxin.sp.work.controller.activity.plan;

import com.bcxin.sp.config.annotation.AuditLogAn;
import com.bcxin.sp.work.controller.sys.BaseController;
import com.bcxin.sp.work.entity.PageResult;
import com.bcxin.sp.work.entity.domain.activity.base.Activity;
import com.bcxin.sp.work.entity.domain.activity.plan.ActivityPlan;
import com.bcxin.sp.work.entity.domain.sys.ConfigFunction;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.AjaxResult;
import com.bcxin.sp.work.entity.dto.ApproveUtil;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.service.activity.base.ActivityService;
import com.bcxin.sp.work.service.activity.plan.ActivityPlanService;
import com.bcxin.sp.work.service.user.UserService;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.MenuConst;
import com.bcxin.sp.work.util.sys.ConfigUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <b> 活动预案管理 </b>
 * @author ZXF
 * @create 2020/11/04 0004 13:54
 * @version
 * @注意事项 </b>
 */
@Controller
@RequestMapping("/business/plan")
@Slf4j
public class ActivityPlanController extends BaseController {

    @Resource
    private ActivityService activityService;

    @Resource
    private UserService userService;

    @Resource
    private ActivityPlanService activityPlanService;


    /**
     * 审批列表
     *
     * @param request
     * @return
     */
    @RequestMapping("list")
    public ModelAndView list(HttpServletRequest request) {
        ModelAndView index = new ModelAndView("/activity/plan/list");
        User user = super.getCurrentUser(request);
        index.addObject("communicateMode", ConfigUtil.getValue("pss-communicate-mode"));
        index.addObject("crole",user.getRole() );
        index.addObject("approveType", ApproveUtil.getApproveType());
        index.addObject("status",request.getParameter("status"));
        //隐藏左边菜单树标记
        index.addObject("hiddenLeft",request.getParameter("hiddenLeft"));
        //活动类型
        index.addObject("typeCode",request.getParameter("typeCode"));
        //市级编码
        index.addObject("city",request.getParameter("city"));
        //区县级编码
        index.addObject("area",request.getParameter("area"));
        //数据源
        index.addObject("dataSource",request.getParameter("dataSource"));
        //活动开始起始日期
        index.addObject("startDate",request.getParameter("startDate"));
        //活动开始截至日期
        index.addObject("endDate",request.getParameter("endDate"));
        /* 审批状态 */
        index.addObject("approveStatus",request.getParameter("approveStatus"));
        /* 审批状态汇总 */
        index.addObject("approveStatusSummary",request.getParameter("approveStatusSummary"));
        /* 时间区间 */
        /*if (StringUtil.isNotEmpty(request.getParameter("dateRange"))) {
            List<String> dateList = StrSpliter.split(request.getParameter("dateRange")," - ",true,true);
            String beginDate = dateList.get(0);
            String endDate = dateList.get(1);
            //活动开始起始日期
            index.addObject("addTimeStartDate",beginDate);
            //活动开始截至日期
            index.addObject("addTimeEndDate",endDate);
        }*/
        index.addObject("user",user);
        index.addObject("switchGradeRank",ConfigUtil.getValue("switch_gradeRank"));
        ConfigFunction configFunction = ConfigUtil.getFunction(user.getProvince());
        index.addObject("switchApprove",configFunction.getSwitchApprove());

        /* 判断当前用户是否是第一级审批人 */
        String isFirstApproveUser = DictConst.N;
        if (userService.isFirstApproveLevel(user)) {
            isFirstApproveUser = DictConst.Y;
        }
        index.addObject("isFirstApproveUser",isFirstApproveUser);
        index.addObject("myEnvi",ConfigUtil.getValue("envi"));
        index.addObject("switch_zjzw20",ConfigUtil.getValue("switch_zjzw20"));
        return index;
    }

    @RequestMapping("findData")
    @ResponseBody
    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.HDGL_XKHD)
    public SearchDto<Activity> findData(HttpServletRequest request,SearchDto<Activity> dto) throws Exception {
        User user = getCurrentUser(request);
        dto.setGov(false);
        /* 查出所有的记录 */
        PageResult pageResult = activityService.findApproveByUser(dto);
        List<Activity> resultList = activityService.authApprove(user,(List<Activity>)pageResult.getRows());
        SearchDto<Activity> result = new SearchDto<Activity>();
        result.setData(resultList);
        result.setIDisplayStart(dto.getIDisplayStart());
        result.setIDisplayLength(dto.getIDisplayLength());
        result.setITotalRecords(pageResult.getTotal());
        result.setITotalDisplayRecords(pageResult.getTotal());

        return result;
    }

    /**
     * <b> 预案列表 </b>
     * @author ZXF
     * @create 2020/11/05 0005 9:29
     * @version
     * @注意事项 </b>
     */
    @RequestMapping("planList/{activityId}")
    public ModelAndView planList(@PathVariable("activityId") String activityId) {
        ModelAndView index = new ModelAndView("/activity/plan/planList");
        index.addObject("activityId",activityId);
        return index;
    }

    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.HDGL_GABA)
    @RequestMapping("findPlanListData")
    @ResponseBody
    public SearchDto<ActivityPlan> findPlanListData(SearchDto<ActivityPlan> dto) throws Exception {
        /* 查出所有的记录 */
        PageResult pageResult = activityPlanService.findPlanListData(dto);
        List<ActivityPlan> resultList = (List<ActivityPlan>)pageResult.getRows();
        SearchDto<ActivityPlan> result = new SearchDto<ActivityPlan>();
        result.setData(resultList);
        result.setIDisplayStart(dto.getIDisplayStart());
        result.setIDisplayLength(dto.getIDisplayLength());
        result.setITotalRecords(pageResult.getTotal());
        result.setITotalDisplayRecords(pageResult.getTotal());
        return result;
    }

    @RequestMapping("planInfo/{id}")
    @ResponseBody
    public ActivityPlan planInfo(@PathVariable("id") Long id){
        return activityPlanService.findById(id);
    }

    @AuditLogAn(operateType = DictConst.OP_SAVE,menuName = MenuConst.HDGL_BAHD_TJHD)
    @RequestMapping("createOrUpdate")
    public @ResponseBody
    AjaxResult createOrUpdate(ActivityPlan plan, HttpServletRequest request) {
        User user = super.getCurrentUser(request);
        plan.setUpdater(user.getUsername());
        activityPlanService.createOrUpdate(plan);
        AjaxResult result = new AjaxResult();
        result.setSuccessful(true);
        result.setObj(plan);
        return result;
    }

    @AuditLogAn(operateType = DictConst.OP_DELETE,menuName = MenuConst.HDGL_BAHD_SCHD)
    @RequestMapping("delete/{id}")
    public @ResponseBody AjaxResult delete(@PathVariable("id") long id) {
        AjaxResult result = new AjaxResult();
        ActivityPlan plan = activityPlanService.findById(id);
        if (plan!=null) {
            activityPlanService.delete(id);
            result.setSuccessful(true);
        } else {
            result.setSuccessful(false);
            result.setMsg("未找到预案记录.");
        }
        return result;
    }
}
