package com.bcxin.sp.work.controller.log;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.bcxin.sp.work.controller.sys.BaseController;
import com.bcxin.sp.work.entity.PageResult;
import com.bcxin.sp.work.entity.domain.log.AuditLog;
import com.bcxin.sp.work.entity.domain.log.OperateLog;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.OperateLogSearchDTO;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.service.log.AuditLogService;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.stream.Collectors;

@Controller
@RequestMapping("/audit/log")
@Slf4j
public class AuditLogController extends BaseController {

    @Autowired
    AuditLogService auditLogService;

    @RequestMapping("list")
    public ModelAndView operateList(HttpServletRequest request) throws Exception {
        ModelAndView view = new ModelAndView("/log/auditList");
        return view;
    }

    /**
     * 宁夏用户操作日志
     * @author wangjianjun
     * @date 2021/1/29
     */
    @RequestMapping("nx/list")
    public ModelAndView nxOperateList(HttpServletRequest request) throws Exception {
        ModelAndView view = new ModelAndView("/log/nxAuditList");
        return view;
    }


    @RequestMapping("findData")
    @ResponseBody
    public SearchDto<AuditLog> findData(OperateLogSearchDTO dto, HttpServletRequest request) throws Exception {
        User user = super.getCurrentUser(request);
        if(user.getRegionLevel() == 1) {
            dto.setProvince(user.getProvince());
        } else if(user.getRegionLevel() == 2) {
            dto.setProvince(user.getProvince());
            dto.setCity(user.getCity());
        } else if(user.getRegionLevel() == 3) {
            dto.setProvince(user.getProvince());
            dto.setCity(user.getCity());
            dto.setArea(user.getArea());
        }
        if(ObjectUtil.equal(user.getRole(),DictConst.ROLE_SUPERADMIN)
            || ObjectUtil.equal(user.getRole(),DictConst.ROLE_ADMIN)
                || ObjectUtil.equal(user.getRole(),DictConst.ROLE_ADMIN_APPROVE)) {
            // 如果是管理员，可以看到所有
        }else {
            dto.setUserId(user.getId());
        }
        // 设置时间
        if(StringUtil.isNotEmpty(dto.getStartDate())){
            dto.setStartDate(dto.getStartDate()+" 00:00:00");
        }
        if(StringUtil.isNotEmpty(dto.getEndDate())){
            dto.setEndDate(dto.getEndDate()+" 23:59:59");
        }

        List<AuditLog> list = auditLogService.searchLog(dto);
        list.stream().filter(s -> {//将202012171619145格式化为2020-12-17 16:19:45
                    s.setOperate_Time(DateUtil.format(DateUtil.parse(s.getOperate_Time()), DatePattern.NORM_DATETIME_PATTERN));
                    return true;
                }
        ).collect(Collectors.toList());
        long count = auditLogService.searchLogCount(dto);
        PageResult pageResult=new PageResult(list,count);
        SearchDto<AuditLog> result = new SearchDto<AuditLog>();
        result.setIDisplayStart(dto.getIDisplayStart());
        result.setIDisplayLength(dto.getIDisplayLength());
        result.setITotalRecords(pageResult.getTotal());
        result.setITotalDisplayRecords(pageResult.getTotal());
        result.setData(list);
        return result;
    }
}
