package com.bcxin.sp.work.controller.log;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.bcxin.sp.work.controller.sys.BaseController;
import com.bcxin.sp.work.entity.domain.log.FileSyncLog;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.service.log.LogService;
import com.bcxin.sp.work.util.Const.GlobalConst;
import com.bcxin.sp.work.util.DESUtil;
import com.bcxin.sp.work.util.StringUtil;
import com.google.common.collect.Maps;
import com.google.common.io.Files;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 日志
 */
@Controller
@RequestMapping("/business/log")
@Slf4j
public class LogController extends BaseController {

    @Resource
    private LogService logService;


    @RequestMapping("list")
    public ModelAndView export(HttpServletRequest request) throws Exception {
        ModelAndView view = new ModelAndView("/log/fileSyncLog");
        List<FileSyncLog> list = logService.findNeedSyncRecord();
        view.addObject("needSyncFileLog",list.size());
        return view;
    }


    @RequestMapping("findSyncFileData")
    public @ResponseBody SearchDto<FileSyncLog> findSyncFileData(SearchDto<FileSyncLog> dto) throws Exception {
        List<FileSyncLog> list = logService.findByPage(dto);
        long count = logService.findCount(dto);
        SearchDto<FileSyncLog> result = new SearchDto<FileSyncLog>();
        result.setData(list);
        result.setIDisplayStart(1);
        result.setIDisplayLength(20);
        result.setITotalRecords(count);
        result.setITotalDisplayRecords(count);
        return result;
    }


    /**
     * 导出文件，同步到外网进行比对
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping("/exportFileLog")
    public void exportFileLog(HttpServletRequest request, HttpServletResponse response) throws Exception {
        List<Map<String, String>> reMapList = new ArrayList<>();

        List<FileSyncLog> list = logService.findNeedSyncRecord();
        for (FileSyncLog log : list) {
            Map<String,String> map = Maps.newHashMap();
            map.put("fileName",log.getFileName());
            map.put("readTime", DateUtil.format(log.getAddTime(), DatePattern.NORM_DATETIME_PATTERN) );
            reMapList.add(map);
        }

        String result = JSON.toJSONString(reMapList);
        byte[] buf = DESUtil.encrypt(result.getBytes("UTF-8"), GlobalConst.KEY);

        File fileDir = Files.createTempDir();
        File file = new File(fileDir.getPath()+"/" + StringUtil.randomString(10) + ".bcxlog");
        Files.write(buf,file);

        ServletOutputStream out = response.getOutputStream();
        String downLoadName = new String("审批系统已读文件-未同步".getBytes("gbk"), "iso8859-1");
        response.addHeader("Content-Disposition", "attachment;filename=" + downLoadName + ".bcxlog");
        InputStream is = new FileInputStream(file);
        int read = 0;
        byte[] buffer = new byte[8192];
        while ((read = is.read(buffer)) != -1) {
            out.write(buffer, 0, read);
        }
        is.close();
        out.flush();
        out.close();
    }

}
