package com.bcxin.sp.work.controller.sys;

import com.bcxin.sp.config.annotation.AuditLogAn;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.*;
import com.bcxin.sp.work.entity.enums.ActivityTypeEnum;
import com.bcxin.sp.work.service.statistic.StatisticsService;
import com.bcxin.sp.work.util.BcxinDateUtil;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.MenuConst;
import com.bcxin.sp.work.util.SpringContextUtil;
import com.bcxin.sp.work.util.StringUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.util.*;

@Controller
@RequestMapping("/business/analysis")
public class AnalysisController extends BaseController {

    @Autowired
    private StatisticsService statisticsService;


    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.DTTS)
    @RequestMapping("/activityMap")
    public ModelAndView activityMap(HttpServletRequest request) throws ParseException {
        ModelAndView view = new ModelAndView("/statistic/activityMap");
        view.addObject("profile",SpringContextUtil.getActiveProfile());
        User user = super.getCurrentUser(request);
        if (user == null) {
            view = new ModelAndView("/auth/login");
            return view;
        }
        return view;
    }

    /**
     * 获取活动信息和接入省市区
     *
     * @param category
     * @param year
     * @param level
     * @param address
     * @param request
     * @return
     */
    @RequestMapping("/actdata0")
    @ResponseBody
    public AnalysisDto actdata0(String category, String year, String level, String address, HttpServletRequest request) {
        String sql_where = "";

        if (category == null) {
            category = "ALL";
        }

        if (!category.equals("ALL")) {
            sql_where = sql_where + " and typeCode = '" + category + "' ";
        }

        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and startDate <= '" + year + "-12-31' ";
        }

        if (StringUtil.isNotEmpty(level)) {
            if ("0".equals(level)) {
            } else if ("1".equals(level)) {
                sql_where+=" and province = '" + address + "' ";
            } else if ("2".equals(level)) {
                sql_where+=" and city = '" + address + "' ";
            } else if ("3".equals(level)) {
                sql_where+=" and area = '" + address + "' ";
            }
        }

        StringBuilder builder = new StringBuilder();
        builder.append(" select ");

        builder.append(" count(DISTINCT province) as provinceTotal ");//活动省份
        builder.append(" ,count(DISTINCT city) as cityTotal ");//活动地市
        builder.append(" ,count(DISTINCT area) as areaTotal ");//活动区县

        builder.append(" ,count(id) as reportTotal ");//活动总量
        builder.append(" ,sum(number) as peopleTotal ");//活动总人数
        builder.append(" ,sum(case approveStatus when 'NO' then 1 when 'REJECT' then 1 else 0 end) as rejectTotal ");//审批不通过活动数
        builder.append(" ,sum(case when SYSDATE() BETWEEN startDate and endDate then 1 else 0 end) as todayTotal ");//今日进行中活动总数
        builder.append(" ,sum(case when SYSDATE() < startDate and approveStatus='YES' then 1 else 0 end) as duringTotal ");//承办中活动总数
        builder.append(" from sp_activity where 1=1 ");
        builder.append(sql_where);
        //活动统计
        AnalysisDto analysisDto = statisticsService.findBySql(builder.toString());
        //场地统计
        StringBuffer sqlStringBuffer = new StringBuffer("SELECT address as pname FROM sp_activity where 1=1 ");
        sqlStringBuffer.append(sql_where);
        List<AnalysisDto> infoList = statisticsService.findListBySql(sqlStringBuffer.toString());
        Set<String> stringSet = Sets.newHashSet();
        for (AnalysisDto dto : infoList) {
            String[] splits = dto.getPname().split(",");
            for (String name : splits) {
                if (StringUtil.isEmpty(name)) {
                    continue;
                }
                stringSet.add(name);
            }
        }
        analysisDto.setAddressTotal(stringSet.size());
        return analysisDto;
    }


    @RequestMapping("/actdata")
    @ResponseBody
    public List<EchartMapData> actdata(String category, String year, String level, String address) {
        String sql_where = "";

        if (category == null) {
            category = "ALL";
        }

        if (!category.equals("ALL")) {
            sql_where = sql_where + " and typeCode = '" + category + "' ";
        }

        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and startDate <= '" + year + "-12-31' ";
        }

        String sql = "";

        if (StringUtil.isNotEmpty(level)) {
            if ("0".equals(level)) {
                sql = "SELECT province as regionid, (SELECT r.fullName from base_region r where province=r.id) pname,count(id) as number FROM sp_activity where 1=1 ";
            } else if ("1".equals(level)) {
                sql = "SELECT city as regionid,(SELECT r.fullName from base_region r where city=r.id) pname,count(id) as number FROM sp_activity where province = '" + address + "' ";
            } else if ("2".equals(level)) {
                sql = "SELECT area as regionid,(SELECT r.fullName from base_region r where area=r.id) pname,count(id) as number FROM sp_activity where city ='" + address + "' ";
            } else if ("3".equals(level)) {
                sql = "SELECT area as regionid,(SELECT r.fullName from base_region r where area=r.id) pname,count(id) as number FROM sp_activity where area ='" + address + "' ";
            }
        }
        sql = sql + sql_where + " group by regionid";

        List<AnalysisDto> dtoList = statisticsService.findListBySql(sql);

        List<EchartMapData> list = Lists.newArrayList();
        for (AnalysisDto dto : dtoList) {
            EchartMapData emd = new EchartMapData();
            emd.setName(dto.getPname());
            emd.setValue(dto.getNumber());
            emd.setData1(dto.getRegionid());
            list.add(emd);
        }
        return list;
    }

    /**
     * 活动类型比例图
     *
     * @param category
     * @param year
     * @param level
     * @param address
     * @return
     */
    @RequestMapping("/actdata2")
    @ResponseBody
    public List<EchartMapData> actdata2(String category, String year, String level, String address) {
        String sql_where = " where 1=1 ";

        if (!category.equals("ALL")) {
            sql_where = sql_where + " and typeCode = '" + category + "' ";
        }

        if (StringUtil.isNotEmpty(level)) {
            if ("0".equals(level)) {
            } else if ("1".equals(level)) {
                sql_where = sql_where + " and province = '" + address + "' ";
            } else if ("2".equals(level)) {
                sql_where = sql_where + " and city = '" + address + "' ";
            } else if ("3".equals(level)) {
                sql_where = sql_where + " and area = '" + address + "' ";
            }
        }
        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and startDate <= '" + year + "-12-31' ";
        }

        String sql = "SELECT typeCode as pname,count(id) number FROM sp_activity";
        sql = sql + sql_where + " group by pname";

        List<AnalysisDto> dtoList = statisticsService.findListBySql(sql);

        List<EchartMapData> list = Lists.newArrayList();

        EchartMapData emd;

        for (AnalysisDto dto : dtoList) {
            emd = new EchartMapData();
            if (dto.getPname().equals("100001")) {
                emd.setName("体育赛事");
            }
            if (dto.getPname().equals("100002")){
                emd.setName("文艺演出");
            }
            if (dto.getPname().equals("100003")){
                emd.setName("展览展销");
            }
            if (dto.getPname().equals("100004")){
                emd.setName("游园灯会");
            }
            if (dto.getPname().equals("100005")){
                emd.setName("招聘会");
            }
            if (dto.getPname().equals("100006")){
                emd.setName("公益慈善");
            }
            if (dto.getPname().equals("100007")){
                emd.setName("其他");
            }

            emd.setValue(dto.getNumber());
            list.add(emd);
        }

        return list;

    }

    /**
     * 观众数量统计图
     *
     * @param category
     * @param year
     * @param level
     * @param address
     * @return
     */
    @RequestMapping("/actdata3")
    @ResponseBody
    public EchartData actdata3(String category, String year, String level, String address) {
        List<String> list = Lists.newArrayList();
        if (!category.equals("ALL")) {
            list.add(category);
        } else {
            list.add("100002");
            list.add("100001");
            list.add("100003");
            list.add("100004");
            list.add("100006");
            list.add("100005");
            list.add("100007");
        }

        List<String> legend = Lists.newArrayList("'文艺演出'", "'体育赛事'", "'展览展销'", "'游园灯会'", "'公益慈善'", "'招聘会'", "'其他'");//数据分组
        List<String> cate = Lists.newArrayList("'一月'", "'二月'", "'三月'", "'四月'", "'五月'", "'六月'", "'七月'", "'八月'", "'九月'", "'十月'", "'十一月'", "'十二月'");//横坐标
        List<Series> series = new ArrayList<Series>();//纵坐标

        List<Integer> listData;

        for (int i = 0; i < list.size(); i++) {
            listData = new ArrayList<Integer>(Arrays.asList(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0));
            List<AnalysisDto> infoList = getData(list.get(i), year, level, address);
            for (AnalysisDto dto : infoList) {
                String m = dto.getPname();
                int index = Integer.parseInt(m) - 1;
                listData.set(index, dto.getNumber());
            }

            series.add(new Series(legend.get(i), "line", listData));
        }

        EchartData data = new EchartData(legend, cate, series);
        return data;
    }

    private List<AnalysisDto> getData(String category, String year, String level, String address) {
        String sql_where = " where 1=1 ";

        if (!category.equals("ALL")) {
            sql_where = sql_where + " and typeCode = '" + category + "' ";
        }

        if (StringUtil.isNotEmpty(level)) {
            if ("0".equals(level)) {
            } else if ("1".equals(level)) {
                sql_where = sql_where + " and province = '" + address + "' ";
            } else if ("2".equals(level)) {
                sql_where = sql_where + " and city = '" + address + "' ";
            } else if ("3".equals(level)) {
                sql_where = sql_where + " and area = '" + address + "' ";
            }
        }

        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and startDate <= '" + year + "-12-31' ";
        }

        String sql = "SELECT month(startDate) as pname,sum(number) number FROM sp_activity ";

        sql = sql + sql_where + "  group by pname ";

        List<AnalysisDto> infoList = statisticsService.findListBySql(sql);

        return infoList;
    }

    /**
     * 场馆活动数统计图
     *
     * @param category
     * @param year
     * @param level
     * @param address
     * @return
     */
    @RequestMapping("/actdata4")
    @ResponseBody
    public List<EchartMapData> actdata4(String category, String year, String level, String address) {
        String sql_where = " where 1=1 ";

        if (!category.equals("ALL")) {
            sql_where = sql_where + " and typeCode = '" + category + "' ";
        }

        if (StringUtil.isNotEmpty(level)) {
            if ("0".equals(level)) {
            } else if ("1".equals(level)) {
                sql_where = sql_where + " and province = '" + address + "' ";
            } else if ("2".equals(level)) {
                sql_where = sql_where + " and city = '" + address + "' ";
            } else if ("3".equals(level)) {
                sql_where = sql_where + " and area = '" + address + "' ";
            }
        }

        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and startDate <= '" + year + "-12-31' ";
        }

        String sql = "SELECT address as pname FROM sp_activity ";
        sql = sql + sql_where;

        List<AnalysisDto> infoList = statisticsService.findListBySql(sql);
        Map<String, Integer> map = Maps.newHashMap();
        for (AnalysisDto dto : infoList) {
            String[] splits = dto.getPname().split(",");
            for (String name : splits) {
                if (StringUtil.isEmpty(name)) {
                    continue;
                }
                if (map.get(name) == null) {
                    map.put(name, 1);
                } else {
                    int value = map.get(name);
                    map.put(name, value + 1);
                }
            }
        }

        List<Map.Entry<String, Integer>> list =
                new ArrayList<Map.Entry<String, Integer>>(map.entrySet());

        Collections.sort(list, new Comparator<Map.Entry<String, Integer>>() { //map的value排序
            public int compare(Map.Entry<String, Integer> o1,
                               Map.Entry<String, Integer> o2) {
                return (o2.getValue() - o1.getValue());
            }
        });


        List<EchartMapData> echartMapDatalist = new ArrayList<EchartMapData>();
        EchartMapData emd;

        int length = list.size() > 6 ? 6 : list.size();
        for (int i = 0; i < length; i++) {
            Map.Entry<String, Integer> row = list.get(i);
            emd = new EchartMapData();
            emd.setName(row.getKey());
            emd.setValue(row.getValue());
            echartMapDatalist.add(emd);
        }

        return echartMapDatalist;
    }

    /**
     * 风评机构活动数统计图
     *
     * @param category
     * @param year
     * @param level
     * @param address
     * @return
     */
    @RequestMapping("/orgDataTotal")
    @ResponseBody
    public EchartData orgDataTotal(String category, String year, String level, String address) {
        ActivityTypeEnum[] types = ActivityTypeEnum.values();

        List<String> legend = Lists.newArrayList("'体育赛事'", "'文艺演出'", "'展览展销'", "'游园灯会'", "'招聘会'", "'公益慈善'", "'其他'");//数据分组
        List<String> cate = Lists.newArrayList();//横坐标

        String sqlString = "select assessmentorg as pname,count(id) total,"
                + " sum(case typeCode when '100002' then 1 else 0 end) li,"
                + " sum(case typeCode when '100001' then 1 else 0 end) sp,"
                + " sum(case typeCode when '100003' then 1 else 0 end) ex,"
                + " sum(case typeCode when '100004' then 1 else 0 end) ga,"
                + " sum(case typeCode when '100005' then 1 else 0 end) re,"
                + " sum(case typeCode when '100006' then 1 else 0 end) ch,"
                + " sum(case typeCode when '100007' then 1 else 0 end) ot from sp_activity ";

        String sql_where = " where 1=1 ";

        if (!category.equals("ALL")) {
            sql_where = sql_where + " and typeCode = '" + category + "' ";
        }

        if (StringUtil.isNotEmpty(level)) {
            if ("0".equals(level)) {
            } else if ("1".equals(level)) {
                sql_where = sql_where + " and province = '" + address + "' ";
            } else if ("2".equals(level)) {
                sql_where = sql_where + " and city = '" + address + "' ";
            } else if ("3".equals(level)) {
                sql_where = sql_where + " and area = '" + address + "' ";
            }
        }

        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and startDate <= '" + year + "-12-31' ";
        }

        sqlString = sqlString + sql_where + "  group by pname order by total desc";
        List<AnalysisDto> list = statisticsService.findListBySql(sqlString);

        Map<String, Map<String, String>> orgmaplist = Maps.newHashMap();
        for (int i = 0; i < (list.size() > 3 ? 3 : list.size()); i++) {
            AnalysisDto dto = list.get(i);
            Map<String, String> orgmap = Maps.newHashMap();
            orgmap.put("name", dto.getPname());
            orgmap.put("total", dto.getTotal());
            orgmap.put("li", dto.getLi());
            orgmap.put("sp", dto.getSp());
            orgmap.put("ex", dto.getEx());
            orgmap.put("ga", dto.getGa());
            orgmap.put("re", dto.getRe());
            orgmap.put("ch", dto.getCh());
            orgmap.put("ot", dto.getOt());
            orgmaplist.put(orgmap.get("name"), orgmap);
            cate.add(dto.getPname());
        }
        List<Series> series = new ArrayList<Series>();//纵坐标

        for (int i = 0; i < types.length; i++) {
            List<Integer> listData = new ArrayList<Integer>(Arrays.asList(0, 0, 0));
            int count = 0;
            if (i == 0) {
                for (String key : orgmaplist.keySet()) {
                    String value_return = orgmaplist.get(key).get("sp");
                    listData.set(count, Integer.parseInt(value_return));
                    count++;
                }
            } else if (i == 1) {
                for (String key : orgmaplist.keySet()) {
                    String value_return = orgmaplist.get(key).get("li");
                    listData.set(count, Integer.parseInt(value_return));
                    count++;
                }
            } else if (i == 2) {
                for (String key : orgmaplist.keySet()) {
                    String value_return = orgmaplist.get(key).get("ex");
                    listData.set(count, Integer.parseInt(value_return));
                    count++;
                }
            } else if (i == 3) {
                for (String key : orgmaplist.keySet()) {
                    String value_return = orgmaplist.get(key).get("ga");
                    listData.set(count, Integer.parseInt(value_return));
                    count++;
                }
            } else if (i == 4) {
                for (String key : orgmaplist.keySet()) {
                    String value_return = orgmaplist.get(key).get("re");
                    listData.set(count, Integer.parseInt(value_return));
                    count++;
                }
            } else if (i == 5) {
                for (String key : orgmaplist.keySet()) {
                    String value_return = orgmaplist.get(key).get("ch");
                    listData.set(count, Integer.parseInt(value_return));
                    count++;
                }
            } else if (i == 6) {
                for (String key : orgmaplist.keySet()) {
                    String value_return = orgmaplist.get(key).get("ot");
                    listData.set(count, Integer.parseInt(value_return));
                    count++;
                }
            }

            series.add(new Series(legend.get(i), "line", listData));
        }

        EchartData data = new EchartData(legend, cate, series);
        return data;
    }


    //**********************************************************勘查分析****************************************************

    @RequestMapping("/surveySummary")
    public ModelAndView surveySummary(HttpServletRequest request) throws ParseException {
        ModelAndView view = new ModelAndView("/statistic/surveySummary");
        User user = super.getCurrentUser(request);
        if (user == null) {
            view = new ModelAndView("/auth/login");
            return view;
        }
        view.addObject("user", user);
        return view;
    }

    @RequestMapping("/actdata6")
    @ResponseBody
    public List<EchartCompareData> actdata6(String category, String year, HttpServletRequest request) {
        List<EchartCompareData> list = Lists.newArrayList();

        EchartCompareData wenyi = new EchartCompareData();
        wenyi.setName("文艺演出");
        list.add(wenyi);

        EchartCompareData tiyu = new EchartCompareData();
        tiyu.setName("体育赛事");
        list.add(tiyu);

        EchartCompareData zhanlan = new EchartCompareData();
        zhanlan.setName("展览展销");
        list.add(zhanlan);

        EchartCompareData youyuan = new EchartCompareData();
        youyuan.setName("游园灯会");
        list.add(youyuan);

        EchartCompareData gongyi = new EchartCompareData();
        gongyi.setName("公益慈善");
        list.add(gongyi);

        EchartCompareData zhaopin = new EchartCompareData();
        zhaopin.setName("招聘会");
        list.add(zhaopin);

        EchartCompareData qita = new EchartCompareData();
        qita.setName("其他");
        list.add(qita);


        String sql = "select a.typeCode as pname,count(i.id) as total from sp_activity a";
        sql = sql + " inner join sp_activity_survey s on s.activityId = a.id ";
        sql = sql + " inner join sp_activity_survey_detail i on i.surveyId = s.id ";

        String sql_where = " where 1=1 ";

        User user = super.getCurrentUser(request);
        if (user.getRegionLevel() == 1) {
            sql_where += " AND a.province = '" + user.getProvince() + "' ";
        } else if (user.getRegionLevel() == 2) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
        } else if (user.getRegionLevel() == 3) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
            sql_where += " AND a.area = '" + user.getArea() + "' ";
        }

        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and s.startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and s.startDate <= '" + year + "-12-31' ";
        }

        if (!category.equals("ALL")) {
            sql_where = sql_where + " and a.typeCode = '" + category + "' ";
        }

        sql = sql + sql_where + " group by pname ";


        List<AnalysisDto> dtoList = statisticsService.findListBySql(sql);
        dtoList.forEach(dto -> {
            if ("100002".equals(dto.getPname())) {
                wenyi.setValue(wenyi.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100001".equals(dto.getPname())) {
                tiyu.setValue(tiyu.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100003".equals(dto.getPname())) {
                zhanlan.setValue(zhanlan.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100004".equals(dto.getPname())) {
                youyuan.setValue(youyuan.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100006".equals(dto.getPname())) {
                gongyi.setValue(gongyi.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100005".equals(dto.getPname())) {
                zhaopin.setValue(zhaopin.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100007".equals(dto.getPname())) {
                qita.setValue(qita.getValue() + Integer.parseInt(dto.getTotal()));
            }

        });


        //---------------------整改-------------------

        //part 3
        sql_where = " where 1=1 ";
        if (user.getRegionLevel() == 1) {
            sql_where += " AND a.province = '" + user.getProvince() + "' ";
        } else if (user.getRegionLevel() == 2) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
        } else if (user.getRegionLevel() == 3) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
            sql_where += " AND a.area = '" + user.getArea() + "' ";
        }

        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and s.startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and s.startDate <= '" + year + "-12-31' ";
        }
        if (!category.equals("ALL")) {
            sql_where = sql_where + " and a.typeCode = '" + category + "' ";
        }
        sql_where = sql_where + " and i.solve is not null ";

        sql = "select a.typeCode as pname,count(s.id) as total from sp_activity a";
        sql = sql + " inner join sp_activity_survey s on s.activityId = a.id ";
        sql = sql + " inner join sp_activity_survey_detail i on i.surveyId = s.id ";

        sql = sql + sql_where + " group by pname";

        List<AnalysisDto> dtoList2 = statisticsService.findListBySql(sql);
        dtoList2.forEach(dto -> {
            if ("100002".equals(dto.getPname())) {
                wenyi.setValue2(wenyi.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100001".equals(dto.getPname())) {
                tiyu.setValue2(tiyu.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100003".equals(dto.getPname())) {
                zhanlan.setValue2(zhanlan.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100004".equals(dto.getPname())) {
                youyuan.setValue2(youyuan.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100006".equals(dto.getPname())) {
                gongyi.setValue2(gongyi.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100005".equals(dto.getPname())) {
                zhaopin.setValue2(zhaopin.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100007".equals(dto.getPname())) {
                qita.setValue2(qita.getValue2() + Integer.parseInt(dto.getTotal()));
            }
        });

        return list;

    }

    @RequestMapping("/actdata7")
    @ResponseBody
    public List<EchartCompareData> actdata7(String category, String year, HttpServletRequest request) {
        List<EchartCompareData> list = Lists.newArrayList();

        EchartCompareData wenyi = new EchartCompareData();
        wenyi.setName("文艺演出");
        list.add(wenyi);

        EchartCompareData tiyu = new EchartCompareData();
        tiyu.setName("体育赛事");
        list.add(tiyu);

        EchartCompareData zhanlan = new EchartCompareData();
        zhanlan.setName("展览展销");
        list.add(zhanlan);

        EchartCompareData youyuan = new EchartCompareData();
        youyuan.setName("游园灯会");
        list.add(youyuan);

        EchartCompareData gongyi = new EchartCompareData();
        gongyi.setName("公益慈善");
        list.add(gongyi);

        EchartCompareData zhaopin = new EchartCompareData();
        zhaopin.setName("招聘会");
        list.add(zhaopin);

        EchartCompareData qita = new EchartCompareData();
        qita.setName("其他");
        list.add(qita);

        String sql_where = " where 1=1 ";
        User user = super.getCurrentUser(request);
        if (user.getRegionLevel() == 1) {
            sql_where += " AND a.province = '" + user.getProvince() + "' ";
        } else if (user.getRegionLevel() == 2) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
        } else if (user.getRegionLevel() == 3) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
            sql_where += " AND a.area = '" + user.getArea() + "' ";
        }
        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and s.startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and s.startDate <= '" + year + "-12-31' ";
        }
        if (!category.equals("ALL")) {
            sql_where = sql_where + " and a.typeCode = '" + category + "' ";
        }

        String sql = "select a.typeCode as pname,count(s.id) as total from sp_activity a";
        sql = sql + " inner join sp_activity_survey s on s.activityId = a.id ";
        sql = sql + sql_where + " group by pname";

        List<AnalysisDto> dtoList = statisticsService.findListBySql(sql);
        dtoList.forEach(dto -> {
            if ("100002".equals(dto.getPname())) {
                wenyi.setValue(wenyi.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100001".equals(dto.getPname())) {
                tiyu.setValue(tiyu.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100003".equals(dto.getPname())) {
                zhanlan.setValue(zhanlan.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100004".equals(dto.getPname())) {
                youyuan.setValue(youyuan.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100006".equals(dto.getPname())) {
                gongyi.setValue(gongyi.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100005".equals(dto.getPname())) {
                zhaopin.setValue(zhaopin.getValue() + Integer.parseInt(dto.getTotal()));
            } else if ("100007".equals(dto.getPname())) {
                qita.setValue(qita.getValue() + Integer.parseInt(dto.getTotal()));
            }
        });


        //part 3 获取活动次数
        sql_where = " where 1=1 ";
        if (user.getRegionLevel() == 1) {
            sql_where += " AND a.province = '" + user.getProvince() + "' ";
        } else if (user.getRegionLevel() == 2) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
        } else if (user.getRegionLevel() == 3) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
            sql_where += " AND a.area = '" + user.getArea() + "' ";
        }
        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and a.startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and a.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and a.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and a.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and a.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and a.startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and a.startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and a.startDate <= '" + year + "-12-31' ";
        }

        sql = "select a.typeCode as pname,count(a.id) as total from sp_activity a";
        sql = sql + sql_where + " group by pname ";


        List<AnalysisDto> dtolist1 = statisticsService.findListBySql(sql);
        dtolist1.forEach(dto -> {
            if ("100002".equals(dto.getPname())) {
                wenyi.setValue2(wenyi.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100001".equals(dto.getPname())) {
                tiyu.setValue2(wenyi.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100003".equals(dto.getPname())) {
                zhanlan.setValue2(wenyi.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100004".equals(dto.getPname())) {
                youyuan.setValue2(wenyi.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100006".equals(dto.getPname())) {
                gongyi.setValue2(wenyi.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100005".equals(dto.getPname())) {
                zhaopin.setValue2(wenyi.getValue2() + Integer.parseInt(dto.getTotal()));
            } else if ("100007".equals(dto.getPname())) {
                qita.setValue2(qita.getValue2() + Integer.parseInt(dto.getTotal()));
            }
        });

        return list;
    }


    @RequestMapping("/actdata8")
    @ResponseBody
    public EchartData actdata8(String category, String year, HttpServletRequest request) {
        List<String> list = new ArrayList<String>();
        list.add("100002");
        list.add("100001");
        list.add("100003");
        list.add("100004");
        list.add("100006");
        list.add("100005");
        list.add("100007");

        List<String> legend = new ArrayList<String>(Arrays.asList(new String[]{"'文艺演出'", "'体育赛事'", "'展览展销'", "'游园灯会'", "'公益慈善'", "'招聘会'", "'其他'"}));//数据分组
        List<String> cate = new ArrayList<String>(Arrays.asList(new String[]{"'一月'", "'二月'", "'三月'", "'四月'", "'五月'", "'六月'", "'七月'", "'八月'", "'九月'", "'十月'", "'十一月'", "'十二月'"}));//横坐标
        List<Series> series = new ArrayList<Series>();//纵坐标

        List<Float> listData;

        if (!category.equals("ALL")) {
            list.clear();
            list.add(category);

            legend.clear();
            legend.add(ActivityTypeEnum.convertStringName(category).toString());
        }
        User user = super.getCurrentUser(request);
        for (int i = 0; i < list.size(); i++) {
            listData = new ArrayList<Float>(Arrays.asList(0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f));

            List<Integer> infoList = getData8(list.get(i), year, user);
            List<Integer> infoList2 = getData82(list.get(i), year, user);

            if (infoList.size() > 0) {
                for (int j = 0; j < infoList.size(); j++) {
                    if (infoList.get(j) != 0) {
                        float x = (float) ((float) infoList2.get(j) / ((float) infoList.get(j)));
                        listData.set(j, x * 100);
                    } else {
                        listData.set(j, (float) 0);
                    }
                }
            }

            series.add(new Series(legend.get(i), "line", listData));
        }

        EchartData data = new EchartData(legend, cate, series);
        return data;
    }

    private List<Integer> getData8(String category, String year, User user) {
        List<Integer> listData;

        listData = new ArrayList<Integer>(Arrays.asList(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0));

        String sql_where = " where 1=1 ";

        if (user.getRegionLevel() == 1) {
            sql_where += " AND a.province = '" + user.getProvince() + "' ";
        } else if (user.getRegionLevel() == 2) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
        } else if (user.getRegionLevel() == 3) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
            sql_where += " AND a.area = '" + user.getArea() + "' ";
        }

        sql_where = sql_where + " and a.typeCode = '" + category + "' ";

        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and s.startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and s.startDate <= '" + year + "-12-31' ";
        }

        String sql = "SELECT month(s.startDate) as pname,count(i.id) as total ";
        sql = sql + " FROM sp_activity a ";
        sql = sql + " inner join sp_activity_survey s on s.activityId = a.id  ";
        sql = sql + " left join sp_activity_survey_detail i on i.surveyid = s.id  ";
        sql = sql + sql_where + "  group by pname ";

        List<AnalysisDto> dtoList = statisticsService.findListBySql(sql);

        for (AnalysisDto dto : dtoList) {
            String m = dto.getPname();
            int index = Integer.parseInt(m) - 1;
            listData.set(index, Integer.parseInt(dto.getTotal()));
        }

        return listData;
    }


    private List<Integer> getData82(String category, String year, User user) {
        List<Integer> listData;

        listData = new ArrayList<Integer>(Arrays.asList(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0));

        //part 3
        String sql_where = " where 1=1 ";

        if (user.getRegionLevel() == 1) {
            sql_where += " AND a.province = '" + user.getProvince() + "' ";
        } else if (user.getRegionLevel() == 2) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
        } else if (user.getRegionLevel() == 3) {
            sql_where += " AND a.city = '" + user.getCity() + "' ";
            sql_where += " AND a.area = '" + user.getArea() + "' ";
        }

        sql_where = sql_where + " and a.typeCode = '" + category + "' ";

        if (year.equals("today")) {
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + currentDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("week")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addDay(new Date(), -7), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else if (year.equals("month")) {
            String beforeDateString = BcxinDateUtil.convertDateToString(BcxinDateUtil.addMonth(new Date(), -1), "yyyy-MM-dd");
            String currentDateString = BcxinDateUtil.getCurrentDate();
            sql_where = sql_where + " and s.startDate >= '" + beforeDateString + "' ";
            sql_where = sql_where + " and s.startDate <= '" + currentDateString + "' ";
        } else {
            sql_where = sql_where + " and s.startDate >= '" + year + "-01-01' ";
            sql_where = sql_where + " and s.startDate <= '" + year + "-12-31' ";
        }

        sql_where = sql_where + " and i.solve is not null";

        String sql = "SELECT month(s.startDate) as pname,count(i.id) as total ";
        sql = sql + " FROM sp_activity a ";
        sql = sql + " inner join sp_activity_survey s on s.activityId = a.id  ";
        sql = sql + " left join sp_activity_survey_detail i on i.surveyid = s.id  ";

        sql = sql + sql_where + " group by pname ";

        List<AnalysisDto> dtoList = statisticsService.findListBySql(sql);

        for (AnalysisDto dto : dtoList) {
            if (dto == null) {
                continue;
            }
            String m = dto.getPname();
            int index = Integer.parseInt(m) - 1;

            listData.set(index, Integer.parseInt(dto.getTotal()));
        }

        return listData;
    }

}