package com.bcxin.sp.work.controller.telphone;

import com.bcxin.sp.config.annotation.AuditLogAn;
import com.bcxin.sp.config.annotation.OperLog;
import com.bcxin.sp.work.controller.sys.BaseController;
import com.bcxin.sp.work.entity.domain.user.TelPhone;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.AjaxResult;
import com.bcxin.sp.work.service.telPhoneService.TelPhoneService;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.MenuConst;
import com.bcxin.sp.work.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;

/**
 * 警局电话
 *
 * @author Administrator
 */
@Controller
@RequestMapping("/business/telPhone")
@Slf4j
public class TelPhoneController extends BaseController {

    @Autowired
    private TelPhoneService telPhoneService;

    /**
     * 保存/修改 警局电话
     *
     * @param request
     * @return
     * @throws Exception
     */
    @AuditLogAn(operateType = DictConst.OP_SAVE,menuName = MenuConst.XTPZ_JJDH_TJ)
    @OperLog(menuName = "警局咨询电话",operateType = "修改警局电话",operateObj = "#address")
    @RequestMapping("saveOrUpdateTelPhone")
    @ResponseBody
    public AjaxResult saveOrUpdateTelPhone(HttpServletRequest request) throws Exception {

        String phone = request.getParameter("telPhone");
        String id = request.getParameter("id");
        User user = super.getCurrentUser(request);
        AjaxResult result = new AjaxResult();

        if (StringUtil.isEmpty(id)) {
            String place = user.getCity();
            if (user.getRegionLevel() == 3) {
                place = user.getArea();
            }
            TelPhone telPhone = telPhoneService.instance(user.getRegionLevel(),place);
            telPhone.setLevel(user.getRegionLevel());
            telPhone.setProvince(user.getProvince());
            telPhone.setCity(user.getCity());
            telPhone.setArea(user.getArea());
            telPhone.setTelPhone(phone);
            int flag = telPhoneService.save(telPhone);
            if (flag < 0) {
                result.setSuccessful(false);
                return result;
            } else {
                result.setSuccessful(true);
                return result;
            }

        } else {
            TelPhone telPhone = telPhoneService.selectById(Long.parseLong(id));
            telPhone.setTelPhone(phone);
            int flag = telPhoneService.update(telPhone);
            if (flag < 0) {
                result.setSuccessful(false);
                return result;
            } else {
                result.setSuccessful(true);
                return result;
            }
        }
    }


    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.XTPZ_JJDH)
    @RequestMapping("/selectTelPhoneByLevel")
    public ModelAndView selectTelPhoneByLevel(HttpServletRequest request) {
        ModelAndView view = new ModelAndView("/telPhone/telPhone");
        User user = super.getCurrentUser(request);
        int level = user.getRegionLevel();
        if (level == 1) {
            return view;
        }
        String place = user.getCity();
        if (level == 3) {
            place = user.getArea();
        }
        TelPhone telPhone = telPhoneService.selectTelPhoneByLevel(user.getRegionLevel(), place);
        if (telPhone != null) {
            view.addObject("telPhone", telPhone.getTelPhone());
            view.addObject("telId", telPhone.getId());
            view.addObject("address", getAddress(user));
        } else {
            view.addObject("telPhone", "");
            view.addObject("telId", "");
            view.addObject("address", getAddress(user));
        }

        return view;
    }

    private String getAddress(User user){
        String addr="";
        if(StringUtil.isNotEmpty(user.getProvincename())){
            addr+=user.getProvincename();
        }
        if(StringUtil.isNotEmpty(user.getCityname())){
            addr+=user.getCityname();
        }
        if(StringUtil.isNotEmpty(user.getAreaname())){
            addr+=user.getAreaname();
        }
        return addr;
    }
}
