package com.bcxin.sp.work.controller.third.anbao;

import com.bcxin.sp.config.annotation.AuditLogAn;
import com.bcxin.sp.config.annotation.SysManageAuth;
import com.bcxin.sp.work.controller.sys.BaseController;
import com.bcxin.sp.work.entity.domain.activity.approve.ApproveRole;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.AjaxResult;
import com.bcxin.sp.work.service.activity.approve.ApproveRoleService;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.MenuConst;
import com.bcxin.sp.work.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;

/**
 * 审批角色管理
 */
@Controller
@RequestMapping("/thirdBusiness/anBaoApproveRole")
@Slf4j
public class AnBaoApproveRoleController extends BaseController {

    @Resource
    private ApproveRoleService approveRoleService;

    @RequestMapping("/list")
    @SysManageAuth(limitRole = DictConst.ROLE_APPROVE)
    public ModelAndView list(HttpServletRequest request) {
        ModelAndView view = new ModelAndView("/activity/anbao/approve/role");
        view.addObject("access_token",request.getParameter("access_token") );
        return view;
    }

//    @AuditLogAn(operateType = DictConst.OP_SAVE,menuName = MenuConst.XTPZ_JSPZ_TJJS)
    @RequestMapping("createOrUpdate")
    public @ResponseBody AjaxResult createOrUpdate(ApproveRole approveRole, HttpServletRequest request) {
        User cuser = super.getCurrentUser(request);
        approveRole.setUpdater(cuser.getUsername());
        if (approveRole.getId() == null) {
            approveRole.setAddTime(new Date());
            approveRole.setProvince(cuser.getProvince());
            approveRole.setCity(cuser.getCity());
            approveRole.setArea(cuser.getArea());
            if(StringUtil.isEmpty(cuser.getProvince())){
                approveRole.setProvince(null);
            }
            approveRoleService.save(approveRole);
        } else {
            approveRole.setUpdateTime(new Date());
            approveRoleService.update(approveRole);
        }
        AjaxResult result = new AjaxResult();
        result.setSuccessful(true);
        result.setObj(approveRole);
        return result;
    }

//    @AuditLogAn(operateType = DictConst.OP_DELETE,menuName = MenuConst.XTPZ_JSPZ_SCJS)
    @RequestMapping("delete")
    public @ResponseBody
    AjaxResult delete(ApproveRole approveRole) {
        approveRoleService.delete(approveRole);
        AjaxResult result = new AjaxResult();
        result.setSuccessful(true);
        result.setObj(approveRole);
        return result;
    }

//    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.XTPZ_JSPZ)
    @RequestMapping("findData")
    public @ResponseBody
    List<ApproveRole> findData(HttpServletRequest request) throws Exception {
        User user = super.getCurrentUser(request);
        String province = "";
        String city = "";
        String area = "";
        String areaCode = request.getParameter("areaCode");
        if (!StringUtils.isEmpty(areaCode)){
            if (areaCode.endsWith("0000")) {//省
                province = areaCode;
            } else if (areaCode.endsWith("00")) {//市
                province = areaCode.substring(0,2)+"0000";
                city = areaCode;
            } else {//区
                province = areaCode.substring(0,2)+"0000";
                city = areaCode.substring(0,4)+"00";
                area = areaCode;
            }
        } else {
            province = user.getProvince();
            city = user.getCity();
            area = user.getArea();
        }
        return approveRoleService.findByRegion(province, city, area);
    }

    @RequestMapping("/approveRoleModal")
    public ModelAndView approveRoleModal(HttpServletRequest request) {
        ModelAndView view = new ModelAndView("/activity/anbao/dialog/approveRoleModal");
        view.addObject("access_token",request.getParameter("access_token") );
        view.addObject("areaCode", request.getParameter("areaCode"));
        return view;
    }
}
