package com.bcxin.sp.work.controller.third.anbao;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Console;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.bcxin.sp.config.annotation.AuditLogAn;
import com.bcxin.sp.work.controller.sys.BaseController;
import com.bcxin.sp.work.entity.dto.FileNumberDTO;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.service.activity.approve.FileNumberService;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.MenuConst;
import com.bcxin.sp.work.util.StringUtil;
import com.google.common.collect.Maps;
import com.google.common.io.Files;
import lombok.Cleanup;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 文件编号管理
 *
 * @author 张建华
 */
@Controller
@RequestMapping("/thirdBusiness/anBaoFileNumber")
@Slf4j
public class AnBaoFileNumberController extends BaseController {

    @Resource
    private FileNumberService fileNumberService;


    @RequestMapping("list")
    public ModelAndView upgradeList(HttpServletRequest request) throws Exception {
        ModelAndView view = new ModelAndView("/activity/anbao/approve/fileNumberList");
        view.addObject("access_token",request.getParameter("access_token") );
        return view;
    }


    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.XTPZ_BHJL)
    @RequestMapping("/fileNumberList")
    @ResponseBody
    public SearchDto<FileNumberDTO> findData(SearchDto<FileNumberDTO> dto, HttpServletRequest request) throws Exception {

        SearchDto<FileNumberDTO> result = new SearchDto<>();

        if (StringUtil.isNotEmpty(dto.getStartDate())) {
            dto.setStartDate(dto.getStartDate() + " 00:00:00");
        }
        if (StringUtil.isNotEmpty(dto.getEndDate())) {
            dto.setEndDate(dto.getEndDate() + " 23:59:59");
        }

        List<FileNumberDTO> fileNumberList = fileNumberService.listFileNumber(dto);

        long count = fileNumberService.countFileNumber(dto);

        result.setData(fileNumberList);
        result.setIDisplayStart(dto.getIDisplayStart());
        result.setIDisplayLength(dto.getIDisplayLength());
        result.setITotalRecords(count);
        result.setITotalDisplayRecords(count);

        return result;
    }


    /**
     * 导出文件编号Excel文件
     *
     * @param request
     * @param response
     * @throws Exception
     * @author 张建华
     * @date 2018年11月28日11:01:17
     */
    @AuditLogAn(operateType = DictConst.OP_SELECT,menuName = MenuConst.XTPZ_BHJL_DCSPJG)
    @RequestMapping("/exportFileNumber")
    public void exportFileNumber(HttpServletRequest request, HttpServletResponse response,Map<String,String> headers) throws Exception {

        @Cleanup ServletOutputStream out = response.getOutputStream();

        ArrayList<Map<String, Object>> rows = CollUtil.newArrayList();

        String province = request.getParameter("province");
        String city = request.getParameter("city");
        String area = request.getParameter("area");
        String keywords = request.getParameter("keywords");
        String startDate = request.getParameter("startDate");
        String endDate = request.getParameter("endDate");

        FileNumberDTO dto = new FileNumberDTO();
        dto.setStartDate(startDate);
        dto.setEndDate(endDate);
        if (StringUtil.isNotEmpty(dto.getStartDate())) {
            dto.setStartDate(dto.getStartDate() + " 00:00:00");
        }
        if (StringUtil.isNotEmpty(dto.getEndDate())) {
            dto.setEndDate(dto.getEndDate() + " 23:59:59");
        }

        dto.setKeywords(keywords);
        dto.setProvince(province);
        dto.setCity(city);
        dto.setArea(area);

        List<Map<String, Object>> fileNumberList = fileNumberService.listFileNumberOfMap(dto);

        Map<String,String> titleMap = Maps.newLinkedHashMap();
        //province 为数据库查出来的别名
        titleMap.put("province", "省份");
        titleMap.put("city", "城市");
        titleMap.put("area", "区县");
        titleMap.put("fileName", "生成文件");
        titleMap.put("addTime", "生成时间");
        titleMap.put("fileNumber", "生成编号");
        titleMap.put("realName", "操作人员");

	    /*循环报表数据*/
        fileNumberList.forEach(info->{
            Map<String, Object> map = Maps.newLinkedHashMap();
             /*循环表头*/
            for (Map.Entry<String, String> entry : titleMap.entrySet()) {
                //将数据库别名替换成表头
                map.put(entry.getValue(), info.get(entry.getKey()));
            }
            rows.add(map);
        });

        //文件名
        String reportName =  DateUtil.today() + "_" + System.currentTimeMillis();

        File dir = Files.createTempDir();
        File file = new File(dir.getPath() + "/" + System.currentTimeMillis() + ".xls");
        // 通过工具类创建writer，默认创建xls格式
        ExcelWriter writer = ExcelUtil.getWriter(file);
        if ( headers != null) {
            writer.setHeaderAlias(headers);
        }

        //一次性写出内容，使用默认样式
        writer.write(rows);
        //out为OutputStream，需要写出到的目标流
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode( reportName + ".xls", "utf-8"));
        Console.log("========================={}", rows.size());
        writer.flush(out);
        writer.close();
        FileUtil.del(file);

    }
}
