package com.bcxin.sp.work.service.activity.base;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.bcxin.sp.work.dao.activity.approve.ApproveHistoryMapper;
import com.bcxin.sp.work.dao.activity.base.ActivityMapper;
import com.bcxin.sp.work.dao.activity.extend.*;
import com.bcxin.sp.work.dao.sys.ConfigApproveLimitMapper;
import com.bcxin.sp.work.dao.user.UserMapper;
import com.bcxin.sp.work.entity.PageResult;
import com.bcxin.sp.work.entity.domain.activity.action.ActivityAction;
import com.bcxin.sp.work.entity.domain.activity.approve.ApproveHistory;
import com.bcxin.sp.work.entity.domain.activity.base.Activity;
import com.bcxin.sp.work.entity.domain.activity.expand.ActivityMaterial;
import com.bcxin.sp.work.entity.domain.activity.expand.ActivityReportDetail;
import com.bcxin.sp.work.entity.domain.activity.expand.Workflow;
import com.bcxin.sp.work.entity.domain.activity.session.ActivitySession;
import com.bcxin.sp.work.entity.domain.sys.ConfigApproveLimit;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.AjaxResult;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.entity.dto.SyncTaskDTO;
import com.bcxin.sp.work.service.activity.action.ActivityActionService;
import com.bcxin.sp.work.service.common.NotifyService;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.GlobalConst;
import com.bcxin.sp.work.util.Const.TipsConst;
import com.bcxin.sp.work.util.StringUtil;
import com.bcxin.sp.work.util.sys.ConfigUtil;
import com.google.common.base.Objects;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class ActivityService {

    @Resource
    private NotifyService notifyService;

    @Resource
    private UserMapper userMapper;

    @Resource
    private ActivityMapper activityMapper;

    @Resource
    private ActivityReportDetailMapper activityReportDetailMapper;

    @Resource
    private ActivityMaterialMapper activityMaterialMapper;

    @Resource
    private WorkflowMapper workflowMapper;

    @Resource
    private ApproveHistoryMapper approveHistoryMapper;

    @Resource
    private ConfigApproveLimitMapper configApproveLimitMapper;

    @Resource
    private ActivitySessionMapper activitySessionMapper;
    @Resource
    private ActivityActionService activityActionService;

    public Activity save(Activity activity) {
        activityMapper.insert(activity);
        return activity;
    }

    public Activity findById(long id) {
        return activityMapper.selectById(id);
    }

    public ActivityReportDetail findReportDetailById(long id) {
        return activityReportDetailMapper.findByActivityId(id);
    }

    public Activity findFullById(long id) {
        Activity activity = activityMapper.findFullById(id);
        ActivityAction activityAction = activityActionService.getByActivityId(id, "");
        if(activityAction != null){
            activity.setActionStatus(activityAction.getActionStatus());
        }
        return activity;
    }

    public Activity findByPssId(long pssId) {
        List<Activity> list = activityMapper.selectList(new EntityWrapper<Activity>().eq("pssId", pssId));
        if (CollectionUtil.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    public Activity findByActivityNo(String activityNo) {
        List<Activity> list = activityMapper.selectList(new EntityWrapper<Activity>().eq("activityNo", activityNo));
        if (CollectionUtil.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    public Activity findLastActivity(User user){
        return activityMapper.findLastActivity(user.getProvince(),user.getCity(),user.getArea());
    }

    /**
     * 查找属于这个用户的审批
     * @param dto
     * @return
     * @throws Exception
     */
    public PageResult findApproveByUser(SearchDto<Activity> dto) throws Exception {
        User user = userMapper.selectById(dto.getUserId());
        dto.setHistorySearch(false);
        //StringBuffer condition = SearchUtil.getCondition(dto,false);
        ///* 排除政府单位的活动 */
        ///* 如果为空，默认只查商业的 */
        //if(StringUtil.isEmpty(dto.getDataSource())){
        //    condition.append(" AND a.isBackUp = 0 ");
        //}
        ///* 从统计那边来的，需要查商业的活动 */
        //else if("biz".equals(dto.getDataSource())){
        //    condition.append(" AND a.isBackUp = 0 ");
        //}
        ///* 从统计那边来的，需要查商业的活动 */
        //else if("gov".equals(dto.getDataSource())){
        //    condition.append(" AND a.isBackUp = 1 ");
        //}
        ///* 从统计那边来的，查所有的活动 */
        //else if("summary".equals(dto.getDataSource())){
        //    /* 查所有的活动，不加条件了 */
        //}
        ///* 从统计那边来的，查所有的活动（包括历史活动） */
        //else if("union".equals(dto.getDataSource())){
        //    /* （包括历史活动） */
        //}
        if (Objects.equal(DictConst.ROLE_SUPERADMIN, user.getRole())
                || Objects.equal(DictConst.ROLE_ADMIN, user.getRole())
                || Objects.equal(DictConst.ROLE_ADMIN_APPROVE, user.getRole())
                || Objects.equal(DictConst.ROLE_EXPORT, user.getRole())
                || Objects.equal(DictConst.ROLE_READ, user.getRole()) || Objects.equal(DictConst.ROLE_DOWNLOAD, user.getRole())) {
            dto.setRole("ADMIN");
            List<Activity> activityList = activityMapper.findActivity(dto);
            long total = activityMapper.findCountOfActivity(dto);
            return new PageResult(activityList,total);
        }
        /* 增加审批级别过滤 */
        dto.setRegionLevel(user.getRegionLevel());
        List<Activity> activityList = activityMapper.findActivityByUser(dto);
        long total = activityMapper.findCountOfActivity(dto);
        return new PageResult(activityList,total);
    }

    /**
     * 添加是否能审批的标示
     */
    public List<Activity> authApprove(User user,List<Activity> activityList) {
        for (Activity activity : activityList) {
            /* 需要审批的状态，审批人为第一级审批的用户 */
            if (DictConst.STATUS_CSZ.equals(activity.getApproveStatus())
                    || DictConst.STATUS_SQCP.equals(activity.getApproveStatus())
                    || DictConst.STATUS_SLZ.equals(activity.getApproveStatus())
                    || DictConst.STATUS_NONE.equals(activity.getApproveStatus())
                    || DictConst.STATUS_IN.equals(activity.getApproveStatus())) {
                /* 如果是指定用户，则用指定用户审批 */
                if (StringUtil.isNotEmpty(activity.getNextApproveUser())) {
                    /* 当前登录用户是指定的用户 */
                    if (user.getId().equals(Long.parseLong(activity.getNextApproveUser()))) {
                        activity.setNeedApproval(DictConst.Y);
                    }
                }
                /* 如果是指定角色，则用指定角色审批 */
                else if (StringUtil.isNotEmpty(activity.getNextApproveRole())) {
                    if (StringUtil.isNotEmpty(user.getWorkRole())) {
                        if (user.getWorkRole().equals(activity.getNextApproveRole())) {
                            activity.setNeedApproval(DictConst.Y);
                        }
                    }
                }
            }
            ///* 需要审批的状态，审批人为下一级审批的用户 */
            //if (DictConst.STATUS_IN.equals(activity.getApproveStatus())){
            //    User appointUser = userMapper.findAppointApproveUser(activity.getId());
            //    if (appointUser == null ) {
            //        continue;
            //    }
            //    if (appointUser.getId().equals(user.getId())) {
            //        activity.setNeedApproval(DictConst.Y);
            //    }
            //}
        }
        return activityList;
    }

    public List<Activity> findActivity(SearchDto<Activity> dto,User user) throws Exception {
        //StringBuffer condition = SearchUtil.getCondition(dto,false);
        //StringBuffer limitCondition = new StringBuffer();
        //limitCondition.append(" limit ");
        //limitCondition.append(dto.getIDisplayStart());
        //limitCondition.append(",");
        //limitCondition.append(dto.getIDisplayLength());
        System.out.println("======> 访问findActivity:1");
        dto.setHistorySearch(false);
        if (Objects.equal(DictConst.ROLE_SUPERADMIN, user.getRole())
                || Objects.equal(DictConst.ROLE_ADMIN, user.getRole())
                || Objects.equal(DictConst.ROLE_ADMIN_APPROVE, user.getRole())
                || Objects.equal(DictConst.ROLE_EXPORT, user.getRole())
                || Objects.equal(DictConst.ROLE_READ, user.getRole())
                || Objects.equal(DictConst.ROLE_DOWNLOAD, user.getRole())) {
            System.out.println("======> 访问findActivity:2");
            return activityMapper.findActivity(dto);
        }

        /* 增加审批级别过滤 */
        if (Objects.equal(DictConst.ROLE_APPROVE, user.getRole())){
            System.out.println("======> 访问findActivity:3");
            if (user.getRegionLevel()==2) {
                System.out.println("======> 访问findActivity:4");
                // 市级审批人员，只查询到市
                //condition.append(" and a.area is null ");
                dto.setRole(DictConst.ROLE_APPROVE);
            }
        }
        System.out.println("======> 访问findActivity:5");
        return activityMapper.findActivity(dto);
    }


    public long findCountOfActivity(SearchDto<Activity> dto,User user) {
        dto.setHistorySearch(false);
        //StringBuffer condition = SearchUtil.getCondition(dto,false);
        if (Objects.equal(DictConst.ROLE_APPROVE, user.getRole())) {
            /* 增加审批级别过滤 */
            if(user.getRegionLevel()==2){
                // 市级审批人员，只查询到市
                //condition.append(" and a.area is null ");
                dto.setRole(DictConst.ROLE_APPROVE);
            }
        }
        return activityMapper.findCountOfActivity(dto);
    }

    public List<SyncTaskDTO> findNeedSyncTask(User user) {
        /*导出员角色，导出所有符合条件的活动*/
        if (DictConst.ROLE_EXPORT.equalsIgnoreCase(user.getRole())) {
            return activityMapper.findNeedSyncTask(null, null,null);
        }
        /*超级管理员角色，导出所有符合条件的活动*/
        if (DictConst.ROLE_SUPERADMIN.equalsIgnoreCase(user.getRole())) {
            return activityMapper.findNeedSyncTask(null, null,null);
        }
        /* 省级用户 */
        if (DictConst.APPROVELEVEL_PROVINCE.equals(user.getRegionLevel()+"")) {
            return activityMapper.findNeedSyncTask(user.getProvince(), null,null);
        }
        /* 市级用户 */
        else if (DictConst.APPROVELEVEL_CITY.equals(user.getRegionLevel()+"")) {
            return activityMapper.findNeedSyncTask(user.getProvince(), user.getCity(),null);
        }
        /* 区级用户 */
        else if (DictConst.APPROVELEVEL_AREA.equals(user.getRegionLevel()+"")) {
            return activityMapper.findNeedSyncTask(user.getProvince(), user.getCity(),user.getArea());
        }
        return Lists.newArrayList();
    }

    public List<Activity> findNeedSyncActivity(User user) {
        List<Activity> activityList = Lists.newArrayList();

        /*导出员角色，导出所有符合条件的活动*/
        if (DictConst.ROLE_EXPORT.equalsIgnoreCase(user.getRole())) {
            activityList.addAll(activityMapper.findAllNeedSyncActivity());
            activityList.addAll(activityMapper.findAllNeedPrimarySyncActivity());
            return activityList;
        }

        /*超级管理员角色，导出所有符合条件的活动*/
        if (DictConst.ROLE_SUPERADMIN.equalsIgnoreCase(user.getRole())) {
            activityList.addAll(activityMapper.findAllNeedSyncActivity());
            activityList.addAll(activityMapper.findAllNeedPrimarySyncActivity());
            return activityList;
        }

        /* 省级用户 */
        if (DictConst.APPROVELEVEL_PROVINCE.equals(user.getRegionLevel()+"")) {
            activityList.addAll(activityMapper.findNeedSyncActivity(
                    user.getRegionLevel()+"",user.getProvince(), null, null));
            activityList.addAll(activityMapper.findNeedPrimarySyncActivity(
                    user.getRegionLevel()+"",user.getProvince(), null, null));
        }
        /* 市级用户 */
        else if (DictConst.APPROVELEVEL_CITY.equals(user.getRegionLevel()+"")) {
            activityList.addAll(activityMapper.findNeedSyncActivity(
                    user.getRegionLevel()+"",user.getProvince(), user.getCity(), null));
            activityList.addAll(activityMapper.findNeedPrimarySyncActivity(
                    user.getRegionLevel()+"",user.getProvince(), user.getCity(), null));
        }
        /* 区级用户 */
        else if (DictConst.APPROVELEVEL_AREA.equals(user.getRegionLevel()+"")) {
            activityList.addAll(activityMapper.findNeedSyncActivity(
                    user.getRegionLevel()+"",user.getProvince(), user.getCity(), user.getArea()));
            activityList.addAll(activityMapper.findNeedPrimarySyncActivity(
                    user.getRegionLevel()+"",user.getProvince(), user.getCity(), user.getArea()));
        }
        return activityList;
    }

    public Activity createOrUpdate(Activity activity){
        return createOrUpdate(activity,true);
    }

    /**
     *
     * @param activity
     * @param validNumberLimit 是否校验人数
     * @return
     */
    public Activity createOrUpdate(Activity activity,boolean validNumberLimit) {

        if (activity.getId() != null) {
            /* 现在随时都可以修改区域了 */
            if (StringUtil.isEmpty(activity.getWorkflowVersion()) ) {
                setWorkFlow(activity,validNumberLimit);
            }
            activityMapper.updateById(activity);
            // 清除区域
            if(StringUtil.isEmpty(activity.getArea())) {
                activityMapper.clearArea(activity.getId());
            }
            //如果存在没有文件记录的就新增下记录
            ActivityMaterial material = activityMaterialMapper.findByActivityId(activity.getId());
            if(material == null){
                material = new ActivityMaterial();
                material.setActivityId(activity.getId());
                activityMaterialMapper.insert(material);
            }
        } else {
            // 不为政府项目 或者 不是备案项目，可创建流程
            if(!activity.isGov() || !Objects.equal(DictConst.Y,activity.getIsBackUp())){
                setWorkFlow(activity,validNumberLimit);
            }
            activityMapper.insert(activity);
            /* 插入活动材料表 */
            ActivityMaterial material = new ActivityMaterial();
            material.setActivityId(activity.getId());
            activityMaterialMapper.insert(material);

            ApproveHistory history = new ApproveHistory();

            // 政府项目且报备项目
            if (activity.isGov() && Objects.equal(DictConst.Y,activity.getIsBackUp())) {
                history.setApproveType(DictConst.APPROVETYPE_SP);
                history.setContent(TipsConst.TJBB);
            } else {
                history.setApproveType(DictConst.APPROVETYPE_CS);
                String content = TipsConst.TJCS;
                if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_gradeRank"))) {
                    content = TipsConst.TJDJPD;
                }
                history.setContent(content);
                //计算今天与活动开始时间相差几天
                long dateDiff = DateUtil.between(new Date(), activity.getStartDate(), DateUnit.DAY);
                if (dateDiff < 30){
                    history.setContent(content+"<br><span style='color: red'>活动举办日期距今已不到30天</span>");
                }
            }
            history.setIsReset(DictConst.Y);
            history.setAddTime(new Date());
            history.setActivityId(activity.getId());
            history.setUserName(activity.getSponsor());
            approveHistoryMapper.insert(history);

            /* 消息通知 */
            if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_gradeRank"))) {
                notifyService.tjdjpd(activity);
            } else {
                notifyService.tjcs(activity);
            }
        }
        return activity;
    }

    public boolean setWorkFlow(Activity activity){
        return setWorkFlow(activity,true);
    }


    /**
     * 根据活动信息匹配审批流程
     * @param activity
     * @param needValidNumberLimit
     * @return
     */
    public List<Workflow> matchWorkFlow(Activity activity, boolean needValidNumberLimit) {
        /*跨市活动*/
        if (StringUtil.isEmpty(activity.getCity())) {
            return workflowMapper.findActive(activity.getProvince(), null, null);
        }
        /*跨区活动*/
        if (StringUtil.isEmpty(activity.getArea())) {
           return workflowMapper.findActive(activity.getProvince(), activity.getCity(), activity.getArea());
        }
        // 不需要验证人数
        if (!needValidNumberLimit) {
            if (StringUtil.isNotEmpty(activity.getUnit())) {
                /* 对应了申报系统的4级 */
                if (activity.getUnit().length() == 8 && !activity.getUnit().endsWith("01")) {
                    activity.setArea(activity.getUnit());
                    return workflowMapper.findActive(activity.getProvince(), activity.getCity(), activity.getUnit());
                } else {
                    return workflowMapper.findActive(activity.getProvince(), activity.getCity(), activity.getArea());
                }
            } else {
                return workflowMapper.findActive(activity.getProvince(), activity.getCity(), activity.getArea());
            }
        }
        /* 获取区级最高审批人数 */
        int areaLimit = 5000;
        //如果配置区规模先取
        ConfigApproveLimit configAreaLimit = configApproveLimitMapper.selectByRegionId(Long.parseLong(activity.getArea()));
        if (configAreaLimit != null && configAreaLimit.getApproveLimit() > 0) {
            areaLimit = configAreaLimit.getApproveLimit();
        }else{
            configAreaLimit = configApproveLimitMapper.selectByRegionId(Long.parseLong(activity.getCity()));
            if (configAreaLimit != null) {
                areaLimit = configAreaLimit.getApproveLimit();
            }
        }
        /*configAreaLimit = configApproveLimitMapper.selectByRegionId(Long.parseLong(activity.getCity()));
        if (configAreaLimit != null) {
            areaLimit = configAreaLimit.getApproveLimit();
        }
        if("330700".equals(activity.getCity())){
            if("330702,330703,330790".contains(activity.getArea())){
                areaLimit = 10000;
            } else if("330781,330782,330783,330784,330723,330726,330727".contains(activity.getArea())){
                areaLimit = 20000;
            }
        }*/
        /* 如果活动人数小于区级最高审批人数 */
        if (activity.getNumber() < areaLimit) {
            /* 处理一个区对应多个分局 */
            if (StringUtil.isNotEmpty(activity.getUnit())) {
                /* 对应了申报系统的4级 */
                if (activity.getUnit().length() == 8 && !activity.getUnit().endsWith("01")) {
                    activity.setArea(activity.getUnit());
                    return workflowMapper.findActive(activity.getProvince(), activity.getCity(), activity.getUnit());
                } else {
                    return workflowMapper.findActive(activity.getProvince(), activity.getCity(), activity.getArea());
                }
            } else {
                return workflowMapper.findActive(activity.getProvince(), activity.getCity(), activity.getArea());
            }
        }

        /* 获取市级最高审批人数 */
        ConfigApproveLimit configCityLimit = configApproveLimitMapper.selectByRegionId(Long.parseLong(activity.getProvince()));
        int cityLimit = Integer.MAX_VALUE;
        if (configCityLimit != null) {
            cityLimit = configCityLimit.getApproveLimit();
        }
        if (activity.getNumber() < cityLimit) {
            return workflowMapper.findActive(activity.getProvince(), activity.getCity(), null);
        }
        return workflowMapper.findActive(activity.getProvince(), null, null);
    }

    /**
     * 设置审批版本
     *
     * @param activity
     * @param validNumberLimit 是否需要校验人数
     * @return
     */
    public boolean setWorkFlow(Activity activity, boolean validNumberLimit) {
        List<Workflow> workflows = matchWorkFlow(activity, validNumberLimit);
        if (CollectionUtil.isNotEmpty(workflows)) {
            Workflow workflow = workflows.get(0);
            activity.setNextApproveRole(workflow.getRole());
            activity.setWorkflowVersion(workflow.getVersion());
            if (StrUtil.isNotEmpty(workflow.getArea())) {
                activity.setApproveLevel(DictConst.APPROVELEVEL_AREA);
            } else if (StrUtil.isNotEmpty(workflow.getCity())) {
                activity.setApproveLevel(DictConst.APPROVELEVEL_CITY);
            } else if (StrUtil.isNotEmpty(workflow.getProvince())) {
                activity.setApproveLevel(DictConst.APPROVELEVEL_PROVINCE);
            }
            return true;
        }
        /* 未找到流程，修改流程未空 */
        else {
            activity.setWorkflowVersion(null);
        }

        return false;
    }

    public void delete(long id) {
        activityMapper.deleteById(id);
    }

    public void repairActivitySync() {
        activityMapper.repairActivitySync();
    }

    public void repairApproval2SL() {
        if (!GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_gradeRank"))) {
            activityMapper.repairApproval2SL();
        }
    }

    public ActivityMaterial findMaterialByActivity(long activityId) {
        return activityMaterialMapper.findByActivityId(activityId);
    }

    public Activity activityApproveUserList(Long activityId) {
        return activityMapper.findActivityForApproveUser(activityId);
    }

    public List<Map> findActivityApproveUser(Long activityId){
        return activityMapper.findActivityApproveUser(activityId);
    }

    public String reportSave(Map<String,String> parameterMap){
        String activityId = parameterMap.get("reportActivityId");
        ActivityReportDetail activityReportDetail = activityReportDetailMapper.findByActivityId(Long.parseLong(activityId));
        if (activityReportDetail==null) {
            activityReportDetail = new ActivityReportDetail();
            activityReportDetail.setAddTime(DateUtil.date());
        }
        activityReportDetail.setActivityId(Long.parseLong(activityId));
        activityReportDetail.setBaryzs(parameterMap.get("reportActivitySecuritySum"));
        activityReportDetail.setJczs(parameterMap.get("reportActivityPoliceSum"));
        activityReportDetail.setXccys(parameterMap.get("reportActivityPersonSum"));
        activityReportDetail.setZyzzs(parameterMap.get("reportActivityVolunteerSum"));

        activityReportDetail.setRjbaryzs(parameterMap.get("reportActivitySecurityAverage"));
        activityReportDetail.setRjjczs(parameterMap.get("reportActivityPoliceAverage"));
        activityReportDetail.setRjxccys(parameterMap.get("reportActivityPersonAverage"));
        activityReportDetail.setRjzyzzs(parameterMap.get("reportActivityVolunteerAverage"));

        activityReportDetail.setCityChargeName(parameterMap.get("reportCityChargeName"));
        activityReportDetail.setCityChargeJob(parameterMap.get("reportCityChargeJob"));
        activityReportDetail.setCityChargePhone(parameterMap.get("reportCityChargePhone"));

        activityReportDetail.setSceneChargeName(parameterMap.get("reportSceneChargeName"));
        activityReportDetail.setSceneChargeJob(parameterMap.get("reportSceneChargeJob"));
        activityReportDetail.setSceneChargePhone(parameterMap.get("reportSceneChargePhone"));

        activityReportDetail.setContractorChargeName(parameterMap.get("reportContractorChargeName"));
        activityReportDetail.setContractorChargeJob(parameterMap.get("reportContractorChargeJob"));
        activityReportDetail.setContractorChargePhone(parameterMap.get("reportContractorChargePhone"));

        activityReportDetail.setUpdateTime(DateUtil.date());
        if (activityReportDetail.getId()==null) {
            activityReportDetailMapper.insert(activityReportDetail);
        } else {
            activityReportDetailMapper.updateById(activityReportDetail);
        }
        return "";
    }


    /**
     * <b> 根据用户角色判断查询活动信息 </b>
     * @author ZXF
     * @create 2020/03/03 0003 13:31
     * @version
     * @注意事项 </b>
     */
    public List<Map> findActivityByUser(Long userId) {
        User user = userMapper.selectById(userId);
        String approveLevel = String.valueOf(user.getRegionLevel());
        if (Objects.equal(DictConst.ROLE_SUPERADMIN, user.getRole())
                || Objects.equal(DictConst.ROLE_ADMIN, user.getRole())
                || Objects.equal(DictConst.ROLE_ZDHD, user.getRole())
                || Objects.equal(DictConst.ROLE_ADMIN_APPROVE, user.getRole())
                || Objects.equal(DictConst.ROLE_EXPORT, user.getRole())
                || Objects.equal(DictConst.ROLE_READ, user.getRole()) || Objects.equal(DictConst.ROLE_DOWNLOAD, user.getRole())) {
            approveLevel = null;
        }
        return activityMapper.findActivityByApproveLevel(approveLevel);
    }

    /**
     * <b> 根据活动id查询活动部分详情 </b>
     * @author ZXF
     * @create 2020/03/03 0003 15:12
     * @version
     * @注意事项 </b>
     */
    public Map getActivityById(Long activityId) throws Exception {
        return activityMapper.getActivityById(activityId);
    }

    public AjaxResult submitaction(User user,Long activityId){
        AjaxResult result = new AjaxResult();
        Activity activity = findFullById(activityId);
        if(StringUtil.isEmpty(activity.getSponsorAddress())||StringUtil.isEmpty(activity.getIntroduce())||StringUtil.isEmpty(activity.getCoordinate())){
            result.setSuccessful(false);
            result.setMsg("活动坐标、简介、承办单位地址为空（历史活动不支持上报，请前往部级上报平台手动录入）");
            return result;
        }
        List<ActivitySession> resultList = activitySessionMapper.selectByActivityId(activityId);
        if(resultList.size()==0){
            result.setSuccessful(false);
            result.setMsg("活动场次信息为空（历史活动不支持上报，请前往部级上报平台手动录入）");
            return result;
        }
        for (ActivitySession activitySession : resultList){
            if(activitySession.getIsImprove() == 0){
                result.setSuccessful(false);
                result.setMsg("存在场次信息未完善");
                return result;
            }
        }
        if(!"330000".equals(activity.getProvince())){
            result.setSuccessful(false);
            result.setMsg("当前活动上报功能未开通");
            return result;
        }
        //记录上报时间
        activity = findById(activityId);
        activity.setReportTime(com.bcxin.sp.work.util.sys.DateUtil.getCurrentDateTime());
        createOrUpdate(activity);
        // 增加上报表记录
        activityActionService.createOrUpdate(user.getId(),activityId, null,0);
        result.setSuccessful(true);
        result.setMsg("上报成功");
        return result;
    }
}
