package com.bcxin.sp.work.service.activity.extend;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.bcxin.sp.work.dao.activity.extend.ActivityContractorMapper;
import com.bcxin.sp.work.entity.PageResult;
import com.bcxin.sp.work.entity.domain.activity.approve.Approve;
import com.bcxin.sp.work.entity.domain.activity.base.Activity;
import com.bcxin.sp.work.entity.domain.activity.expand.ActivityContractor;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.util.BcxinFileUtil;
import com.bcxin.sp.work.util.Const.GlobalConst;
import com.bcxin.sp.work.util.DESUtil;
import com.bcxin.sp.work.util.StringUtil;
import com.bcxin.sp.work.util.sys.ConfigUtil;
import com.google.common.collect.Maps;
import com.google.common.io.Files;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * <b> 承办者业务 </b>
 * @author ZXF
 * @create 2020/11/06 0006 10:50
 * @version
 * @注意事项 </b>
 */
@Service
@Transactional
@Slf4j
public class ActivityContractorService {

    /**
     * 同步承办者结果文件夹名称
     */
    private final static String SPJG_FOLDER = "spjg";

    /**
     * 承办者内容，文件标识
     */
    private final static String SPCFREE = "_spcfree_";

    /**
     * 承办者文件，文件标识
     */
    private final static String SPCFREEFILE = "_spcfreefile_";

    @Autowired
    private ActivityContractorMapper activityContractorMapper;

    public List<ActivityContractor> selectByActivityId(Long activityId){
        return activityContractorMapper.selectByActivityId(activityId);
    }

    public ActivityContractor findById(long id) {
        return activityContractorMapper.selectById(id);
    }

    public PageResult findListData(SearchDto<ActivityContractor> dto) throws Exception {
        List<ActivityContractor> list = activityContractorMapper.findListData(dto);
        Long total = activityContractorMapper.findListDataCnt(dto);
        return new PageResult(list, total);
    }

    public ActivityContractor createOrUpdate(ActivityContractor ac) {
        if (ac.getId() != null) {
            ac.setUpdateTime(new Date());
            activityContractorMapper.updateById(ac);
        } else {
            ac.setAddTime(new Date());
            activityContractorMapper.insert(ac);
        }
        return ac;
    }

    /**
     * <b> 冻结/解冻 </b>
     * @author ZXF
     * @create 2020/11/09 0009 13:49
     * @version
     * @注意事项 </b>
     */
    public void freeze(ActivityContractor ac) {
        if(1==ac.getIsFreeze()){
            ac.setIsFreeze(0);
        }else{
            ac.setIsFreeze(1);
        }
        try {
            communicateContractorPSS(toMap(ac));
        } catch (Exception e) {
            e.printStackTrace();
        }
        ac.setIsSync(1);
        activityContractorMapper.updateById(ac);
    }

    private Map<String, String> toMap(ActivityContractor ac) {
        Map<String, String> map = Maps.newHashMap();
        map.put("pssId", ac.getPssId() + "");
        map.put("fullName", ac.getFullName());
        map.put("contractorType", ac.getContractorType());
        map.put("adminTel", ac.getAdminTel());
        map.put("province", ac.getProvince());
        map.put("phone", ac.getPhone());
        map.put("city", ac.getCity());
        map.put("contact", ac.getContact());
        map.put("organizationCode", ac.getOrganizationCode());
        map.put("area", ac.getArea());
        map.put("address", ac.getAddress());
        map.put("applyTime", DateUtil.formatDateTime(ac.getApplyTime()));
        map.put("orgUrl", ac.getOrgUrl());
        map.put("email", ac.getEmail());
        map.put("isFreeze", ac.getIsFreeze()+"");
        return map;
    }

    /**
     * 同步承办者
     *
     * @param map
     */
    private void communicateContractorPSS(Map<String, String> map) throws Exception {

        /*文件生成，同步*/
        if (Objects.equals("file", ConfigUtil.getValue("pss-communicate-mode"))) {
            String spDir = ConfigUtil.getValue("spjg_folder");
            String uploadPath = ConfigUtil.getValue("stuff-path");
            // 组装文件夹
            String[] folderPath = new String[]{uploadPath, spDir, DateUtil.today(), SPJG_FOLDER};
            if (StringUtil.isNotEmpty(ConfigUtil.getValue("profile_envi"))
                    && ConfigUtil.getValue("profile_envi").startsWith("pre")) {
                folderPath = new String[]{uploadPath, spDir, ConfigUtil.getValue("envi"), DateUtil.today(), SPJG_FOLDER};
            }
            String dir = BcxinFileUtil.appendFolderPath(folderPath);
            String pssId = map.get("pssId");
            //打包承办者信息 文件名： _sprank_100021_t_20170101010101
            String fileName = BcxinFileUtil.packTimestamp(SPCFREE + pssId) + ".bcx";
            String detailUrl = dir + fileName;
            final File newFile = new File(detailUrl);

            String newOrgFileName = "";
            String orgUrl = map.get("orgUrl");
            if (StringUtil.isNotEmpty(orgUrl)) {
                /* 将承办者文件拷贝一份到同步目录下*/
                File orgFile = new File(orgUrl.substring(orgUrl.indexOf("path=") + 5));
                newOrgFileName = BcxinFileUtil.packTimestamp(SPCFREEFILE + pssId) + "." + Files.getFileExtension(orgFile.getName());
                File newOrgFile = new File(dir + newOrgFileName);
                Files.copy(orgFile, newOrgFile);
            }
            map.put("orgUrl",newOrgFileName);
            String result = JSON.toJSONString(map);
            try {
                byte[] buf = DESUtil.encrypt(result.getBytes("UTF-8"), GlobalConst.KEY);
                Files.write(buf, newFile);
            } catch (Exception ex) {
                ex.printStackTrace();
                log.error("生成承办者文件失败,PSSID" + pssId);
            }

        } else if (Objects.equals("auto", ConfigUtil.getValue("pss-communicate-mode"))) {
        }

    }
}
