package com.bcxin.sp.work.service.activity.extend;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.bcxin.sp.config.annotation.SysLog;
import com.bcxin.sp.work.dao.activity.approve.ApproveDetailMapper;
import com.bcxin.sp.work.dao.activity.approve.ApproveHistoryMapper;
import com.bcxin.sp.work.dao.activity.approve.ApproveMapper;
import com.bcxin.sp.work.dao.activity.approve.ApproveRoleMapper;
import com.bcxin.sp.work.dao.activity.base.ActivityMapper;
import com.bcxin.sp.work.dao.activity.extend.SyncTaskMapper;
import com.bcxin.sp.work.dao.activity.extend.WorkflowMapper;
import com.bcxin.sp.work.dao.sys.RegionMapper;
import com.bcxin.sp.work.dao.user.UserMapper;
import com.bcxin.sp.work.entity.domain.activity.approve.Approve;
import com.bcxin.sp.work.entity.domain.activity.approve.ApproveDetail;
import com.bcxin.sp.work.entity.domain.activity.approve.ApproveHistory;
import com.bcxin.sp.work.entity.domain.activity.approve.ApproveRole;
import com.bcxin.sp.work.entity.domain.activity.base.Activity;
import com.bcxin.sp.work.entity.domain.activity.expand.SyncTask;
import com.bcxin.sp.work.entity.domain.activity.expand.Workflow;
import com.bcxin.sp.work.entity.domain.sys.CSMsgTemplate;
import com.bcxin.sp.work.entity.domain.sys.SPMsgTemplate;
import com.bcxin.sp.work.entity.domain.sys.SlMsgTemplate;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.AjaxResult;
import com.bcxin.sp.work.entity.dto.GradeRankUtil;
import com.bcxin.sp.work.entity.dto.SyncTaskDTO;
import com.bcxin.sp.work.service.activity.approve.ApproveFileService;
import com.bcxin.sp.work.service.activity.approve.ApproveService;
import com.bcxin.sp.work.service.activity.base.ActivityService;
import com.bcxin.sp.work.service.common.NotifyService;
import com.bcxin.sp.work.service.sys.CSMsgTemplateService;
import com.bcxin.sp.work.service.sys.SPMsgTemplateService;
import com.bcxin.sp.work.service.sys.SlMsgTemplateService;
import com.bcxin.sp.work.service.third.nxzw.NxZwSbService;
import com.bcxin.sp.work.util.BcxinDateUtil;
import com.bcxin.sp.work.util.BcxinFileUtil;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.GlobalConst;
import com.bcxin.sp.work.util.DESUtil;
import com.bcxin.sp.work.util.StringUtil;
import com.bcxin.sp.work.util.sys.ConfigUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.io.Files;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
@Transactional
@Slf4j
public class WorkflowService {

    @Resource
    private WorkflowMapper workflowDao;

    @Resource
    private UserMapper userDao;

    @Resource
    private ApproveService approveService;

    @Resource
    private ActivityService activityService;

    @Resource
    private SPMsgTemplateService spMsgTemplateService;

    @Resource
    private CSMsgTemplateService csMsgTemplateService;

    @Resource
    private SlMsgTemplateService slService;

    @Resource
    private ActivityMapper activityMapper;

    @Resource
    private ApproveMapper approveMapper;

    @Resource
    private SyncTaskMapper syncTaskMapper;

    @Resource
    private ApproveRoleMapper approveRoleMapper;

    @Resource
    private ApproveHistoryMapper approveHistoryMapper;

    @Resource
    private ApproveFileService approveFileService;

    @Resource
    private NotifyService notifyService;

    @Resource
    private ApproveDetailMapper approveDetailMapper;

    @Resource
    private UserMapper userMapper;
    @Resource
    private NxZwSbService nxZwSbService;


    /**
     * 审批活动结果，文件标识
     */
    private final static String SPAPRR = "_spaprr_";

    /**
     * 审批意见反馈，文件标识
     */
    private final static String SPFEEDBACK = "_spfeedback_";

    /**
     * 审批等级评定内容，文件标识
     */
    private final static String SPRANK = "_sprank_";

    /**
     * 审批等级评定重新评级，文件标识
     */
    private final static String SPRERANK = "_sprerank_";

    /**
     * 审批等级评定文件，文件标识
     */
    private final static String SPRANKFILE = "_sprankfile_";

    /**
     * 同步审批结果文件夹名称
     */
    private final static String SPJG_FOLDER = "spjg";


    public List<User> findNextUser(User user, Activity activity) {
        /* 非指定人进行审批 */
        if (!GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_appointApprove"))) {
            return Lists.newArrayList();
        }

        //已经审批的次数
        List<Approve> history = approveMapper.selectList(
                new EntityWrapper<Approve>()
                        .eq("activityId", activity.getId())
                        .eq("approveType", DictConst.APPROVETYPE_SP)
                        .eq("logicDel", DictConst.N));
        //审批需要的次数
        List<Workflow> workflowList = workflowDao.findByVersion(activity.getWorkflowVersion());
        //如果需要的审批次数等于已经审批的次数加一，则不需要下一级，说明当前已经是最后一级
        if (history.size() == 0) {
            if (workflowList.size() == 1) {
                return Lists.newArrayList();
            }
        } else {
            if (workflowList.size() == history.size()) {
                return Lists.newArrayList();
            }
        }

        int currentSeq = CollectionUtil.isEmpty(history) ? 1 : history.get(history.size() - 1).getSeq();
        Workflow next = null;
        String role = null;
        List<User> users = Lists.newArrayList();

        for (Workflow active : workflowList) {
            if (active.getSeq() > currentSeq) {
                next = active;
                break;
            }
        }
        if (null != next) {
            role = next.getRole();
        }
        if (null != role) {
            users = userDao.findByRegionAndRole(user.getProvince(), user.getCity(), user.getArea(), role);
        }
        return users;

    }

    @SysLog("创建流程")
    public Workflow createOrUpdate(Workflow flow) {
        /* 修改流程 */
        if (flow.getId() != null) {
            Workflow workflow = workflowDao.selectById(flow.getId());
            if (workflow == null) {
                return flow;
            }
            flow.setSeq(workflow.getSeq());
            flow.setActive(DictConst.Y);
            flow.setUpdateTime(DateUtil.date());
            workflowDao.updateById(flow);
        } else { /* 创建一个新的流程 */
            List<Workflow> workflowList = workflowDao.findActive(flow.getProvince(), flow.getCity(), flow.getArea());
            //没有审批流程
            if (CollectionUtil.isEmpty(workflowList)) {
                String version = StringUtil.isEmpty(flow.getArea()) ? flow.getCity() : flow.getArea();
                flow.setSeq(1);
                flow.setVersion(version);
                flow.setActive(DictConst.Y);
                flow.setUpdateTime(new Date());
                flow.setAddTime(new Date());
                workflowDao.insert(flow);
                fixActivityWorkflow(flow);
            } else {//有审批流程
                int seq = workflowDao.findMaxSeq(flow.getProvince(), flow.getCity(), flow.getArea()) + 1;
                Workflow firstFlow = workflowList.get(0);

                flow.setSeq(seq);
                flow.setVersion(firstFlow.getVersion());
                flow.setActive(DictConst.Y);
                flow.setAddTime(new Date());
                flow.setUpdateTime(new Date());
                workflowDao.insert(flow);

                /* 修复未审批的活动流程 */
                List<Activity> activityList = activityMapper.findNeedRepairWorkflowVersion(flow.getProvince(), flow.getCity(), flow.getArea());
                for (Activity activity : activityList) {
                    activityService.setWorkFlow(activity);
                    activityMapper.updateById(activity);
                }
            }
        }
        return flow;
    }

    /**
     * 修复活动的审批流程
     *
     * @param workflow
     */
    private void fixActivityWorkflow(Workflow workflow) {
        String province = workflow.getProvince();
        List<Activity> activityList = activityMapper.findNeedWorkflowActivity(province);
        if (CollectionUtil.isEmpty(activityList)) {
            return;
        }
        for (Activity activity : activityList) {
            boolean updateFlag = activityService.setWorkFlow(activity);
            if (updateFlag) {
                activityMapper.updateById(activity);
            }
        }
    }


    public Workflow delete(Workflow deleteFlow) {
        Workflow workflow = workflowDao.selectById(deleteFlow.getId());
        if (workflow == null) {
            return deleteFlow;
        }
        List<Workflow> workflows = workflowDao.findByVersion(workflow.getVersion());
        int newSeq = 1;
        for (Workflow newflow : workflows) {
            if (newflow.getId().equals(deleteFlow.getId())) {
                workflowDao.deleteById(deleteFlow);
                continue;
            }
            newflow.setSeq(newSeq);
            newflow.setActive(DictConst.Y);
            newflow.setUpdateTime(new Date());
            workflowDao.updateById(newflow);
            newSeq++;
        }
        return deleteFlow;
    }

    public long findCountOfWork(String province, String city, String area) {
        return workflowDao.findCountOfWork(province, city, area);
    }

    /**
     * 提交下一个审批
     *
     * @param approve       当前审批环节
     * @param nextApprover
     * @param rejectContent
     */
    @SysLog("活动审批")
    public AjaxResult moveNext(User user, Approve approve, Long nextApprover, String rejectContent) {
        if (StringUtil.isEmpty(approve.getApprove())) {
            throw new RuntimeException("failed to get approved status! " + approve);
        }

        boolean canApprove = checkApproveUser(approve.getActivityId(),user);
        if (!canApprove) {
            return AjaxResult.fail("当前用户无审批权限，请刷新列表后重试");
        }

        String switchFullApprove = ConfigUtil.getValue("switch_fullApprove");
        Activity activity = activityMapper.selectById(approve.getActivityId());
        List<Approve> completeApproval = approveMapper.findRecordByActivityIdAndType(approve.getActivityId(), DictConst.APPROVETYPE_SP);

        String version = activity.getWorkflowVersion();
        List<Workflow> workflowList = workflowDao.findByVersion(version);
        Workflow nextFlow = workflowDao.findByVersionAndSeq(version, completeApproval.size() + 2);

        if (CollectionUtil.isNotEmpty(completeApproval)) {
            //第N次审批
            int approvalLevel = workflowList.size(); //审批级数
            int currentSeq = workflowList.get(completeApproval.size()).getSeq(); //当前审批序号
            int nextSeq = currentSeq + 1; //下一个审批序号
            int lastSeq = workflowList.get(approvalLevel - 1).getSeq();

            String approveStatus = approve.getApprove();
            /* 需要进行全部审批，意味着审批最后一级才是最终的审批 */
            if (GlobalConst.SWITCH_ON.equals(switchFullApprove)) {
                /* 到最后一级审批了 */
                if (currentSeq == lastSeq) {
                    /* 当前的审批状态就是最终的状态 */
                } else {
                    approveStatus = DictConst.STATUS_IN;
                }
            }

            if (DictConst.STATUS_NO.equals(approve.getApprove())) { //如果审批不通过，则全部终止
                approve.setSeq(currentSeq);
                approve.setUpdateTime(new Date());
                approve.setApproveType(DictConst.APPROVETYPE_SP);
                approveService.save(approve);

                activity.setApproveStatus(approveStatus);
                activity.setSync(DictConst.N);
                activity.setRejectContent(rejectContent);
                activityMapper.updateById(activity);

                /* 添加审批历史记录 */
                ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
                String elapsed = "";
                if (last != null) {
                    elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
                }
                ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
                ApproveHistory history = new ApproveHistory();
                history.setIsReset(DictConst.N);
                history.setApproveType(DictConst.APPROVETYPE_SP);
                history.setActivityId(activity.getId());
                history.setApproveId(approve.getId());
                history.setApprove(DictConst.STATUS_NO);
                history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
                history.setOpinion(rejectContent);
                history.setElapsed(elapsed);
                history.setFinish(DictConst.Y);
                approveHistoryMapper.insert(history);

                if (DictConst.STATUS_IN.equals(approveStatus)) {
                    /* 指定审批用户 */
                    activity.setNextApproveRole(nextFlow.getRole());
                    if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_appointApprove"))) {
                        activity.setNextApproveUser(nextApprover.toString());
                    }
                    activityMapper.updateById(activity);
                } else {
                    /* 生成文书 */
                    approveFileService.generatexkjds(DictConst.STATUS_NO, activity);
                    /* 消息通知 */
                    notifyService.spbtg(activity, user);
                    communicateApprovePSS(activity, false);
                    new Thread(() -> {
                        nxZwSbService.receive(activity.getId());
                    }).start();
                }
            } else if (DictConst.STATUS_REJECT.equals(approve.getApprove())) { //如果审批退回，则全部终止
                approve.setSeq(currentSeq);
                approve.setUpdateTime(new Date());
                approve.setApproveType(DictConst.APPROVETYPE_SP);
                approveService.save(approve);

                activity.setRejectContent(rejectContent);
                activity.setApproveStatus(approveStatus);
                activity.setSync(DictConst.N);
                activityMapper.updateById(activity);

                /* 添加审批历史记录 */
                ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
                String elapsed = "";
                if (last != null) {
                    elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
                }
                ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
                ApproveHistory history = new ApproveHistory();
                history.setIsReset(DictConst.N);
                history.setApproveType(DictConst.APPROVETYPE_SP);
                history.setActivityId(activity.getId());
                history.setApproveId(approve.getId());
                history.setApprove(DictConst.STATUS_REJECT);
                history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
                history.setOpinion(rejectContent);
                history.setElapsed(elapsed);
                approveHistoryMapper.insert(history);

                /* 不是审批中状态，才可以进行生成操作 */
                if (DictConst.STATUS_IN.equals(approveStatus)) {
                    /* 指定审批用户 */
                    activity.setNextApproveRole(nextFlow.getRole());
                    if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_appointApprove"))) {
                        activity.setNextApproveUser(nextApprover.toString());
                    }
                    activityMapper.updateById(activity);
                } else {
                    /* 消息通知 */
                    notifyService.spth(activity, user);
                    communicateApprovePSS(activity, false);
                }
            } else {
                if (currentSeq == lastSeq) { //说明当前审批已经是最后的阶段
                    approve.setSeq(currentSeq);
                    approve.setUpdateTime(new Date());
                    approve.setApproveType(DictConst.APPROVETYPE_SP);
                    approveService.save(approve);

                    activity.setApproveStatus(approveStatus);
                    activity.setSync(DictConst.N);
                    activityMapper.updateById(activity);

                    /* 添加审批历史记录 */
                    ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
                    String elapsed = "";
                    if (last != null) {
                        elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
                    }
                    ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
                    ApproveHistory history = new ApproveHistory();
                    history.setIsReset(DictConst.N);
                    history.setApproveType(DictConst.APPROVETYPE_SP);
                    history.setActivityId(activity.getId());
                    history.setApproveId(approve.getId());
                    history.setApprove(DictConst.STATUS_YES);
                    history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
                    history.setElapsed(elapsed);
                    history.setFinish(DictConst.Y);
                    approveHistoryMapper.insert(history);

                    /* 生成文书 */
                    approveFileService.generatexkjds(DictConst.STATUS_YES, activity);
                    /* 消息通知 */
                    notifyService.tgsp(activity, user);
                    /* 同步PSS */
                    communicateApprovePSS(activity, false);
                    new Thread(() -> {
                        nxZwSbService.receive(activity.getId());
                    }).start();
                } else {
                    //先创建这一条，再更新状态，如果审批通过，则再创建下一条
                    approve.setUpdateTime(new Date());
                    approve.setApproveType(DictConst.APPROVETYPE_SP);
                    approveService.save(approve);

                    /* 指定审批用户 */
                    activity.setNextApproveRole(nextFlow.getRole());
                    if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_appointApprove"))) {
                        activity.setNextApproveUser(nextApprover.toString());
                    }
                    activity.setApproveStatus(approveStatus);
                    activityMapper.updateById(activity);

                    /* 添加审批历史记录 */
                    ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
                    String elapsed = "";
                    if (last != null) {
                        elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
                    }
                    ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
                    ApproveHistory history = new ApproveHistory();
                    history.setIsReset(DictConst.N);
                    history.setApproveType(DictConst.APPROVETYPE_SP);
                    history.setActivityId(activity.getId());
                    history.setApproveId(approve.getId());
                    history.setApprove(DictConst.STATUS_YES);
                    history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
                    history.setElapsed(elapsed);
                    approveHistoryMapper.insert(history);

                }
            }
        } else {
            //第一次审批
            int approvalLevel = workflowList.size(); //审批级数
            if (CollectionUtil.isEmpty(workflowList)) {
                return AjaxResult.fail("未配置审批流程");
            }
            int currentSeq = workflowList.get(0).getSeq(); //当前审批序号
            int nextSeq = currentSeq + 1;//下一个审批序号
            int lastSeq = workflowList.get(approvalLevel - 1).getSeq();

            String approveStatus = approve.getApprove();
            /* 需要进行全部审批，意味着审批最后一级才是最终的审批 */
            if (GlobalConst.SWITCH_ON.equals(switchFullApprove)) {
                /* 到最后一级审批了 */
                if (currentSeq == lastSeq) {
                    /* 当前的审批状态就是最终的状态 */
                } else {
                    approveStatus = DictConst.STATUS_IN;
                }
            }

            if (DictConst.STATUS_NO.equals(approve.getApprove())) { //如果审批不通过，则全部终止
                approve.setSeq(currentSeq);
                approve.setAddTime(new Date());
                approve.setUpdateTime(new Date());
                approve.setApproveType(DictConst.APPROVETYPE_SP);
                approveService.save(approve);

                activity.setApproveStatus(approveStatus);
                activity.setSync(DictConst.N);
                activity.setRejectContent(rejectContent);
                activityMapper.updateById(activity);

                /* 添加审批历史记录 */
                ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
                String elapsed = "";
                if (last != null) {
                    elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
                }
                ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
                ApproveHistory history = new ApproveHistory();
                history.setIsReset(DictConst.N);
                history.setApproveType(DictConst.APPROVETYPE_SP);
                history.setActivityId(activity.getId());
                history.setApproveId(approve.getId());
                history.setApprove(DictConst.STATUS_NO);
                history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
                history.setOpinion(rejectContent);
                history.setElapsed(elapsed);
                history.setFinish(DictConst.Y);
                approveHistoryMapper.insert(history);

                /* 不是审批中状态，才可以进行生成操作 */
                if (DictConst.STATUS_IN.equals(approveStatus)) {
                    /* 指定审批用户 */
                    activity.setNextApproveRole(nextFlow.getRole());
                    if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_appointApprove"))) {
                        activity.setNextApproveUser(nextApprover.toString());
                    }
                    activityMapper.updateById(activity);
                } else {
                    /* 生成文书 */
                    approveFileService.generatexkjds(DictConst.STATUS_NO, activity);
                    /* 消息通知 */
                    notifyService.spbtg(activity, user);
                    /* 同步PSS */
                    communicateApprovePSS(activity, false);
                }

            } else if (DictConst.STATUS_REJECT.equals(approve.getApprove())) {//如果审批退回，则全部终止
                approve.setSeq(currentSeq);
                approve.setAddTime(activity.getAddTime());
                approve.setUpdateTime(activity.getAddTime());
                approve.setApproveType(DictConst.APPROVETYPE_SP);
                approveService.save(approve);

                activity.setRejectContent(rejectContent);
                activity.setApproveStatus(approveStatus);
                activity.setSync(DictConst.N);
                activityMapper.updateById(activity);

                /* 添加审批历史记录 */
                ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
                String elapsed = "";
                if (last != null) {
                    elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
                }
                ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
                ApproveHistory history = new ApproveHistory();
                history.setIsReset(DictConst.N);
                history.setApproveType(DictConst.APPROVETYPE_SP);
                history.setActivityId(activity.getId());
                history.setApproveId(approve.getId());
                history.setApprove(DictConst.STATUS_REJECT);
                history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
                history.setElapsed(elapsed);
                history.setOpinion(rejectContent);
                approveHistoryMapper.insert(history);

                /* 不是审批中状态，才可以进行生成操作 */
                if (!DictConst.STATUS_IN.equals(approveStatus)) {
                    /* 指定审批用户 */
                    activity.setNextApproveRole(nextFlow.getRole());
                    if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_appointApprove"))) {
                        activity.setNextApproveUser(nextApprover.toString());
                    }
                    activityMapper.updateById(activity);
                } else {
                    /* 消息通知 */
                    notifyService.spth(activity, user);
                    communicateApprovePSS(activity, false);
                }
            } else {
                if (currentSeq == lastSeq) { //只有一级审批，则终止
                    approve.setSeq(currentSeq);
                    approve.setAddTime(new Date());
                    approve.setUpdateTime(new Date());
                    approve.setApproveType(DictConst.APPROVETYPE_SP);
                    approveService.save(approve);

                    activity.setApproveStatus(approveStatus);
                    activity.setSync(DictConst.N);
                    activityMapper.updateById(activity);

                    /* 添加审批历史记录 */
                    ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
                    String elapsed = "";
                    if (last != null) {
                        elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
                    }
                    ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
                    ApproveHistory history = new ApproveHistory();
                    history.setIsReset(DictConst.N);
                    history.setApproveType(DictConst.APPROVETYPE_SP);
                    history.setActivityId(activity.getId());
                    history.setApproveId(approve.getId());
                    history.setApprove(DictConst.STATUS_YES);
                    history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
                    history.setElapsed(elapsed);
                    history.setOpinion(rejectContent);
                    history.setFinish(DictConst.Y);
                    approveHistoryMapper.insert(history);

                    /* 生成文书 */
                    approveFileService.generatexkjds(DictConst.STATUS_YES, activity);
                    /* 消息通知 */
                    notifyService.tgsp(activity, user);

                    communicateApprovePSS(activity, false);
                } else {
                    //先创建第一条，再更新状态，如果审批通过，则再创建下一条
                    approve.setSeq(currentSeq);
                    approve.setAddTime(new Date());
                    approve.setUpdateTime(new Date());
                    approve.setApproveType(DictConst.APPROVETYPE_SP);
                    approveService.save(approve);

                    /* 指定审批用户 */
                    activity.setNextApproveRole(nextFlow.getRole());
                    if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_appointApprove"))) {
                        activity.setNextApproveUser(nextApprover.toString());
                    }
                    activity.setApproveStatus(approveStatus);
                    activityMapper.updateById(activity);

                    /* 添加审批历史记录 */
                    ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
                    String elapsed = "";
                    if (last != null) {
                        elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
                    }
                    ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
                    ApproveHistory history = new ApproveHistory();
                    history.setIsReset(DictConst.N);
                    history.setApproveType(DictConst.APPROVETYPE_SP);
                    history.setActivityId(activity.getId());
                    history.setApproveId(approve.getId());
                    history.setApprove(DictConst.STATUS_YES);
                    history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
                    history.setElapsed(elapsed);
                    history.setOpinion(rejectContent);
                    approveHistoryMapper.insert(history);

                }
            }
        }
        //生成安全许可审批表
        approveFileService.generateaqxkspb(activity);

        return AjaxResult.success("");
    }


    /**
     * 同步审批状态
     *
     * @param activity
     * @param isDirectSync 是否直接同步，优先级最高
     */
    private void communicateApprovePSS(Activity activity, boolean isDirectSync) {
        // 不是直接同步，需要验证项目
        if (!isDirectSync) {
            //如果是政府类项目且是备案项目，则不同步
            if (DictConst.Y.equals(activity.getIsBackUp()) && Objects.equals(activity.getIsBackUp(), DictConst.Y)) {
                return;
            }
        }

        /*文件生成，同步*/
        if (Objects.equals("file", ConfigUtil.getValue("pss-communicate-mode"))) {
            String spDir = ConfigUtil.getValue("spjg_folder");
            String uploadPath = ConfigUtil.getValue("stuff-path");
            // 组装文件夹
            String[] folderPath = new String[]{uploadPath, spDir, DateUtil.today(), SPJG_FOLDER};
            if (StringUtil.isNotEmpty(ConfigUtil.getValue("profile_envi"))
                    && ConfigUtil.getValue("profile_envi").startsWith("pre")) {
                folderPath = new String[]{uploadPath, spDir, ConfigUtil.getValue("envi"), DateUtil.today(), SPJG_FOLDER};
            }
            String dir = BcxinFileUtil.appendFolderPath(folderPath);
            //打包活动信息 文件名： _acd_100021_t_20170101010101
            String fileName = BcxinFileUtil.packTimestamp(SPAPRR + activity.getPssId()) + ".bcx";
            String detailUrl = dir + fileName;
            final File newFile = new File(detailUrl);

            List<Map<String, String>> reMapList = new ArrayList<>();
            reMapList.add(generateApproveFileMap(activity));
            String result = JSON.toJSONString(reMapList);
            try {
                byte[] buf = DESUtil.encrypt(result.getBytes("UTF-8"), GlobalConst.KEY);
                Files.write(buf, newFile);
            } catch (Exception ex) {
                ex.printStackTrace();
                log.error("生成审批结果文件失败,活动" + activity.getPssId());
            }

        } else if (Objects.equals("auto", ConfigUtil.getValue("pss-communicate-mode"))) {
        }

    }

    /**
     * 同步安全等级评定
     *
     * @param activity
     */
    private void communicateGradePSS(Activity activity, Approve approve) throws Exception {
        //如果是政府类项目且是备案项目，则不同步
        if (DictConst.Y.equals(activity.getIsBackUp()) && Objects.equals(activity.getIsBackUp(), DictConst.Y)) {
            return;
        }

        /*文件生成，同步*/
        if (Objects.equals("file", ConfigUtil.getValue("pss-communicate-mode"))) {
            String spDir = ConfigUtil.getValue("spjg_folder");
            String uploadPath = ConfigUtil.getValue("stuff-path");
            // 组装文件夹
            String[] folderPath = new String[]{uploadPath, spDir, DateUtil.today(), SPJG_FOLDER};
            if (StringUtil.isNotEmpty(ConfigUtil.getValue("profile_envi"))
                    && ConfigUtil.getValue("profile_envi").startsWith("pre")) {
                folderPath = new String[]{uploadPath, spDir, ConfigUtil.getValue("envi"), DateUtil.today(), SPJG_FOLDER};
            }
            String dir = BcxinFileUtil.appendFolderPath(folderPath);
            //打包活动信息 文件名： _sprank_100021_t_20170101010101
            String fileName = BcxinFileUtil.packTimestamp(SPRANK + activity.getPssId()) + ".bcx";
            String detailUrl = dir + fileName;
            final File newFile = new File(detailUrl);

            String newRankFileName = "";
            if (StringUtil.isNotEmpty(approve.getRankFile())) {
                /* 将等级评定文件拷贝一份到同步目录下*/
                File rankFile = new File(approve.getRankFile().substring(approve.getRankFile().indexOf("path=") + 5));
                newRankFileName = BcxinFileUtil.packTimestamp(SPRANKFILE + activity.getPssId()) + "." + Files.getFileExtension(rankFile.getName());
                File newRankFile = new File(dir + newRankFileName);
                Files.copy(rankFile, newRankFile);
            }

            Map<String, String> gradeMap = generateGradeFileMap(activity, approve, newRankFileName);
            String result = JSON.toJSONString(gradeMap);
            try {
                byte[] buf = DESUtil.encrypt(result.getBytes("UTF-8"), GlobalConst.KEY);
                Files.write(buf, newFile);
            } catch (Exception ex) {
                ex.printStackTrace();
                log.error("生成审批意见文件失败,活动" + activity.getPssId());
            }

        } else if (Objects.equals("auto", ConfigUtil.getValue("pss-communicate-mode"))) {
        }

    }

    /**
     * 同步安全等级评定，重新评级
     *
     * @param activity
     */
    private void communicateReGradePSS(Activity activity) {
        //如果是政府类项目且是备案项目，则不同步
        if (DictConst.Y.equals(activity.getIsBackUp()) && Objects.equals(activity.getIsBackUp(), DictConst.Y)) {
            return;
        }
        /*文件生成，同步*/
        if (Objects.equals("file", ConfigUtil.getValue("pss-communicate-mode"))) {
            String spDir = ConfigUtil.getValue("spjg_folder");
            String uploadPath = ConfigUtil.getValue("stuff-path");
            // 组装文件夹
            String[] folderPath = new String[]{uploadPath, spDir, DateUtil.today(), SPJG_FOLDER};
            if (StringUtil.isNotEmpty(ConfigUtil.getValue("profile_envi"))
                    && ConfigUtil.getValue("profile_envi").startsWith("pre")) {
                folderPath = new String[]{uploadPath, spDir, ConfigUtil.getValue("envi"), DateUtil.today(), SPJG_FOLDER};
            }
            String dir = BcxinFileUtil.appendFolderPath(folderPath);
            //打包活动信息 文件名： _sprerank_100021_t_20170101010101
            String fileName = BcxinFileUtil.packTimestamp(SPRERANK + activity.getPssId()) + ".bcx";
            String detailUrl = dir + fileName;
            final File newFile = new File(detailUrl);

            Map<String, String> gradeMap = generateReGradeFileMap(activity);
            String result = JSON.toJSONString(gradeMap);
            try {
                byte[] buf = DESUtil.encrypt(result.getBytes("UTF-8"), GlobalConst.KEY);
                Files.write(buf, newFile);
            } catch (Exception ex) {
                ex.printStackTrace();
                log.error("生成审批意见文件重新评级失败,活动" + activity.getPssId());
            }

        } else if (Objects.equals("auto", ConfigUtil.getValue("pss-communicate-mode"))) {
        }

    }


    /**
     * 同步审批意见
     *
     * @param activity
     */
    private void communicateFeedbackPSS(Activity activity, String feedbackType, String comments) {
        //如果是政府类项目且是备案项目，则不同步
        if (DictConst.Y.equals(activity.getIsBackUp()) && Objects.equals(activity.getIsBackUp(), DictConst.Y)) {
            return;
        }

        /*文件生成，同步*/
        if (Objects.equals("file", ConfigUtil.getValue("pss-communicate-mode"))) {
            String spDir = ConfigUtil.getValue("spjg_folder");
            String uploadPath = ConfigUtil.getValue("stuff-path");
            // 组装文件夹
            String[] folderPath = new String[]{uploadPath, spDir, DateUtil.today(), SPJG_FOLDER};
            if (StringUtil.isNotEmpty(ConfigUtil.getValue("profile_envi"))
                    && ConfigUtil.getValue("profile_envi").startsWith("pre")) {
                folderPath = new String[]{uploadPath, spDir, ConfigUtil.getValue("envi"), DateUtil.today(), SPJG_FOLDER};
            }
            String dir = BcxinFileUtil.appendFolderPath(folderPath);
            //打包活动信息 文件名： _spfeedback_100021_t_20170101010101
            String fileName = BcxinFileUtil.packTimestamp(SPFEEDBACK + activity.getPssId()) + ".bcx";
            String detailUrl = dir + fileName;
            final File newFile = new File(detailUrl);

            Map<String, String> gradeMap = generateFeedbackFileMap(activity, feedbackType, comments);
            String result = JSON.toJSONString(gradeMap);
            try {
                byte[] buf = DESUtil.encrypt(result.getBytes("UTF-8"), GlobalConst.KEY);
                Files.write(buf, newFile);
            } catch (Exception ex) {
                ex.printStackTrace();
                log.error("生成审批意见文件失败,活动" + activity.getPssId());
            }

        } else if (Objects.equals("auto", ConfigUtil.getValue("pss-communicate-mode"))) {
        }

    }

    public Workflow findById(long id) {
        return workflowDao.selectById(id);
    }

    public Workflow findWorkFlow(int seq, Long activityId) {
        return workflowDao.findWorkFlow(seq, activityId);
    }

    public Integer findMaxSeq(String province, String city, String area) {
        return workflowDao.findMaxSeq(province, city, area);
    }

    public List<Workflow> findAll() {
        return workflowDao.selectList(new EntityWrapper<Workflow>());
    }

    public List<Workflow> findActive(String province, String city, String area) {
        return workflowDao.findActive(province, city, area);
    }


    /**
     * 初审
     *
     * @param approve
     * @param approveStatus
     * @param rejectContent
     */
    @SysLog("活动初审")
    public AjaxResult primaryApprove(User user, Approve approve, String approveStatus, String rejectContent) {
        if (StringUtil.isEmpty(approveStatus)) {
            throw new RuntimeException("failed to get approved status! " + approve);
        }
        boolean canApprove = checkApproveUser(approve.getActivityId(),user);
        if (!canApprove) {
            return AjaxResult.fail("当前用户无审批权限，请刷新列表后重试");
        }
        Activity activity = activityMapper.selectById(approve.getActivityId());
        // 设置项目未转交项目，主要用于发送短信，验证是否同步消息标识
        activity.setIsTransfer(DictConst.N);
        if (Objects.equals(approveStatus, DictConst.STATUS_REJECT)) {
            activity.setApproveStatus(DictConst.STATUS_CSXG);
            activity.setRejectContent(rejectContent);
            activity.setSync(DictConst.N);
            /* 更新下一个审批角色为审批第一级 */
            Workflow nextFlow = workflowDao.findByVersionAndSeq(activity.getWorkflowVersion(), 1);
            activity.setNextApproveRole(nextFlow.getRole());
            activityMapper.updateById(activity);

            /* 增加审批记录 */
            approve.setApproveType(DictConst.APPROVETYPE_CS);
            approve.setAddTime(new Date());
            approve.setUpdateTime(new Date());
            approve.setUserId(user.getId());
            approve.setUpdater(user.getRealName());
            approve.setActivityId(activity.getId());
            approve.setSeq(1);
            approve.setApprove(DictConst.STATUS_REJECT);
            approveMapper.insert(approve);
            //审批详情
            approveDetailMapper.deleteById(approve.getId());
            for (ApproveDetail detail : approve.getDetails()) {
                detail.setApproveId(approve.getId());
                approveDetailMapper.insert(detail);
            }

            /* 添加审批历史记录 */
            ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
            String elapsed = "";
            if (last != null) {
                elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
            }
            ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
            ApproveHistory history = new ApproveHistory();
            history.setIsReset(DictConst.N);
            history.setApproveType(DictConst.APPROVETYPE_CS);
            history.setActivityId(activity.getId());
            history.setApproveId(approve.getId());
            history.setApprove(DictConst.STATUS_REJECT);
            history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
            history.setOpinion(rejectContent);
            history.setElapsed(elapsed);
            approveHistoryMapper.insert(history);

            /* 消息通知 */
            notifyService.csth(activity, user);

            /* 审批结果同步 */
            communicateApprovePSS(activity, false);

        } else if (Objects.equals(approveStatus, DictConst.STATUS_YES)) {
            activity.setApproveStatus(DictConst.STATUS_CSTG);
            activity.setSync(DictConst.N);
            /* 更新下一个审批角色为审批第一级 */
            Workflow nextFlow = workflowDao.findByVersionAndSeq(activity.getWorkflowVersion(), 1);
            activity.setNextApproveRole(nextFlow.getRole());
            activityMapper.updateById(activity);

            /* 增加审批记录 */
            approve.setApproveType(DictConst.APPROVETYPE_CS);
            approve.setAddTime(new Date());
            approve.setUpdateTime(new Date());
            approve.setUserId(user.getId());
            approve.setUpdater(user.getRealName());
            approve.setActivityId(activity.getId());
            approve.setSeq(1);
            approve.setApprove(DictConst.STATUS_CSTG);
            approveMapper.insert(approve);
            //审批详情
            approveDetailMapper.deleteById(approve.getId());
            for (ApproveDetail detail : approve.getDetails()) {
                detail.setApproveId(approve.getId());
                approveDetailMapper.insert(detail);
            }

            /* 添加审批历史记录 */
            ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
            String elapsed = "";
            if (last != null) {
                elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
            }
            ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
            ApproveHistory history = new ApproveHistory();
            history.setIsReset(DictConst.N);
            history.setApproveType(DictConst.APPROVETYPE_CS);
            history.setActivityId(activity.getId());
            history.setApproveId(approve.getId());
            history.setApprove(DictConst.STATUS_YES);
            history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
            history.setOpinion(rejectContent);
            history.setElapsed(elapsed);
            history.setFinish(DictConst.Y);
            approveHistoryMapper.insert(history);

            /* 消息通知 */
            notifyService.tgcs(activity, user);

            /* 审批结果同步 */
            communicateApprovePSS(activity, false);

            /* 直接跳转受理中 */
            if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_redirectsl"))) {
                activity.setApproveStatus(DictConst.STATUS_SLZ);
                activityMapper.updateById(activity);

                /* 增加同步任务 */
                SyncTask syncTask = new SyncTask();
                syncTask.setActivityId(activity.getId());
                syncTask.setSyncTaskType(DictConst.SYNCTASKTYPE_CS);
                syncTask.setApproveStatus(DictConst.STATUS_CSTG);
                syncTask.setAddTime(new Date());
                syncTask.setSync(DictConst.N);
                syncTaskMapper.insert(syncTask);
            }
        }
        return AjaxResult.success("");
    }


    /**
     * 等级评定
     */
    @SysLog("活动等级评定-审批意见反馈")
    public void gradeFeedbackApprove(User user, Map<String, String> param) {
        String feedbackComment = param.get("feedbackComment");
        String feedbackType = param.get("grade_feedback_type");
        String activityId = param.get("grade_feedback_activityId");

        ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
        ApproveHistory history = new ApproveHistory();
        history.setIsReset(DictConst.N);
        history.setApproveType(feedbackType);
        history.setActivityId(Long.parseLong(activityId));
        history.setApprove("INFO");
        history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
        history.setOpinion(feedbackComment);
        history.setFinish(DictConst.N);
        approveHistoryMapper.insert(history);

        /* 审批意见反馈同步 */
        Activity activity = activityMapper.selectById(Long.parseLong(activityId));

        if ("on".equalsIgnoreCase(ConfigUtil.getValue("switch_zjzw20"))) {
            if ("330000".equals(activity.getProvince())) {
                if (DictConst.APPROVETYPE_YJ_BCCL.equals(feedbackType)) {
                    activityMapper.updateActivityByIsAddFiles(activity.getId(),null, DictConst.N);
                }
            }
        }
        communicateFeedbackPSS(activity, feedbackType, feedbackComment);
    }

    /**
     * 等级评定
     */
    @SysLog("活动等级评定")
    public AjaxResult gradeApprove(User user, Map<String, String> param) throws Exception {
        String gradeRank = param.get("gradeRank");
        String rankFile = param.get("grade_rankFile");
        String gradeRankComment = param.get("gradeRankComment");
        String activityId = param.get("grade_activityId");

        boolean canApprove = checkApproveUser(Long.parseLong(activityId),user);
        if (!canApprove) {
            return AjaxResult.fail("当前用户无审批权限，请刷新列表后重试");
        }

        String approveStatus = DictConst.STATUS_CSTG;
        String approveFinish = DictConst.Y;
        int currentLevel = 1;
        /* 自定义配置审批级别 */
        String level = ConfigUtil.getValue("approve_level");
        if (StringUtil.isNotEmpty(level)) {
            List<Approve> csList = approveMapper.findRecordByActivityIdAndType(Long.parseLong(activityId), DictConst.APPROVETYPE_CS);
            /* 还没到最后的审批级别 */
            if ((csList.size() + 1) != Integer.parseInt(level)) {
                approveStatus = DictConst.STATUS_CSZ;
                approveFinish = DictConst.N;
            }
            currentLevel = csList.size() + 1;
        }

        /* 修改活动信息 */
        Activity activity = activityMapper.selectById(activityId);
        activity.setIsTransfer(DictConst.N);
        activity.setApproveStatus(approveStatus);
        activity.setSync(DictConst.N);
        /* 如果是最后一级，则更新下一个审批人为第一级 */
        if (DictConst.STATUS_CSTG.equals(approveStatus)) {
            /* 更新下一个审批角色为审批第一级 */
            Workflow nextFlow = workflowDao.findByVersionAndSeq(activity.getWorkflowVersion(), 1);
            activity.setNextApproveRole(nextFlow.getRole());
        }
        /* 如果不是最后一级，则更新下一个审批人为下一级 */
        else {
            Workflow nextFlow = workflowDao.findByVersionAndSeq(activity.getWorkflowVersion(), currentLevel + 1);
            activity.setNextApproveRole(nextFlow.getRole());
        }
        activityMapper.updateById(activity);

        /* 增加审批记录 */
        Approve approve = new Approve();
        approve.setApproveType(DictConst.APPROVETYPE_CS);
        approve.setAddTime(new Date());
        approve.setUpdateTime(new Date());
        approve.setUserId(user.getId());
        approve.setUpdater(user.getRealName());
        approve.setActivityId(activity.getId());
        approve.setSeq(currentLevel);
        approve.setApprove(approveStatus);
        approve.setIsActive(DictConst.Y);
        /* 如果是最后一级了，他的意见需要复制上一级的意见 */
        if (DictConst.Y.equals(approveFinish)) {
            Approve lastApprove = approveMapper.findFirstRecordByActivityIdAndType(activity.getId(), DictConst.APPROVETYPE_CS);
            if (lastApprove != null) {
                if (StringUtil.isEmpty(gradeRank)) {
                    approve.setGradeRank(lastApprove.getGradeRank());
                } else {
                    approve.setGradeRank(gradeRank);
                }
                if (StringUtil.isEmpty(rankFile)) {
                    approve.setRankFile(lastApprove.getRankFile());
                } else {
                    approve.setRankFile(rankFile);
                }
                if (StringUtil.isEmpty(gradeRank)) {
                    approve.setRankComments(lastApprove.getRankComments());
                } else {
                    approve.setRankComments(gradeRankComment);
                }
            } else {
                approve.setGradeRank(gradeRank);
                approve.setRankFile(rankFile);
                approve.setRankComments(gradeRankComment);
            }
        } else {
            approve.setGradeRank(gradeRank);
            approve.setRankFile(rankFile);
            approve.setRankComments(gradeRankComment);
        }
        approveMapper.insert(approve);

        ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
        String elapsed = "";
        if (last != null) {
            elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
        }
        ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
        ApproveHistory history = new ApproveHistory();
        history.setIsReset(DictConst.N);
        history.setApproveType(DictConst.APPROVETYPE_CS);
        history.setActivityId(activity.getId());
        history.setApproveId(approve.getId());
        history.setApprove("GRADE");
        history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
        history.setOpinion("评定为：" + GradeRankUtil.getRankName(approve.getGradeRank()));
        history.setElapsed(elapsed);
        history.setFinish(approveFinish);
        approveHistoryMapper.insert(history);

        if (DictConst.STATUS_CSTG.equals(approveStatus)) {
            /* 消息通知 */
            notifyService.tgdjpd(activity, user);
            /* 审批结果同步 */
            communicateGradePSS(activity, approve);

            /* 增加同步任务 */
            SyncTask syncTask = new SyncTask();
            syncTask.setActivityId(activity.getId());
            syncTask.setSyncTaskType(DictConst.SYNCTASKTYPE_CS);
            syncTask.setApproveStatus(DictConst.STATUS_CSTG);
            syncTask.setAddTime(new Date());
            syncTask.setSync(DictConst.N);
            syncTaskMapper.insert(syncTask);
        }
        return AjaxResult.success("");
    }


    /**
     * 承办方等级评定重新评定
     */
    @SysLog("承办方等级评定重新评定")
    public AjaxResult contractorReGradeSave(User user, Map<String, String> param) {
        String gradeRank = param.get("contractor_re_gradeRank");
        String activityId = param.get("contractor_re_grade_activityId");

        boolean canApprove = checkApproveUser(Long.parseLong(activityId),user);
        if (!canApprove) {
            return AjaxResult.fail("当前用户无审批权限，请刷新列表后重试");
        }

        /* 修改活动信息 */
        Activity activity = activityMapper.selectById(activityId);
        /**
         * 同意重新调整
         */
        if ("1".equals(gradeRank)) {
            /* 修改为评定中 */
            activity.setIsTransfer(DictConst.N);
            activity.setApproveStatus(DictConst.STATUS_CSZ);
            activity.setSync(DictConst.N);
            /* 更新下一个审批角色为审批第一级 */
            Workflow nextFlow = workflowDao.findByVersionAndSeq(activity.getWorkflowVersion(), 1);
            activity.setNextApproveRole(nextFlow.getRole());
            activityMapper.updateById(activity);

            ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
            String elapsed = "";
            if (last != null) {
                elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
            }
            ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
            ApproveHistory history = new ApproveHistory();
            history.setIsReset(DictConst.N);
            history.setApproveType(DictConst.APPROVETYPE_CS);
            history.setActivityId(activity.getId());
            history.setApprove(DictConst.STATUS_YES);
            history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
            history.setOpinion("承办方重新评定安全管理等级申请为：同意调整");
            history.setElapsed(elapsed);
            history.setFinish(DictConst.N);
            approveHistoryMapper.insert(history);

            /* 重新评定等级，需要将之前的审批变成不是有效审批操作 */
            approveMapper.updateApproveActiveN(activity.getId(), DictConst.APPROVETYPE_CS);
        }
        /**
         * 不同意重新调整
         */
        else if ("2".equals(gradeRank)) {
            /* 修改为已评定 */
            activity.setIsTransfer(DictConst.N);
            activity.setApproveStatus(DictConst.STATUS_CSTG);
            activity.setSync(DictConst.N);
            /* 更新下一个审批角色为审批第一级 */
            Workflow nextFlow = workflowDao.findByVersionAndSeq(activity.getWorkflowVersion(), 1);
            activity.setNextApproveRole(nextFlow.getRole());
            activityMapper.updateById(activity);

            ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
            String elapsed = "";
            if (last != null) {
                elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
            }
            ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
            ApproveHistory history = new ApproveHistory();
            history.setIsReset(DictConst.N);
            history.setApproveType(DictConst.APPROVETYPE_CS);
            history.setActivityId(activity.getId());
            history.setApprove(DictConst.STATUS_NO);
            history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
            history.setOpinion("承办方重新评定安全管理等级申请为：不同意调整");
            history.setElapsed(elapsed);
            history.setFinish(DictConst.N);
            approveHistoryMapper.insert(history);
        }
        /* 同步结果给子站 */
        communicateReGradePSS(activity);
        return AjaxResult.success("");
    }


    /**
     * 承办方等级评定重新评定
     */
    @SysLog("公安等级评定重新评定")
    public void policeReGradeSave(User user, Map<String, String> param) {
        String activityId = param.get("police_re_grade_activityId");

        /* 修改活动信息 */
        Activity activity = activityMapper.selectById(activityId);
        /* 修改为评定中 */
        activity.setIsTransfer(DictConst.N);
        activity.setApproveStatus(DictConst.STATUS_CSZ);
        activity.setSync(DictConst.N);
        /* 更新下一个审批角色为审批第一级 */
        Workflow nextFlow = workflowDao.findByVersionAndSeq(activity.getWorkflowVersion(), 1);
        activity.setNextApproveRole(nextFlow.getRole());
        activityMapper.updateById(activity);

        ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
        String elapsed = "";
        if (last != null) {
            elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
        }
        ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
        ApproveHistory history = new ApproveHistory();
        history.setIsReset(DictConst.N);
        history.setApproveType(DictConst.APPROVETYPE_CS);
        history.setActivityId(activity.getId());
        history.setApprove(DictConst.STATUS_YES);
        history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
        history.setOpinion("公安重新评定安全管理等级");
        history.setElapsed(elapsed);
        history.setFinish(DictConst.N);
        approveHistoryMapper.insert(history);

        /* 重新评定等级，需要将之前的审批变成不是有效审批操作 */
        approveMapper.updateApproveActiveN(activity.getId(), DictConst.APPROVETYPE_CS);

        /* 同步结果给子站 */
        communicateReGradePSS(activity);
    }


    /**
     * 活动受理
     */
    @SysLog("活动受理-审批意见反馈")
    public void shouliFeedbackApprove(User user, Map<String, String> param) {
        String feedbackComment = param.get("shouliFeedbackComment");
        String activityId = param.get("shouli_feedback_activityId");

        ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
        ApproveHistory history = new ApproveHistory();
        history.setIsReset(DictConst.N);
        history.setApproveType(DictConst.APPROVETYPE_YJ_BCCL);
        history.setActivityId(Long.parseLong(activityId));
        history.setApprove("INFO");
        history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
        history.setOpinion(feedbackComment);
        history.setFinish(DictConst.N);
        approveHistoryMapper.insert(history);

        /* 审批意见反馈同步 */
        Activity activity = activityMapper.selectById(Long.parseLong(activityId));
        if ("on".equalsIgnoreCase(ConfigUtil.getValue("switch_zjzw20"))) {
            if ("330000".equals(activity.getProvince())) {
                activityMapper.updateActivityByIsAddFiles(activity.getId(),null, DictConst.N);
            }
        }
        communicateFeedbackPSS(activity, DictConst.APPROVETYPE_YJ_BCCL, feedbackComment);
    }


    /**
     * 受理
     *
     * @param bslContent
     * @param approveStatus
     */
    @SysLog("活动受理")
    public AjaxResult slApprove(User user, String approveStatus, String bslContent, String activityId) {
        if (StringUtil.isEmpty(approveStatus)) {
            throw new RuntimeException("failed to get approved status! " + approveStatus);
        }

        boolean canApprove = checkApproveUser(Long.parseLong(activityId),user);
        if (!canApprove) {
            return AjaxResult.fail("当前用户无审批权限，请刷新列表后重试");
        }

        Activity activity = activityMapper.selectById(Long.parseLong(activityId));

        /* 受理 */
        if (Objects.equals(approveStatus, DictConst.STATUS_SL)) {
            /* -------------------更新审批状态-------------------- */
            int currentLevel = 1;
            int nextLevel = currentLevel + 1;
            String approveFinish = DictConst.Y;
            /* 自定义配置审批级别 */
            String level = ConfigUtil.getValue("approve_level");
            if (StringUtil.isNotEmpty(level)) {
                List<Approve> slList = approveMapper.findRecordByActivityIdAndType(Long.parseLong(activityId), DictConst.APPROVETYPE_SL);
                /* 还没到最后的审批级别 */
                if ((slList.size() + 1) != Integer.parseInt(level)) {
                    approveStatus = DictConst.STATUS_SLZ;
                    approveFinish = DictConst.N;
                }
            }
            /* 如果是最后一级，则更新下一个审批人为第一级 */
            if (DictConst.STATUS_SL.equals(approveStatus)) {
                /* 更新下一个审批角色为审批第一级 */
                Workflow nextFlow = workflowDao.findByVersionAndSeq(activity.getWorkflowVersion(), 1);
                activity.setNextApproveRole(nextFlow.getRole());
            }
            /* 如果不是最后一级，则更新下一个审批人为下一级 */
            else {
                Workflow nextFlow = workflowDao.findByVersionAndSeq(activity.getWorkflowVersion(), nextLevel);
                activity.setNextApproveRole(nextFlow.getRole());
            }
            activity.setApproveStatus(approveStatus);
            activity.setSync(DictConst.N);
            activityMapper.updateById(activity);
            /* -------------------更新审批状态-------------------- */

            Approve approve = new Approve();
            /* 增加审批记录 */
            approve.setApproveType(DictConst.APPROVETYPE_SL);
            approve.setAddTime(new Date());
            approve.setUpdateTime(new Date());
            approve.setUserId(user.getId());
            approve.setUpdater(user.getRealName());
            approve.setActivityId(activity.getId());
            approve.setSeq(1);
            approve.setIsActive(DictConst.Y);
            approve.setApprove(DictConst.STATUS_SL);
            approveMapper.insert(approve);

            /* 添加审批历史记录 */
            ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
            String elapsed = "";
            if (last != null) {
                elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
            }
            ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
            ApproveHistory history = new ApproveHistory();
            history.setIsReset(DictConst.N);
            history.setApproveType(DictConst.APPROVETYPE_SL);
            history.setActivityId(activity.getId());
            history.setApproveId(approve.getId());
            history.setApprove(DictConst.STATUS_SL);
            history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
            history.setElapsed(elapsed);
            history.setFinish(approveFinish);
            approveHistoryMapper.insert(history);

            if (DictConst.STATUS_SL.equals(approveStatus)) {
                /* 生成文书 */
                approveFileService.generateslws(DictConst.STATUS_SL, activity, user);
                //2018-07-24 梁欣彤 受理通过生成
                //生成安全许可审批表
                approveFileService.generateaqxkspb(activity);
                /* 消息通知 */
                notifyService.acceptedNotify(activity, user);
                /* 审批结果同步 */
                communicateApprovePSS(activity, false);

                /* 直接跳转审批中 */
                if (GlobalConst.SWITCH_ON.equals(ConfigUtil.getValue("switch_redirectsp"))) {
                    activity.setApproveStatus(DictConst.STATUS_NONE);
                    activityMapper.updateById(activity);

                    /* 增加同步任务 */
                    SyncTask syncTask = new SyncTask();
                    syncTask.setActivityId(activity.getId());
                    syncTask.setSyncTaskType(DictConst.SYNCTASKTYPE_SL);
                    syncTask.setApproveStatus(DictConst.STATUS_SL);
                    syncTask.setAddTime(new Date());
                    syncTask.setSync(DictConst.N);
                    syncTaskMapper.insert(syncTask);
                }
            }
        }
        /* 不受理 */
        else if (Objects.equals(approveStatus, DictConst.STATUS_BSL)) {
            /* -------------------更新审批状态-------------------- */
            int currentLevel = 1;
            int nextLevel = currentLevel + 1;
            String approveFinish = DictConst.Y;
            /* 自定义配置审批级别 */
            String level = ConfigUtil.getValue("approve_level");
            if (StringUtil.isNotEmpty(level)) {
                List<Approve> csList = approveMapper.findRecordByActivityIdAndType(Long.parseLong(activityId), DictConst.APPROVETYPE_SL);
                /* 还没到最后的审批级别 */
                if ((csList.size() + 1) != Integer.parseInt(level)) {
                    approveStatus = DictConst.STATUS_SLZ;
                    approveFinish = DictConst.N;
                }
            }
            /* 如果是最后一级，则更新下一个审批人为第一级 */
            if (DictConst.STATUS_BSL.equals(approveStatus)) {
                /* 更新下一个审批角色为审批第一级 */
                Workflow nextFlow = workflowDao.findByVersionAndSeq(activity.getWorkflowVersion(), 1);
                activity.setNextApproveRole(nextFlow.getRole());
            }
            /* 如果不是最后一级，则更新下一个审批人为下一级 */
            else {
                Workflow nextFlow = workflowDao.findByVersionAndSeq(activity.getWorkflowVersion(), nextLevel);
                activity.setNextApproveRole(nextFlow.getRole());
            }
            activity.setApproveStatus(approveStatus);
            activity.setRejectContent(bslContent);
            activity.setSync(DictConst.N);
            activityMapper.updateById(activity);
            /* -------------------更新审批状态-------------------- */

            Approve approve = new Approve();
            /* 增加审批记录 */
            approve.setApproveType(DictConst.APPROVETYPE_SL);
            approve.setAddTime(new Date());
            approve.setUpdateTime(new Date());
            approve.setUserId(user.getId());
            approve.setUpdater(user.getRealName());
            approve.setActivityId(activity.getId());
            approve.setSeq(1);
            approve.setApprove(DictConst.STATUS_BSL);
            approveMapper.insert(approve);

            /* 添加审批历史记录 */
            ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
            String elapsed = "";
            if (last != null) {
                elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
            }

            ApproveRole approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
            ApproveHistory history = new ApproveHistory();
            history.setIsReset(DictConst.N);
            history.setApproveType(DictConst.APPROVETYPE_SL);
            history.setActivityId(activity.getId());
            history.setApproveId(approve.getId());
            history.setApprove(DictConst.STATUS_BSL);
            history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
            history.setOpinion(activity.getRejectContent());
            history.setElapsed(elapsed);
            history.setFinish(approveFinish);
            approveHistoryMapper.insert(history);

            if (DictConst.STATUS_BSL.equals(approveStatus)) {
                /* 生成文书 */
                approveFileService.generateslws(DictConst.STATUS_BSL, activity, user);
                /* 消息通知 */
                notifyService.notAcceptedNotify(activity, user);
                /* 审批结果同步 */
                communicateApprovePSS(activity, false);
            }
        }
        return AjaxResult.success("");
    }

    /**
     * 生成审批意见反馈文件
     *
     * @param activity
     * @return
     */
    private Map<String, String> generateFeedbackFileMap(Activity activity, String feedbackType, String comments) {
        Map<String, String> map = Maps.newHashMap();
        map.put("pssId", activity.getPssId() + "");
        map.put("feedbackType", feedbackType);
        map.put("comments", comments);
        return map;
    }


    /**
     * 生成审批意见反馈文件
     *
     * @param activity
     * @return
     */
    private Map<String, String> generateGradeFileMap(Activity activity, Approve approve, String rankFileName) {
        Map<String, String> map = Maps.newHashMap();
        map.put("pssId", activity.getPssId() + "");
        map.put("activityNo", activity.getActivityNo() );
        map.put("comments", approve.getRankComments());
        map.put("gradeRank", approve.getGradeRank());
        map.put("rankDate", DateUtil.now());
        map.put("rankFile", rankFileName);
        return map;
    }

    /**
     * 生成重新评级文件
     *
     * @param activity
     * @return
     */
    private Map<String, String> generateReGradeFileMap(Activity activity) {
        Map<String, String> map = Maps.newHashMap();
        map.put("pssId", activity.getPssId() + "");
        map.put("activityNo", activity.getActivityNo() );
        map.put("approveStatus", activity.getApproveStatus());
        return map;
    }


    /**
     * 生成审批结果文件
     *
     * @param activity
     * @return
     */
    public Map<String, String> generateApproveFileMap(Activity activity) {
        Map<String, String> map = Maps.newHashMap();
        map.put("pssId", activity.getPssId() + "");
        map.put("activityNo", activity.getActivityNo() );
        /* 审批单位 */
        map.put("approveOrg", getApproveOrg(activity));
        map.put("approveStatus", activity.getApproveStatus());
        map.put("rejectContent", activity.getRejectContent());

        /*活动审批*/
        if ("YES,NO,REJECT".contains(activity.getApproveStatus())) {
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日");
            ApproveHistory history = approveHistoryMapper.findByActivityIdAndApprove(activity.getId(), activity.getApproveStatus());
            String approvalTime;
            if (history != null) {
                approvalTime = simpleDateFormat.format(history.getAddTime());
            } else {
                approvalTime = simpleDateFormat.format(activity.getUpdateTime());
            }
            map.put("approvalTime", approvalTime);
            //组装发送短信信息
            SPMsgTemplate SPMsgTemplate = spMsgTemplateService.findAdministrativeByArea(activity);
            if (SPMsgTemplate != null) {
                //激活状态
                if (Objects.equals("1", SPMsgTemplate.getActive())) {
                    map.put("material", SPMsgTemplate.getMaterial());
                    map.put("address", SPMsgTemplate.getAddress());
                    map.put("tel", SPMsgTemplate.getTel());
                }
            }
        }
        /* 活动初审 */
        else if ("20".contains(activity.getApproveStatus()) || "30".contains(activity.getApproveStatus())) {
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日");
            ApproveHistory history = approveHistoryMapper.findByActivityIdAndApprove(activity.getId(), activity.getApproveStatus());
            String approvalTime;
            if (history != null) {
                approvalTime = simpleDateFormat.format(history.getAddTime());
            } else {
                approvalTime = simpleDateFormat.format(activity.getUpdateTime());
            }
            map.put("approvalTime", approvalTime);

            //组装发送短信信息
            CSMsgTemplate CSMsgTemplate = csMsgTemplateService.findPrimaryNoticeByArea(activity);
            if (CSMsgTemplate != null) {
                //激活状态
                if (Objects.equals("1", CSMsgTemplate.getActive())) {
                    map.put("address", CSMsgTemplate.getAddress());
                    map.put("tel", CSMsgTemplate.getTel());
                }
            }
        }
        /* 活动受理 */
        else if ("SL20".contains(activity.getApproveStatus()) || "SL30".contains(activity.getApproveStatus())) {
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日");
            ApproveHistory history = approveHistoryMapper.findByActivityIdAndApprove(activity.getId(), activity.getApproveStatus());
            String approvalTime;
            if (history != null) {
                approvalTime = simpleDateFormat.format(history.getAddTime());
            } else {
                approvalTime = simpleDateFormat.format(activity.getUpdateTime());
            }
            map.put("approvalTime", approvalTime);
            //组装发送短信信息
            SlMsgTemplate slMsgTemplate = slService.findTemplateByArea(activity);
            if (slMsgTemplate != null) {
                //激活状态
                if (Objects.equals("1", slMsgTemplate.getActive())) {
                    map.put("address", slMsgTemplate.getAddress());
                    map.put("tel", slMsgTemplate.getTel());
                }
            }
        }
        /*
         * 转交信息
         */
        if (Objects.equals(DictConst.Y, activity.getIsTransfer())) {
            String newApproveAddr = getApproveArea(activity.getProvince(), activity.getCity(), activity.getArea());
            map.put("transferContent", activity.getTransferContent());
            map.put("isTransfer", DictConst.Y);
            String title = "【" + activity.getFullName() + "】已被转交";
            String content = "【" + activity.getFullName() + "】已被公安机关转交至" + newApproveAddr + "。";
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日");
            ApproveHistory history = approveHistoryMapper.findByActivityIdAndApprove(activity.getId(), activity.getApproveStatus());
            String approvalTime;
            if (history != null) {
                approvalTime = simpleDateFormat.format(history.getAddTime());
            } else {
                approvalTime = simpleDateFormat.format(activity.getUpdateTime());
            }
            /* 审批单位 */
            map.put("approveOrg", getApproveOrg(activity));
            map.put("approvalTime", approvalTime);
            map.put("sendTitle", title);
            map.put("sendContent", content);
            // 需要同步转交省市区
            map.put("transferProvince", activity.getProvince());
            map.put("transferCity", activity.getCity());
            map.put("transferArea", activity.getArea());
            map.put("newApproveAddr", newApproveAddr);
        }
        return map;
    }


    /**
     * 生成审批结果文件
     *
     * @param syncTaskDTO
     * @return
     */
    public Map<String, String> generateApproveFileMap(SyncTaskDTO syncTaskDTO) {
        Map<String, String> map = Maps.newHashMap();
        map.put("pssId", syncTaskDTO.getPssId() + "");
        map.put("activityNo", syncTaskDTO.getActivityNo() );
        /* 审批单位 */
        map.put("approveOrg", getApproveOrg(syncTaskDTO.getActivityId()));
        map.put("approveStatus", syncTaskDTO.getApproveStatus());
        map.put("rejectContent", syncTaskDTO.getApproveContent());
        map.put("approveTime", DateUtil.format(syncTaskDTO.getApprovalTime(), DatePattern.NORM_DATETIME_PATTERN));

        /* 活动初审通过 */
        if (DictConst.STATUS_CSTG.contains(syncTaskDTO.getApproveStatus())) {
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日");
            map.put("approvalTime", simpleDateFormat.format(syncTaskDTO.getApprovalTime()));
            //组装发送短信信息
            CSMsgTemplate CSMsgTemplate = csMsgTemplateService.findPrimaryNoticeByArea(
                    syncTaskDTO.getProvince(), syncTaskDTO.getCity(), syncTaskDTO.getArea());
            if (CSMsgTemplate != null) {
                //激活状态
                if (Objects.equals("1", CSMsgTemplate.getActive())) {
                    map.put("address", CSMsgTemplate.getAddress());
                    map.put("tel", CSMsgTemplate.getTel());
                }
            }
        }
        /* 活动受理 */
        else if (DictConst.STATUS_SL.contains(syncTaskDTO.getApproveStatus())) {
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日");
            map.put("approvalTime", simpleDateFormat.format(syncTaskDTO.getApprovalTime()));

            //组装发送短信信息
            SlMsgTemplate slMsgTemplate = slService.findTemplateByArea(
                    syncTaskDTO.getProvince(), syncTaskDTO.getCity(), syncTaskDTO.getArea());
            if (slMsgTemplate != null) {
                //激活状态
                if (Objects.equals("1", slMsgTemplate.getActive())) {
                    map.put("address", slMsgTemplate.getAddress());
                    map.put("tel", slMsgTemplate.getTel());
                }
            }
        }

        /*
         * 转交信息
         */
        Activity activity = activityMapper.findFullById(syncTaskDTO.getActivityId());
        if (Objects.equals(DictConst.Y, activity.getIsTransfer())) {
            String newApproveAddr = getApproveArea(activity.getProvince(), activity.getCity(), activity.getArea());
            map.put("transferContent", activity.getTransferContent());
            map.put("isTransfer", DictConst.Y);
            String title = "【" + activity.getFullName() + "】已被转交";
            String content = "【" + activity.getFullName() + "】已被公安机关转交至" + newApproveAddr + "。";
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日");
            /* 审批单位 */
            map.put("approveOrg", getApproveOrg(syncTaskDTO.getActivityId()));
            map.put("approvalTime", simpleDateFormat.format(activity.getUpdateTime()));
            map.put("sendTitle", title);
            map.put("sendContent", content);
            // 需要同步转交省市区
            map.put("transferProvince", activity.getProvince());
            map.put("transferCity", activity.getCity());
            map.put("transferArea", activity.getArea());
            map.put("newApproveAddr", newApproveAddr);
        }
        return map;
    }


    /**
     * 备案活动打回
     */
    public void rejectBackUpAct(User user, String activityId, String rejectContent) {
        Activity activity = activityMapper.selectById(activityId);
        // 1.设置备案活动退回状态
        activity.setApproveStatus(DictConst.STATUS_REJECT);
        activity.setRejectContent(rejectContent);
        activity.setSync(DictConst.N);
        activityMapper.updateById(activity);

        /* 2.增加审批记录 */
        Approve approve = new Approve();
        approve.setActivityId(activity.getId());
        approve.setApproveType(DictConst.APPROVETYPE_SP);
        approve.setAddTime(new Date());
        approve.setUpdateTime(new Date());
        approve.setUserId(user.getId());
        approve.setUpdater(user.getRealName());
        approve.setActivityId(activity.getId());
        approve.setSeq(1);
        approve.setApprove(DictConst.STATUS_REJECT);
        approveMapper.insert(approve);

        /* 3.添加审批历史记录 */
        ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
        String elapsed = "";
        if (last != null) {
            elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
        }

        ApproveRole approveRole = null;
        try {
            // 有些WorkRole不是数字
            approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));

        } catch (Exception e) {

        }
        ApproveHistory history = new ApproveHistory();
        history.setIsReset(DictConst.N);
        history.setApproveType(DictConst.APPROVETYPE_SP);
        history.setActivityId(activity.getId());
        history.setApproveId(approve.getId());
        history.setApprove(DictConst.STATUS_REJECT);
        history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
        history.setOpinion(rejectContent);
        history.setElapsed(elapsed);
        approveHistoryMapper.insert(history);

        /* 4.增加状态同步任务 */
        SyncTask syncTask = new SyncTask();
        syncTask.setActivityId(activity.getId());
        syncTask.setSyncTaskType(DictConst.APPROVETYPE_SP);
        syncTask.setApproveStatus(DictConst.STATUS_REJECT);
        syncTask.setAddTime(new Date());
        syncTask.setSync(DictConst.N);
        syncTaskMapper.insert(syncTask);

        /* 5.备案打回，消息通知 */
        notifyService.backUpActReject(activity, user);

        /* 6.审批结果同步 */
        communicateApprovePSS(activity, true);
    }

    /**
     * 转交活动
     */
    public void transferAct(User user, String activityId, String province, String city, String area, String transferContent) {
        Activity activity = activityMapper.selectById(Long.parseLong(activityId));
        //保留原始数据，进行消息通知
        Activity oldActivity = new Activity();
        BeanUtils.copyProperties(activity, oldActivity);
        // 获取已经走完审批的用户
        List<Map> alreadyApproveUser = activityMapper.findAlreadyApproveUser(activity.getId());
        // 获取原审批地区
        String sourceAddr = getApproveArea(activity.getProvince(), activity.getCity(), activity.getArea());
        // 根据转交的省市，重新获取流程版本
        List<Workflow> workflowList = workflowDao.findActive(province, city, area);
        if (workflowList == null || workflowList.size() == 0) {
            throw new RuntimeException("该地区没有配置流程，不能转交，请联系管理员！");
        }
        // 取最新版本的流程版本
        Workflow workflow = workflowList.get(0);
        String workFlowVersion = workflow.getVersion();

        /** 1.重新设置活动省市区 */
        activity.setProvince(province);
        activity.setCity(city);
        activity.setArea(area);
        // 重置为初审状态
        activity.setApproveStatus(DictConst.STATUS_CSZ);
        // 重置版本
        activity.setWorkflowVersion(workFlowVersion);
        // 设置审批等级
        if (StrUtil.isNotEmpty(area)) {
            activity.setApproveLevel(DictConst.APPROVELEVEL_AREA);
        } else if (StrUtil.isNotEmpty(city)) {
            activity.setApproveLevel(DictConst.APPROVELEVEL_CITY);
        } else if (StrUtil.isNotEmpty(province)) {
            activity.setApproveLevel(DictConst.APPROVELEVEL_PROVINCE);
        }
        activity.setTransferContent(transferContent);
        // 设置项目未转交，等初审的时候重置为 DictConst.N
        activity.setIsTransfer(DictConst.Y);
        activityMapper.updateById(activity);

        /* 2.增加审批记录 */
        Approve approve = new Approve();
        approve.setActivityId(activity.getId());
        // 转交类型
        approve.setApproveType(DictConst.APPROVETYPE_TRANSFER);
        approve.setAddTime(new Date());
        approve.setUpdateTime(new Date());
        approve.setUserId(user.getId());
        approve.setUpdater(user.getRealName());
        approve.setActivityId(activity.getId());
        approve.setSeq(1);
        approve.setApprove(DictConst.STATUS_TRANSFER);
        approveMapper.insert(approve);

        /* 3.添加审批历史记录 */
        ApproveHistory last = approveHistoryMapper.findLastByActivityId(activity.getId());
        String elapsed = "";
        if (last != null) {
            elapsed = BcxinDateUtil.diffFormatChina(last.getAddTime(), new Date());
        }
        ApproveRole approveRole = null;
        try {
            // 有些WorkRole不是数字
            approveRole = approveRoleMapper.findById(Long.parseLong(user.getWorkRole()));
        } catch (Exception e) {

        }
        ApproveHistory history = new ApproveHistory();
        history.setIsReset(DictConst.N);
        history.setApproveType(DictConst.APPROVETYPE_TRANSFER);
        history.setActivityId(activity.getId());
        history.setApproveId(approve.getId());
        history.setApprove(DictConst.STATUS_TRANSFER);
        history.setUserName(user.getRealName() + "（" + (approveRole == null ? "" : approveRole.getFullName()) + "）");
        history.setOpinion(transferContent);
        history.setElapsed(elapsed);
        history.setRemark(sourceAddr);
        approveHistoryMapper.insert(history);


        /* 4.增加状态同步任务 */
        SyncTask syncTask = new SyncTask();
        syncTask.setActivityId(activity.getId());
        syncTask.setSyncTaskType(DictConst.APPROVETYPE_TRANSFER);
        syncTask.setApproveStatus(DictConst.STATUS_TRANSFER);
        syncTask.setAddTime(new Date());
        syncTask.setSync(DictConst.N);
        syncTaskMapper.insert(syncTask);

        /* 5.发送通知*/

        // 通知已经审批过该活动的人员
        List<String> userIdList = new ArrayList<>();
        String transferAddr = getApproveArea(province, city, area);
        for (Map map : alreadyApproveUser) {
            userIdList.add(map.get("userId").toString());
        }
        // 当前转出人员
        if (!userIdList.contains(user.getId().toString())) {
            userIdList.add(user.getId().toString());
        }
        notifyService.transferActNotify(activity, userIdList, user, transferAddr);

        // 通知接收活动的审批人员
        List<User> list = userMapper.findApproveUser(activity.getApproveLevel(), activity.getId());
        List<String> newUserIdList = new ArrayList<>();
        for (User user1 : list) {
            newUserIdList.add(user1.getId().toString());
        }
        notifyService.transferActApproveNotify(oldActivity, newUserIdList, user, sourceAddr, transferAddr);

        /** 6.生成转交信息，同步到子站，提示消息给(承办方，风评)*/
        communicateApprovePSS(activity, false);
    }


    @Autowired
    private RegionMapper regionMapper;

    /**
     * 获取审批地区
     *
     * @param province
     * @param city
     * @param area
     * @return
     */
    private String getApproveArea(String province, String city, String area) {
        String addr = "";
        if (StringUtil.isNotEmpty(province)) {
            addr += regionMapper.findNameById(Long.parseLong(province));
        }
        if (StringUtil.isNotEmpty(city)) {
            addr += "-" + regionMapper.findNameById(Long.parseLong(city));
        }
        if (StringUtil.isNotEmpty(area)) {
            addr += "-" + regionMapper.findNameById(Long.parseLong(area));
        }
        return addr;
    }

    private String getApproveOrg(Activity activity) {
        Workflow workflow = workflowDao.findRegionByVersion(activity.getId());
        if (workflow == null) {
            return "";
        }
        if (StringUtil.isNotEmpty(workflow.getArea())) {
            return workflow.getArea() + "分局";
        }
        if (StringUtil.isNotEmpty(workflow.getCity())) {
            return workflow.getCity() + "公安局";
        }
        return workflow.getProvince();
    }

    private String getApproveOrg(Long activityId) {
        Workflow workflow = workflowDao.findRegionByVersion(activityId);
        if (workflow == null) {
            return "";
        }
        if (StringUtil.isNotEmpty(workflow.getArea())) {
            return workflow.getArea() + "分局";
        }
        if (StringUtil.isNotEmpty(workflow.getCity())) {
            return workflow.getCity() + "公安局";
        }
        return workflow.getProvince();
    }

    /**
     * 验证用户是否具备审批权限，如果返回true则具备
     * @param activityId
     * @param user
     * @return
     */
    private boolean checkApproveUser(Long activityId,User user){
        Activity activity = activityMapper.selectById(activityId);
        if (DictConst.STATUS_CSZ.equals(activity.getApproveStatus())
            || DictConst.STATUS_SQCP.equals(activity.getApproveStatus())
            || DictConst.STATUS_SLZ.equals(activity.getApproveStatus())
            || DictConst.STATUS_NONE.equals(activity.getApproveStatus())
            || DictConst.STATUS_IN.equals(activity.getApproveStatus())) {
            /* 如果是指定用户，则用指定用户审批 */
            if (StringUtil.isNotEmpty(activity.getNextApproveUser())) {
                /* 当前登录用户是指定的用户 */
                if (user.getId().equals(Long.parseLong(activity.getNextApproveUser()))) {
                    return true;
                }
            }
            /* 如果是指定角色，则用指定角色审批 */
            else if (StringUtil.isNotEmpty(activity.getNextApproveRole())) {
                if (StringUtil.isNotEmpty(user.getWorkRole())) {
                    if (user.getWorkRole().equals(activity.getNextApproveRole())) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    public static void main(String[] args) throws Exception {
        String[] folderPath = new String[]{"/data/pss/material/", "spjg", DateUtil.today(), SPJG_FOLDER};
        String dir = BcxinFileUtil.appendFolderPath(folderPath);
        System.out.println(dir);
        String fileName = BcxinFileUtil.packTimestamp(SPRANK + "392") + ".bcx";
        String detailUrl = "D:/"+dir + fileName;
        final File newFile = new File(detailUrl);
        System.out.println(detailUrl);

        String newRankFileName = "";
        Map<String, String> map = Maps.newHashMap();
        map.put("pssId", "392");
        map.put("activityNo", "OD167867365079889670" );
        map.put("comments", "经磐安县保安服务有限公司评定，风险等级为中级。");
        map.put("gradeRank", "2");
        map.put("rankDate", DateUtil.now());
        map.put("rankFile", newRankFileName);
        String result = JSON.toJSONString(map);
        try {
            byte[] buf = DESUtil.encrypt(result.getBytes("UTF-8"), GlobalConst.KEY);
            Files.write(buf, newFile);
        } catch (Exception ex) {
            ex.printStackTrace();
            log.error("生成审批意见文件失败,活动392");
        }
//        String file = "http://www.bac.com/getre?path=D:/test/111111121bbbsss.jpg";
//        File rankFile = new File(file.substring(file.indexOf("path=") + 5));
//        String newRankFileName = BcxinFileUtil.packTimestamp(SPRANKFILE + "123123") + "." + Files.getFileExtension(rankFile.getName());
//        File newRankFile = new File("D:\\test\\2019-08-06\\" + newRankFileName);
//        Files.copy(rankFile, newRankFile);
    }
}
