package com.bcxin.sp.work.service.base;

import com.baomidou.mybatisplus.plugins.Page;
import com.bcxin.sp.work.dao.base.VenueMapper;
import com.bcxin.sp.work.entity.PageResult;
import com.bcxin.sp.work.entity.domain.base.Venue;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.entity.dto.VenueDTO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class VenueService {

    @Resource
    private VenueMapper venueMapper;

    /**
     * <b> 根据活动查询场馆信息 </b>
     * @author ZXF
     * @create 2020/02/28 0028 15:03
     * @version
     * @注意事项 </b>
     */
    public List<Map> findVenueListByActivityId(long activityId) {
        return venueMapper.findVenueListByActivityId(activityId);
    }

    /**
     * <b> 根据活动id查询首条场馆信息 </b>
     * @author ZXF
     * @create 2020/03/03 0003 15:32
     * @version
     * @注意事项 </b>
     */
    public Map getVenueByIdForOne(long activityId) {
        return venueMapper.getVenueByIdForOne(activityId);
    }

    public PageResult findCggdListData(SearchDto<VenueDTO> dto) throws Exception {
        List<Map> list = venueMapper.findCggdListData(dto);
        Long total = venueMapper.findCggdListDataCnt(dto);
        return new PageResult(list,total);
    }

    /**
     * 查出所有场馆-分页
     * @author wangjianjun
     * @date 2020/6/3
     */
    public PageResult findAll(SearchDto<VenueDTO> searchDTO) throws Exception {
        Page<Venue> page = new Page<>(searchDTO.getIDisplayStart()/searchDTO.getIDisplayLength()+1, searchDTO.getIDisplayLength());
        List<Venue> list = venueMapper.findAllPage(page, searchDTO);
        return new PageResult(list, (long) page.getTotal());
    }

    public List<Venue> findAll() throws Exception {
        return venueMapper.findAll();
    }

    /**
     * 以下为自动生成的方法
     * @param id
     * @return
     */

    public int deleteByPrimaryKey(Long id) {
        return venueMapper.deleteByPrimaryKey(id);
    }


    public int insert(Venue record) {
        return venueMapper.insert(record);
    }


    public int insertSelective(Venue record) {
        return venueMapper.insertSelective(record);
    }


    public Venue selectByPrimaryKey(Long id) {
        return venueMapper.selectByPrimaryKey(id);
    }


    public int updateByPrimaryKeySelective(Venue record) {
        return venueMapper.updateByPrimaryKeySelective(record);
    }


    public int updateByPrimaryKey(Venue record) {
        return venueMapper.updateByPrimaryKey(record);
    }

}
