package com.bcxin.sp.work.service.statistic;

import com.bcxin.sp.work.dao.activity.base.ActivityMapper;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.entity.dto.SummaryDto;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.SearchUtil;
import com.bcxin.sp.work.util.StringUtil;
import com.google.common.base.Objects;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class SummaryService {
	
	@Autowired
	private ActivityMapper activityMapper;

	public List<SummaryDto> findSummary(SearchDto<SummaryDto> searchDto,User currentUser) throws Exception {
		StringBuffer condition = SearchUtil.getSummaryCondition(searchDto,currentUser);
		List<SummaryDto> result = activityMapper.findSummary(condition.toString());
		for(SummaryDto data : result) {
			data.setAll(data.getAll()+data.getCharity());
			data.setAll(data.getAll()+data.getExhibition());
			data.setAll(data.getAll()+data.getGardenlantern());
			data.setAll(data.getAll()+data.getLiterature());
			data.setAll(data.getAll()+data.getRecruit());
			data.setAll(data.getAll()+data.getSport());
			data.setAll(data.getAll()+data.getOther());
			data.setAll(data.getAll()+data.getGonganBeian());
		}
		return result;
	}

	/**
	 * 获得审批进度统计各个数据
	 * @param user
	 * @return
	 */
	public Map<String,BigDecimal> approvalSummary(User user){
		Map<String,BigDecimal> resultMap = Maps.newHashMap();

		int regionLevel = 0;
		if (!(Objects.equal(DictConst.ROLE_SUPERADMIN, user.getRole())
			|| Objects.equal(DictConst.ROLE_ADMIN, user.getRole())
			|| Objects.equal(DictConst.ROLE_ADMIN_APPROVE, user.getRole())
			|| Objects.equal(DictConst.ROLE_EXPORT, user.getRole())
			|| Objects.equal(DictConst.ROLE_READ, user.getRole())
			|| Objects.equal(DictConst.ROLE_DOWNLOAD, user.getRole()))) {
			/* 增加审批级别过滤 */
			regionLevel = user.getRegionLevel();
		}


		Map<String,BigDecimal> map1 = activityMapper.approvalSummary(user.getProvince(),user.getCity(),user.getArea(),regionLevel);
		Map<String,BigDecimal> map2 = activityMapper.approvalFinalSummary(user.getProvince(),user.getCity(),user.getArea(),null,null,regionLevel);
		Map<String,BigDecimal> map3 = activityMapper.approvalTodaySummary(user.getProvince(),user.getCity(),user.getArea(),regionLevel);
		if (map1 != null) {
			resultMap.putAll(map1);
			/* 初审阶段总计*/
			resultMap.put("csjdzj",map1.get("csz").add(map1.get("csxg")).add(map1.get("cstg")));
			/* 受理阶段总计*/
			resultMap.put("sljdzj",map1.get("slz").add(map1.get("bsl")).add(map1.get("ysl")));
			/* 许可审批阶段总计*/
			resultMap.put("xkspjdzj",map1.get("wsp").add(map1.get("spz")).add(map1.get("spth")));
		} else {
			/* 初审阶段总计*/
			resultMap.put("csjdzj",BigDecimal.ZERO);
			/* 受理阶段总计*/
			resultMap.put("sljdzj",BigDecimal.ZERO);
			/* 许可审批阶段总计*/
			resultMap.put("xkspjdzj",BigDecimal.ZERO);
		}
		if (map2 != null) {
			resultMap.putAll(map2);
			/* 许可审批结果总计*/
			resultMap.put("xkspjgzj",map2.get("xk").add(map2.get("bxk")));
		} else {
			/* 许可审批结果总计*/
			resultMap.put("xkspjgzj",BigDecimal.ZERO);
		}
		if (map3 != null) {
			resultMap.putAll(map3);
		} else {
			/* 今日许可审批 */
			resultMap.put("xksp",BigDecimal.ZERO);
			/* 今日报备 */
			resultMap.put("bb",BigDecimal.ZERO);
		}
		return resultMap;
	}


	/**
	 * 获得审批进度统计各个数据（审批结果）
	 * @param user
	 * @return
	 */
	public Map<String,BigDecimal> approvalSummarySPJG(User user,String beginDate,String endDate){
		Map<String,BigDecimal> resultMap;

		int regionLevel = 0;
		if (!(Objects.equal(DictConst.ROLE_SUPERADMIN, user.getRole())
			|| Objects.equal(DictConst.ROLE_ADMIN, user.getRole())
			|| Objects.equal(DictConst.ROLE_ADMIN_APPROVE, user.getRole())
			|| Objects.equal(DictConst.ROLE_EXPORT, user.getRole())
			|| Objects.equal(DictConst.ROLE_READ, user.getRole())
			|| Objects.equal(DictConst.ROLE_DOWNLOAD, user.getRole()))) {
			/* 增加审批级别过滤 */
			regionLevel = user.getRegionLevel();
		}

		if (StringUtil.isNotEmpty(beginDate) && StringUtil.isNotEmpty(endDate)) {
			resultMap = activityMapper.approvalFinalSummary(user.getProvince(),user.getCity(),user.getArea(), beginDate,endDate,regionLevel);
		} else {
			resultMap = activityMapper.approvalFinalSummary(user.getProvince(),user.getCity(),user.getArea(),null,null,regionLevel);
		}
		if (resultMap != null) {
			/* 许可审批结果总计*/
			resultMap.put("xkspjgzj",resultMap.get("xk").add(resultMap.get("bxk")));
		} else {
			/* 许可审批结果总计*/
			resultMap = new HashMap<String,BigDecimal>();
			resultMap.put("xkspjgzj",BigDecimal.ZERO);
		}
		return resultMap;
	}

}
