/**
 * date: 2020-04-26 17:58
 * copyright(c) 2015-2020 百川信
 */
package com.bcxin.sp.work.service.third.activityIsr;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.plugins.Page;
import com.bcxin.sp.work.dao.activity.extend.ActivityAssessmentMapper;
import com.bcxin.sp.work.entity.PageResult;
import com.bcxin.sp.work.entity.domain.activity.expand.ActivityAssessment;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.entity.dto.accident.ActivityAssessmentSearchDTO;
import com.bcxin.sp.work.util.BcxinFileUtil;
import com.bcxin.sp.work.util.Const.GlobalConst;
import com.bcxin.sp.work.util.DESUtil;
import com.bcxin.sp.work.util.StringUtil;
import com.bcxin.sp.work.util.sys.ConfigUtil;
import com.google.common.collect.Maps;
import com.google.common.io.Files;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author: wangjianjun
 * @description: 风险评估
 * @data: 2020-04-26 17:58
 **/
@Service
@Transactional
@Slf4j
public class ActivityAssessmentService {

    /**
     * 同步服务商结果文件夹名称
     */
    private final static String SPJG_FOLDER = "spjg";

    /**
     * 承办者内容，文件标识
     */
    private final static String SPAFREE = "_spafree_";

    /**
     * 承办者文件，文件标识
     */
    private final static String SPAFREEFILE = "_spafreefile_";

    @Resource
    private ActivityAssessmentMapper activityAssessmentMapper;

    /**
     * 列表
     * @param searchDTO
     * @return
     */
    public PageResult findActivityAssessmentList(ActivityAssessmentSearchDTO searchDTO) {
//        PageHelper.startPage(1, 10);
        Page<ActivityAssessment> page = new Page<>(searchDTO.getIDisplayStart()/searchDTO.getIDisplayLength()+1, searchDTO.getIDisplayLength());
        List<ActivityAssessment> list = activityAssessmentMapper.findAllByUpdateTime(page, searchDTO);
        return new PageResult(list, (long) page.getTotal());
    }

    /**
     * 详情
     * @param id
     * @return
     */
    public ActivityAssessment findActivityAssessmentDetail(Long id) {
        return activityAssessmentMapper.selectByPrimaryKey(id);
    }

    public ActivityAssessment findById(long id) {
        return activityAssessmentMapper.selectById(id);
    }

    public PageResult findListData(SearchDto<ActivityAssessment> dto) throws Exception {
        List<ActivityAssessment> list = activityAssessmentMapper.findListData(dto);
        Long total = activityAssessmentMapper.findListDataCnt(dto);
        return new PageResult(list, total);
    }

    public ActivityAssessment createOrUpdate(ActivityAssessment aa) {
        if (aa.getActivityAssessmentId() != null) {
            aa.setUpdateTime(new Date());
            //标识是否要进行同步
            if(aa.getIsSync() == 3){
                try {
                    communicateAssessmentPSS(toMap(aa));
                } catch (Exception e) {
                    e.printStackTrace();
                }
                aa.setIsSync(1);
            }
            activityAssessmentMapper.updateById(aa);
        } else {
            aa.setAddTime(new Date());
            activityAssessmentMapper.insert(aa);
        }
        return aa;
    }

    /**
     * <b> 冻结/解冻 </b>
     * @author ZXF
     * @create 2020/11/09 0009 13:49
     * @version
     * @注意事项 </b>
     */
    public void freeze(ActivityAssessment aa) {
        if(1==aa.getIsFreeze()){
            aa.setIsFreeze(0);
        }else{
            aa.setIsFreeze(1);
        }
        try {
            communicateAssessmentPSS(toMap(aa));
        } catch (Exception e) {
            e.printStackTrace();
        }
        aa.setIsSync(1);
        activityAssessmentMapper.updateById(aa);
    }

    private Map<String, String> toMap(ActivityAssessment aa) {
        Map<String, String> map = Maps.newHashMap();
        map.put("pssId", aa.getPssId() + "");
        map.put("fullName", aa.getFullName());
        map.put("assessmentType", aa.getAssessmentType());
        map.put("approveStatus", aa.getApproveStatus()+"");
        map.put("adminTel", aa.getAdminTel());
        map.put("province", aa.getProvince());
        map.put("phone", aa.getPhone());
        map.put("city", aa.getCity());
        map.put("contact", aa.getContact());
        map.put("organizationCode", aa.getOrganizationCode());
        map.put("area", aa.getArea());
        map.put("address", aa.getAddress());
        map.put("applyTime", DateUtil.formatDateTime(aa.getApplyTime()));
        map.put("orgUrl", aa.getOrgUrl());
        map.put("email", aa.getEmail());
        map.put("isFreeze", aa.getIsFreeze()+"");
        return map;
    }

    /**
     * 同步服务商
     *
     * @param map
     */
    private void communicateAssessmentPSS(Map<String, String> map) throws Exception {

        /*文件生成，同步*/
        if (Objects.equals("file", ConfigUtil.getValue("pss-communicate-mode"))) {
            String spDir = ConfigUtil.getValue("spjg_folder");
            String uploadPath = ConfigUtil.getValue("stuff-path");
            // 组装文件夹
            String[] folderPath = new String[]{uploadPath, spDir, DateUtil.today(), SPJG_FOLDER};
            if (StringUtil.isNotEmpty(ConfigUtil.getValue("profile_envi"))
                    && ConfigUtil.getValue("profile_envi").startsWith("pre")) {
                folderPath = new String[]{uploadPath, spDir, ConfigUtil.getValue("envi"), DateUtil.today(), SPJG_FOLDER};
            }
            String dir = BcxinFileUtil.appendFolderPath(folderPath);
            String pssId = map.get("pssId");
            //打包承办者信息 文件名： _sprank_100021_t_20170101010101
            String fileName = BcxinFileUtil.packTimestamp(SPAFREE + pssId) + ".bcx";
            String detailUrl = dir + fileName;
            final File newFile = new File(detailUrl);

            String newOrgFileName = "";
            String orgUrl = map.get("orgUrl");
            if (StringUtil.isNotEmpty(orgUrl)) {
                /* 将承办者文件拷贝一份到同步目录下*/
                File orgFile = new File(orgUrl.substring(orgUrl.indexOf("path=") + 5));
                newOrgFileName = BcxinFileUtil.packTimestamp(SPAFREEFILE + pssId) + "." + Files.getFileExtension(orgFile.getName());
                File newOrgFile = new File(dir + newOrgFileName);
                Files.copy(orgFile, newOrgFile);
            }
            map.put("orgUrl",newOrgFileName);
            String result = JSON.toJSONString(map);
            System.out.println("result===>> " +result);
            try {
                byte[] buf = DESUtil.encrypt(result.getBytes("UTF-8"), GlobalConst.KEY);
                Files.write(buf, newFile);
            } catch (Exception ex) {
                ex.printStackTrace();
                log.error("生成服务商文件失败,PSSID" + pssId);
            }

        } else if (Objects.equals("auto", ConfigUtil.getValue("pss-communicate-mode"))) {
        }

    }
}