package com.bcxin.sp.work.service.third.shanma;

import com.alibaba.fastjson.JSON;
import com.bcxin.sp.work.dao.activity.base.ActivityMapper;
import com.bcxin.sp.work.dao.activity.extend.ActivityCameraMapper;
import com.bcxin.sp.work.dao.activity.extend.ActivityCameraTotalMapper;
import com.bcxin.sp.work.dao.third.shanma.DeviceCameraMapper;
import com.bcxin.sp.work.entity.PageResult;
import com.bcxin.sp.work.entity.domain.activity.monitor.ActivityCamera;
import com.bcxin.sp.work.entity.domain.activity.monitor.ActivityCameraTotal;
import com.bcxin.sp.work.entity.domain.base.Camera;
import com.bcxin.sp.work.entity.dto.AjaxResult;
import com.bcxin.sp.work.entity.dto.CameraDTO;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.util.MyConverUtil;
import com.google.common.collect.Maps;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class CommunicateService {

    @Resource
    private DeviceCameraMapper deviceCameraMapper;

    @Resource
    private ActivityCameraMapper activityCameraMapper;

    @Resource
    private ActivityCameraTotalMapper activityCameraTotalMapper;

    @Resource
    private ActivityMapper activityMapper;

    /**
     * <b> 保存摄像头信息 </b>
     *
     * @author ZXF
     * @create 2020/02/27 0027 10:40
     * @version
     * @注意事项 </b>
     */
    public AjaxResult saveCamera(Map map) {
        try {
            Map<String, Object> columnMap = new HashMap<String, Object>();
            columnMap.put("venueId", map.get("venueId"));
            columnMap.put("serialNumber", map.get("serialNumber"));
            List<Camera> list = deviceCameraMapper.selectByMap(columnMap);
            Camera camera;
            if (list.size() > 0) {
                camera = list.get(0);
            } else {
                camera = new Camera();
                camera.setCreateOn(new Date());
            }
            MyConverUtil.mapToObj(map, camera);
            if (camera.getCameraId() != null) {
                deviceCameraMapper.updateById(camera);
            } else {
                deviceCameraMapper.insert(camera);
            }
            return AjaxResult.success(AjaxResult.SUCCESS_MSG);
        } catch (Exception e) {
            return AjaxResult.fail("无效token！");
        }
    }

    /**
     * <b> 保存摄像头视频分析数据 （实时）</b>
     *
     * @author ZXF
     * @create 2020/02/27 0027 10:40
     * @version
     * @注意事项 </b>
     */
    public AjaxResult saveAnalyze(Map map) {
        try {
            Long deviceCameraId = deviceCameraMapper.getIdBySerialNumber(String.valueOf(map.get("cameraSerialNumber")));
            if (deviceCameraId == null) {
                return AjaxResult.fail("未匹配到摄像头信息！");
            }
            Long activityId = activityMapper.getIdByActivityNo(String.valueOf(map.get("activitySerialNumber")));
            if (activityId == null) {
                return AjaxResult.fail("未匹配到活动信息！");
            }
            Map<String, Object> columnMap = new HashMap<String, Object>();
            columnMap.put("deviceCameraId", deviceCameraId);
            columnMap.put("activityId", activityId);
            ActivityCamera activityCamera = new ActivityCamera();
            activityCamera.setActivityId(activityId);
            activityCamera.setDeviceCameraId(deviceCameraId);
            activityCamera.setAddTime(new Date());
            MyConverUtil.mapToObj(map, activityCamera);
            activityCameraMapper.insert(activityCamera);
            return AjaxResult.success(AjaxResult.SUCCESS_MSG);
        } catch (Exception e) {
            return AjaxResult.fail("数据更新失败！");
        }
    }

    /**
     * <b> 保存摄像头视频分析数据 （统计）</b>
     *
     * @author ZXF
     * @create 2020/02/27 0027 10:40
     * @version
     * @注意事项 </b>
     */
    public AjaxResult saveAnalyzeTotal(Map paramMap) {
        try {
            Long deviceCameraId = deviceCameraMapper.getIdBySerialNumber(String.valueOf(paramMap.get("cameraSerialNumber")));
            if (deviceCameraId == null) {
                return AjaxResult.fail("未匹配到摄像头信息！");
            }
            Long activityId = activityMapper.getIdByActivityNo(String.valueOf(paramMap.get("activitySerialNumber")));
            if (activityId == null) {
                return AjaxResult.fail("未匹配到活动信息！");
            }
            Map<String, Object> columnMap = Maps.newHashMap();
            columnMap.put("deviceCameraId", deviceCameraId);
            columnMap.put("activityId", activityId);
            String eventTypeData = String.valueOf(paramMap.get("eventTypeData"));
            List<Map> eventTypeDataList = JSON.parseArray(eventTypeData, Map.class);
            for (Map map:eventTypeDataList) {
                columnMap.put("eventType",map.get("eventType").toString());
                List<ActivityCameraTotal> list = activityCameraTotalMapper.selectByMap(columnMap);
                ActivityCameraTotal activityCameraTotal;
                if (list.size() > 0) {
                    activityCameraTotal = list.get(0);
                    activityCameraTotal.setUpdateTime(new Date());
                } else {
                    activityCameraTotal = new ActivityCameraTotal();
                    activityCameraTotal.setActivityId(activityId);
                    activityCameraTotal.setDeviceCameraId(deviceCameraId);
                }
                activityCameraTotal.setEventType(map.get("eventType").toString());
                activityCameraTotal.setEventCount(Integer.parseInt(map.get("count").toString()));
                MyConverUtil.mapToObj(paramMap, activityCameraTotal);
                if (activityCameraTotal.getId() != null) {
                    activityCameraTotalMapper.updateById(activityCameraTotal);
                } else {
                    activityCameraTotalMapper.insert(activityCameraTotal);
                }
            }
            return AjaxResult.success(AjaxResult.SUCCESS_MSG);
        } catch (Exception e) {
            return AjaxResult.fail("数据更新失败！");
        }
    }

    public PageResult findSpsbListData(SearchDto<CameraDTO> dto) throws Exception {
        List<CameraDTO> list = deviceCameraMapper.findSpsbListData(dto);
        Long total = deviceCameraMapper.findSpsbListDataCnt(dto);
        return new PageResult(list,total);
    }

}