package com.bcxin.sp.work.service.user;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.bcxin.sp.work.dao.log.UserLoginLogMapper;
import com.bcxin.sp.work.dao.user.UserMapper;
import com.bcxin.sp.work.entity.domain.log.UserLoginLog;
import com.bcxin.sp.work.entity.domain.user.LoginUserDetail;
import com.bcxin.sp.work.entity.domain.user.User;
import com.bcxin.sp.work.entity.dto.SearchDto;
import com.bcxin.sp.work.entity.dto.UserSummarySearchDto;
import com.bcxin.sp.work.util.Const.GlobalConst;
import com.bcxin.sp.work.util.HttpContextUtils;
import com.bcxin.sp.work.util.ServletUtils;
import com.bcxin.sp.work.util.StringUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.net.InetAddress;
import java.util.Date;
import java.util.List;

@Service
@Transactional
public class UserService {

    @Resource
    private UserMapper userMapper;

    @Resource
    private UserLoginLogMapper userLoginLogMapper;

    public LoginUserDetail getCurrentUserDetail() {
        Object userObj = ServletUtils.getRequest().getSession().getAttribute(GlobalConst.LOGIN_USER_DETAIL);
        if (userObj == null) {
            return null;
        }
        return (LoginUserDetail)userObj;
    }

    public User findByUserName(String username) {
        List<User> list = userMapper.selectList(new EntityWrapper<User>().eq("username", username));
        if (CollectionUtil.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    public User findById(long id) {
        return userMapper.selectById(id);
    }

    public User createOrUpdate(User user) {
        if (user.getId() != null) {
            user.setUpdateTime(new Date());
            userMapper.updateById(user);
        } else {
            user.setPic("/img/default_head.jpg");//默认头像
            user.setPassword(DigestUtils.md5Hex(user.getPassword()));
            user.setUpdateTime(new Date());
            user.setAddTime(new Date());
            userMapper.insert(user);
        }
        return user;
    }

    public List<User> findUser(SearchDto<User> dto) throws Exception {
        //StringBuffer condition = SearchUtil.getUserCondition(dto);
        //condition.append(" ORDER BY id DESC");
        //condition.append(" limit ");
        //condition.append(dto.getIDisplayStart());
        //condition.append(",");
        //condition.append(dto.getIDisplayLength());
        if (StrUtil.isNotEmpty(dto.getAreaCode())){
            if (dto.getAreaCode().endsWith("0000")) {//省
                dto.setAreaCodeLevel(1);
            } else if (dto.getAreaCode().endsWith("00")) {//市
                dto.setAreaCodeLevel(2);
            } else {//区
                dto.setAreaCodeLevel(3);
            }
        }
        return userMapper.findUser(dto);
    }

    public long findCountOfUser(SearchDto<User> dto) throws Exception {
        //StringBuffer condition = SearchUtil.getUserCondition(dto);
        if (StrUtil.isNotEmpty(dto.getAreaCode())){
            if (dto.getAreaCode().endsWith("0000")) {//省
                dto.setAreaCodeLevel(1);
            } else if (dto.getAreaCode().endsWith("00")) {//市
                dto.setAreaCodeLevel(2);
            } else {//区
                dto.setAreaCodeLevel(3);
            }
        }
        return userMapper.findCountOfUser(dto);
    }

    public UserSummarySearchDto summarySearch(UserSummarySearchDto searchDto) {
        searchDto.setData(userMapper.summarySearch(searchDto));
        Long count = userMapper.summarySearchCount(searchDto);
        searchDto.setITotalDisplayRecords(count);
        searchDto.setITotalRecords(count);
        return searchDto;
    }

    public boolean isFirstApproveLevel(User user){
        String seq = userMapper.selectApproveLevelByUserId(user.getId());
        if (StringUtil.isEmpty(seq)) {
            return false;
        }
        if (Integer.parseInt(seq) == 1) {
            return true;
        }
        return false;
    }

    public void logLogin(User user){
        UserLoginLog log = new UserLoginLog();
        String ip = "";
        try{
            InetAddress addr = InetAddress.getLocalHost();
            ip = addr.getHostAddress(); //获得本机IP
        }catch(Exception e){
            ip = "127.0.0.1";
        }
        log.setIpAddress(ip);
        log.setUserId(user.getId());
        log.setLoginTime(new Date());
        userLoginLogMapper.insert(log);
    }
}
