package com.bcxin.sp.work.timer;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.bcxin.sp.work.dao.activity.extend.ActivityContractorMapper;
import com.bcxin.sp.work.entity.domain.activity.expand.ActivityContractor;
import com.bcxin.sp.work.entity.domain.log.FileSyncLog;
import com.bcxin.sp.work.service.log.LogService;
import com.bcxin.sp.work.util.BcxinDateUtil;
import com.bcxin.sp.work.util.BcxinFileUtil;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.StringUtil;
import com.bcxin.sp.work.util.sys.ConfigUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.tools.ant.util.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.util.*;

/**
 * <b>Function </b> 从pss获取承办者信息
 *
 * @author zxf
 * @注意事项 </b>
 * <b>
 */
@Service("syncContractorService")
@Transactional
@Slf4j
public class SyncContractorService {

    /**
     * 同步活动报批材料信息，文件标识
     */
    private final static String CONTRACTOR = "_contractor_";

    /**
     * 同步活动文件夹名称
     */
    private final static String CONTRACTOR_FOLDER = "contractor";

    /**
     * 将url包装成可下载的路径前缀
     */
    private final static String PACK_DOWNLOAD = "/getResource.do?path=";

    @Autowired
    private ActivityContractorMapper activityContractorMapper;

    @Autowired
    private LogService logService;


    /**
     * 同步承办者信息
     * @throws Exception
     */
    public void importData() throws Exception {
        log.info("同步承办者信息");
        String envi = ConfigUtil.getValue("envi");
        String mode = ConfigUtil.getValue("pss-communicate-mode");
        String upload_path = ConfigUtil.getValue("stuff-path");

        String[] dateDirs = new String[]{BcxinDateUtil.getCurrentDate(), BcxinDateUtil.getPastDate(1),
                BcxinDateUtil.getPastDate(2), BcxinDateUtil.getPastDate(3),
                BcxinDateUtil.getPastDate(4),  BcxinDateUtil.getPastDate(5),
                BcxinDateUtil.getPastDate(6)};

        /* 文件自动读取，只要读两天的文件就可以了 */
        if ("file".equals(mode)) {
            dateDirs = new String[]{BcxinDateUtil.getCurrentDate(), BcxinDateUtil.getPastDate(1)};
        }

        for (String dateDir : dateDirs) {
            List<File> fileList = Lists.newArrayList();
            //遍历当天的文件夹
            String[] folderPath = new String[]{upload_path, envi, dateDir, CONTRACTOR_FOLDER};
            String dir = BcxinFileUtil.appendFolderPath(folderPath);

            File dirFile = new File(dir);
            log.info("同步的目录：" + dir);
            if (dirFile.exists()) {
                File[] files = dirFile.listFiles();
                fileList.addAll(Arrays.asList(files));
            } else {
                dirFile.mkdirs();
            }

            if (CollectionUtil.isEmpty(fileList)) {
                log.info("文件夹是空的");
                continue;
            }

            List<String> fileNameList = Lists.newArrayList();
            for (File file : fileList) {
                //因为测试环境内外网用的同一个数据库测试，下面文件名统一加‘NW’前缀用于区分
                String fileName = "NW"+file.getName();
                if (fileName.startsWith("NW"+CONTRACTOR)) {
                    FileSyncLog fileSyncLog = logService.findRecord(fileName);
					/*如果文件名存在，证明已经同步过，则不再同步*/
                    if (fileSyncLog != null) {
                        if (DictConst.Y.equals(fileSyncLog.getMarkRead())) {
                            continue;
                        }
                    }
                    if (fileNameList.contains(fileName)) {
                        continue;
                    }

                    if (fileSyncLog == null) {
                        fileSyncLog = new FileSyncLog();
                    }
                    fileSyncLog.setFileName(fileName);

                    boolean flag = saveContractor(file, dir);
                    if (flag) {
                        fileSyncLog.setMarkRead(DictConst.Y);
                    }

                    logService.mergeMark(fileSyncLog);

                    fileNameList.add(fileName);
					/* 阅读完后删除文件 */
                    FileUtils.delete(file);
                }
            }
        }
    }

    private boolean saveContractor(File file, String dir) throws Exception {
        String content = BcxinFileUtil.txt2String(file);
        Map<String, String> valueMap = JSON.parseObject(content,Map.class);
        if (StringUtil.isEmpty(valueMap.get("pssId"))||StringUtil.isEmpty(valueMap.get("adminTel"))) {
            return false;
        }
        Map<String,Object> columnMap = Maps.newHashMap();
        columnMap.put("pssId", valueMap.get("pssId"));
        columnMap.put("adminTel", valueMap.get("adminTel"));
        List<ActivityContractor> acList = activityContractorMapper.selectByMap(columnMap);
        ActivityContractor ac;
        if (acList.size()<1) {
            ac = new ActivityContractor();
            ac.setPssId(Long.parseLong(valueMap.get("pssId")));
            ac.setContractorType(valueMap.get("contractorType"));
            ac.setContact(valueMap.get("contact"));
            ac.setOrganizationCode(valueMap.get("organizationCode"));
            ac.setFullName(valueMap.get("fullName"));
            ac.setProvince(valueMap.get("province"));
            ac.setCity(valueMap.get("city"));
            ac.setArea(valueMap.get("area"));
            ac.setAddress(valueMap.get("address"));
            ac.setAdminTel(valueMap.get("adminTel"));
            if (StringUtil.isNotEmpty(valueMap.get("applyTime"))) {
                ac.setApplyTime(BcxinDateUtil.matchFormatByLength(valueMap.get("applyTime")));
            }
            ac.setPhone(valueMap.get("phone"));
            ac.setEmail(valueMap.get("email"));
            ac.setAddTime(new Date());
            ac.setUpdateTime(new Date());
            if (StringUtil.isNotEmpty(valueMap.get("orgUrl"))) {
                ac.setOrgUrl(PACK_DOWNLOAD + dir + valueMap.get("orgUrl"));
            }
            activityContractorMapper.insert(ac);
        } else {
            ac = acList.get(0);
            ac.setContractorType(valueMap.get("contractorType"));
            ac.setContact(valueMap.get("contact"));
            ac.setOrganizationCode(valueMap.get("organizationCode"));
            ac.setFullName(valueMap.get("fullName"));
            ac.setProvince(valueMap.get("province"));
            ac.setCity(valueMap.get("city"));
            ac.setArea(valueMap.get("area"));
            ac.setAddress(valueMap.get("address"));
            if (StringUtil.isNotEmpty(valueMap.get("applyTime"))) {
                ac.setApplyTime(BcxinDateUtil.matchFormatByLength(valueMap.get("applyTime")));
            }
            ac.setPhone(valueMap.get("phone"));
            ac.setEmail(valueMap.get("email"));
            ac.setUpdateTime(new Date());
            if (StringUtil.isNotEmpty(valueMap.get("orgUrl"))) {
                ac.setOrgUrl(PACK_DOWNLOAD + dir + valueMap.get("orgUrl"));
            }
            activityContractorMapper.updateById(ac);
        }
        return true;
    }
}
