package com.bcxin.sp.work.timer;

import cn.hutool.core.collection.CollectionUtil;
import com.bcxin.sp.work.entity.domain.log.FileSyncLog;
import com.bcxin.sp.work.service.log.LogService;
import com.bcxin.sp.work.util.BcxinDateUtil;
import com.bcxin.sp.work.util.BcxinFileUtil;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.StringUtil;
import com.bcxin.sp.work.util.sys.ConfigUtil;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.apache.tools.ant.util.FileUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.BufferedReader;
import java.io.File;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <b>Function </b> 同步文件日志
 *
 * @author LuoPeng
 * @注意事项 </b>
 * <b>
 */
@Service("syncFileLogService")
@Transactional
@Slf4j
public class SyncFileLogService {

    /**
     * 同步文件日志，文件标识
     */
    private final static String FSLOG = "_fslog_";

    /**
     * 同步活动文件夹名称
     */
    private final static String LOG_FOLDER = "log";

    @Resource
    private LogService logService;

    /**
     * 同步文件日志
     * @throws Exception
     */
    public void importData() throws Exception {
        log.info("开始同步活动信息");
        String envi = ConfigUtil.getValue("envi");
        String mode = ConfigUtil.getValue("pss-communicate-mode");
        String upload_path = ConfigUtil.getValue("stuff-path");
        List<File> allFileList = Lists.newArrayList();

        String[] dateDirs = new String[]{BcxinDateUtil.getCurrentDate(), BcxinDateUtil.getPastDate(1),
                BcxinDateUtil.getPastDate(2), BcxinDateUtil.getPastDate(3),
                BcxinDateUtil.getPastDate(4),
                BcxinDateUtil.getPastDate(5), BcxinDateUtil.getPastDate(6)};

        /* 文件自动读取，只要读两天的文件就可以了 */
        if ("file".equals(mode)) {
            dateDirs = new String[]{BcxinDateUtil.getCurrentDate(), BcxinDateUtil.getPastDate(1)};
        }

        for(String dateDir:dateDirs) {
            //遍历当天的文件夹
            String[] folerPath = new String[]{upload_path, envi, dateDir, LOG_FOLDER};
            String dir = BcxinFileUtil.appendFolderPath(folerPath);

            File dirFile = new File(dir);
            log.info("同步的目录："+dir);
            if (dirFile.exists()) {
                File[] files = dirFile.listFiles();
                allFileList.addAll(Arrays.asList(files));
            } else {
                dirFile.mkdirs();
            }
        }

        if (CollectionUtil.isEmpty(allFileList)) {
            log.info("文件夹是空的");
            return;
        }

        List<String> syncFileNameList = Lists.newArrayList();
        for (File file : allFileList) {
            String fileName = file.getName();
            //同步活动信息
            if (fileName.startsWith(FSLOG)) {
                System.out.println("fileName:" + fileName);

                FileSyncLog fileSyncLog = logService.findRecord(fileName);
                /*如果文件名存在，证明已经同步过，则不再同步*/
                if (fileSyncLog != null) {
                    if (DictConst.Y.equals(fileSyncLog.getMarkRead())) {
                        continue;
                    }
                }
                if (syncFileNameList.contains(fileName)) {
                    continue;
                }

                if (fileSyncLog == null) {
                    fileSyncLog = new FileSyncLog();
                }
                fileSyncLog.setFileName(fileName);
                fileSyncLog.setSync(DictConst.Y);
                fileSyncLog.setMarkRead(DictConst.Y);

                saveFileLog(file);

                logService.mergeMark(fileSyncLog);
                syncFileNameList.add(fileName);
                /* 阅读完后删除文件 */
                FileUtils.delete(file);
            }
        }
    }


    private boolean saveFileLog(File file) throws Exception {

        Path fpath = Paths.get(file.getPath());
        try (BufferedReader bfr = java.nio.file.Files.newBufferedReader(fpath)) {
            List<String> list = bfr.lines().collect(Collectors.toList());
            for (String line : list) {
                if (StringUtil.isNotEmpty(line)) {
                    FileSyncLog filelog = logService.findRecord(line);
                    filelog.setSync(DictConst.Y);
                    logService.mergeMark(filelog);
                }
                log.info("数据行：" + line);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return true;
    }


}
