package com.bcxin.sp.work.timer;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Console;
import com.bcxin.sp.work.dao.sys.ServiceOrgMapper;
import com.bcxin.sp.work.entity.domain.log.FileSyncLog;
import com.bcxin.sp.work.entity.domain.sys.ServiceOrg;
import com.bcxin.sp.work.service.log.LogService;
import com.bcxin.sp.work.util.BcxinDateUtil;
import com.bcxin.sp.work.util.BcxinFileUtil;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.StringUtil;
import com.bcxin.sp.work.util.sys.ConfigUtil;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.apache.tools.ant.util.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedReader;
import java.io.File;
import java.lang.reflect.Field;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <b>Function </b> 从pss获取服务机构信息
 *
 * @author LuoPeng
 * @注意事项 </b>
 * <b>
 */
@Service("syncServiceOrgService")
@Transactional
@Slf4j
public class SyncServiceOrgService {

    /**
     * 同步活动报批材料信息，文件标识
     */
    private final static String ORG = "_org_";

    /**
     * 同步活动文件夹名称
     */
    private final static String ORG_FOLDER = "org";

    /**
     * 解析文件内容时，键值对的连接符
     */
    private final static String V_CONNECT = "_v_";


    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");

    @Autowired
    private ServiceOrgMapper serviceOrgMapper;

    @Autowired
    private LogService logService;


    /**
     * 同步服务机构信息
     * @throws Exception
     */
    public void importData() throws Exception {
        log.info("同步服务机构信息");
        String envi = ConfigUtil.getValue("envi");
        String mode = ConfigUtil.getValue("pss-communicate-mode");
        String upload_path = ConfigUtil.getValue("stuff-path");
        List<File> allFileList = Lists.newArrayList();

        String[] dateDirs = new String[]{BcxinDateUtil.getCurrentDate(), BcxinDateUtil.getPastDate(1),
                BcxinDateUtil.getPastDate(2), BcxinDateUtil.getPastDate(3),
                BcxinDateUtil.getPastDate(4),  BcxinDateUtil.getPastDate(5),
                BcxinDateUtil.getPastDate(6)};

        /* 文件自动读取，只要读两天的文件就可以了 */
        if ("file".equals(mode)) {
            dateDirs = new String[]{BcxinDateUtil.getCurrentDate(), BcxinDateUtil.getPastDate(1)};
        }

        for(String dateDir:dateDirs) {
            //遍历当天的文件夹
            String[] folerPath = new String[]{upload_path, envi, dateDir, ORG_FOLDER};
            String dir = BcxinFileUtil.appendFolderPath(folerPath);
            File dirFile = new File(dir);
            log.info("同步的目录："+dir);
            if (dirFile.exists()) {
                File[] files = dirFile.listFiles();
                allFileList.addAll(Arrays.asList(files));
            } else {
                dirFile.mkdirs();
            }
        }

        if (CollectionUtil.isEmpty(allFileList)) {
            log.info("文件夹是空的");
            return;
        }

        List<String> syncFileNameList = Lists.newArrayList();
        for (File file : allFileList) {
            String fileName = file.getName();
            if (fileName.startsWith(ORG)) {
                System.out.println("fileName:" + fileName);
                FileSyncLog fileSyncLog = logService.findRecord(fileName);
                /*如果文件名存在，证明已经同步过，则不再同步*/
                if (fileSyncLog != null) {
                    if (DictConst.Y.equals(fileSyncLog.getMarkRead())) {
                        continue;
                    }
                }
                if (syncFileNameList.contains(fileName)) {
                    continue;
                }

                if (fileSyncLog == null) {
                    fileSyncLog = new FileSyncLog();
                }
                fileSyncLog.setFileName(fileName);

                List<String> list = new ArrayList<>();
                /* 读取文件 */
                Path fpath = Paths.get(file.getPath());
                BufferedReader bfr = null;
                try {
                    bfr = java.nio.file.Files.newBufferedReader(fpath);
                    list = bfr.lines().collect(Collectors.toList());
                    bfr.close();
                    ServiceOrg org = new ServiceOrg();
                    for (String str:list) {
                        String[] line = str.split(V_CONNECT);
                        if (line.length < 2) {
                            continue;
                        }
                        Console.log("field:"+line[0]+"--------value:"+line[1]);
                        if (StringUtil.isEmpty(line[1])) {
                            continue;
                        }
                        Field[] fields = org.getClass().getDeclaredFields();
                        for( Field field : fields){
                            if (field.getName().equals(line[0])) {
                                field.setAccessible(true);
                                if( field.getType() == Date.class) {
                                    field.set(org, DateUtil.parse(line[1]));
                                } else {
                                    field.set(org, line[1]);
                                }
                            }
                        }
                    }
                    serviceOrgMapper.insert(org);
                } catch (Exception e) {
                    if(bfr != null){
                        bfr.close();
                    }
                    e.printStackTrace();
                    fileSyncLog.setMarkRead(DictConst.N);
                }
                fileSyncLog.setMarkRead(DictConst.Y);
                logService.mergeMark(fileSyncLog);
                syncFileNameList.add(fileName);
                /* 阅读完后删除文件 */
                FileUtils.delete(file);
            }
        }
    }

}
