package com.bcxin.sp.work.timer;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.bcxin.sp.work.dao.activity.extend.ActivityMaterialMapper;
import com.bcxin.sp.work.entity.domain.activity.base.Activity;
import com.bcxin.sp.work.entity.domain.activity.expand.ActivityMaterial;
import com.bcxin.sp.work.entity.domain.log.FileSyncLog;
import com.bcxin.sp.work.service.activity.base.ActivityService;
import com.bcxin.sp.work.service.activity.survey.SurveyService;
import com.bcxin.sp.work.service.log.LogService;
import com.bcxin.sp.work.util.BcxinDateUtil;
import com.bcxin.sp.work.util.BcxinFileUtil;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.Const.GlobalConst;
import com.bcxin.sp.work.util.sys.ConfigUtil;
import com.google.common.base.CharMatcher;
import com.google.common.collect.Lists;
import com.google.common.io.Files;
import lombok.extern.slf4j.Slf4j;
import org.apache.tools.ant.util.FileUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.util.Arrays;
import java.util.List;

/**
 * <b>Function </b> 同步监理报告信息
 *
 * @author LuoPeng
 * @注意事项 </b>
 * <b>
 */
@Service("syncSupervisorService")
@Transactional
@Slf4j
public class SyncSupervisorService {


    /**
     * 同步监理检查表，文件标识
     */
    private final static String JLJC = "_jljc_";

    /**
     * 同步监理报告，文件标识
     */
    private final static String JLBG = "_jlbg_";

    /**
     * 同步监理文件夹名称
     */
    private final static String JL_FOLDER = "jl";

    /**
     * 存储文件夹
     */
    private final static String STORE_FOLDER = "store";


    @Resource
    private ActivityService activityService;

    @Resource
    private SurveyService surveyService;

    @Resource
    private ActivityMaterialMapper activityMaterialMapper;

    @Resource
    private LogService logService;


    /**
     * 同步活动信息
     * 同步报批材料
     * @throws Exception
     */
    public void importData() throws Exception {
        log.info("开始同步监理报告信息");
        String month = DateUtil.today().substring(0,7);
        String envi = ConfigUtil.getValue("envi");
        String mode = ConfigUtil.getValue("pss-communicate-mode");
        String materialPath = ConfigUtil.getValue("stuff-path");
        List<File> allFileList = Lists.newArrayList();

        String[] dateDirs = new String[]{BcxinDateUtil.getCurrentDate(), BcxinDateUtil.getPastDate(1),
                BcxinDateUtil.getPastDate(2), BcxinDateUtil.getPastDate(3),
                BcxinDateUtil.getPastDate(4),
                BcxinDateUtil.getPastDate(5), BcxinDateUtil.getPastDate(6)};

        /* 文件自动读取，只要读两天的文件就可以了 */
        if ("file".equals(mode)) {
            dateDirs = new String[]{BcxinDateUtil.getCurrentDate(), BcxinDateUtil.getPastDate(1)};
        }

        for(String dateDir:dateDirs) {
            //遍历当天的文件夹
            String[] folerPath = new String[]{materialPath, envi, dateDir, JL_FOLDER};
            String dir = BcxinFileUtil.appendFolderPath(folerPath);

            File dirFile = new File(dir);
            log.info("同步的目录："+dir);
            if (dirFile.exists()) {
                File[] files = dirFile.listFiles();
                allFileList.addAll(Arrays.asList(files));
            } else {
                dirFile.mkdirs();
            }
        }

        if (CollectionUtil.isEmpty(allFileList)) {
            log.info("文件夹是空的");
            return;
        }
        List<String> fileNameList = Lists.newArrayList();
        for (File file : allFileList) {
            String fileName = file.getName();
            //同步监理检查表/监理报告
            if (fileName.startsWith(JLJC) || fileName.startsWith(JLBG)) {
                FileSyncLog fileSyncLog = logService.findRecord(fileName);
                /*如果文件名存在，证明已经同步过，则不再同步*/
                if (fileSyncLog != null) {
                    if (DictConst.Y.equals(fileSyncLog.getMarkRead())) {
                        continue;
                    }
                }
                if (fileNameList.contains(fileName)) {
                    continue;
                }
                if (fileSyncLog == null) {
                    fileSyncLog = new FileSyncLog();
                }
                fileSyncLog.setFileName(fileName);

                String activityId = CharMatcher.DIGIT.retainFrom(BcxinFileUtil.filterTimestamp(fileName));
                if (isNotEmpty(activityId)) {
                    Activity activity = activityService.findByPssId(Long.parseLong(activityId));
                    if (activity != null) { //还未开始导入
                        log.info("开始解析文件：" + fileName);
                        ActivityMaterial material = activityMaterialMapper.findByActivityId(activity.getId());
                        String[] folderPath = new String[]{materialPath, STORE_FOLDER ,month};
                        String dir = BcxinFileUtil.appendFolderPath(folderPath);
                        File newFile = new File(dir + File.separator + file.getName());
                        Files.copy(file, newFile);
                        if(fileName.startsWith(JLJC)){
                            material.setJljcb(GlobalConst.PACK_DOWNLOAD + newFile.getPath());
                        } else {
                            material.setJlbg(GlobalConst.PACK_DOWNLOAD + newFile.getPath());
                        }
                        activityMaterialMapper.updateById(material);

                        log.info("活动id：" + activity.getId());
                        log.info("设置监理检查表：" + material.getJljcb());
                        log.info("设置监理报告：" + material.getJlbg());

                        fileSyncLog.setMarkRead(DictConst.Y);
                        fileNameList.add(fileName);
                        /* 阅读完后删除文件 */
                        FileUtils.delete(file);
                    }
                }
                logService.mergeMark(fileSyncLog);
            }
        }

    }

    private boolean isNotEmpty(String value) {
        if (value == null) {
            return false;
        }
        if ("".equals(value.trim())) {
            return false;
        }
        return true;
    }
}
