package com.bcxin.sp.work.timer;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.bcxin.sp.work.dao.activity.survey.SurveyDetailMapper;
import com.bcxin.sp.work.entity.domain.activity.survey.Survey;
import com.bcxin.sp.work.entity.domain.activity.survey.SurveyDetail;
import com.bcxin.sp.work.entity.domain.log.FileSyncLog;
import com.bcxin.sp.work.service.activity.survey.SurveyService;
import com.bcxin.sp.work.service.log.LogService;
import com.bcxin.sp.work.util.BcxinDateUtil;
import com.bcxin.sp.work.util.BcxinFileUtil;
import com.bcxin.sp.work.util.Const.DictConst;
import com.bcxin.sp.work.util.StringUtil;
import com.bcxin.sp.work.util.sys.ConfigUtil;
import com.google.common.base.CharMatcher;
import com.google.common.base.Objects;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.tools.ant.util.FileUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <b>Function </b> 同步勘查信息
 * 
 * @author LuoPeng
 * @date 2017年1月16日 下午4:03:18
 * @注意事项 </b>
 * <b>
 */
@Service("syncSurveyService")
@Transactional
@Slf4j
public class SyncSurveyService {

	/**
	 * 存储文件夹名称
	 */
	private final static String STORE_FOLDER = "store";

	/**
	 * 同步勘查文件夹名称
	 */
	private final static String TK_FOLDER = "tk";

	/**
	 * 同步勘查文件，文件标识
	 */
	private final static String ST = "_st_";

	/**
	 * 同步勘查文件（回复整改），文件标识
	 */
	private final static String STR = "_str_";

	/**
	 * 将url包装成可下载的路径前缀
	 */
	private final static String PACK_DOWNLOAD = "/getResource.do?path=";


	SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");

	@Resource
	private SurveyService surveyService;

	@Resource
	private SurveyDetailMapper surveyDetailMapper;

	@Resource
	private LogService logService;

	public void importData() throws Exception {
		log.info("开始同步勘查数据...");

		String envi = ConfigUtil.getValue("envi");
		String mode = ConfigUtil.getValue("pss-communicate-mode");
		String upload_path = ConfigUtil.getValue("stuff-path");

		String[] dateDirs = new String[]{BcxinDateUtil.getCurrentDate(), BcxinDateUtil.getPastDate(1),
				BcxinDateUtil.getPastDate(2), BcxinDateUtil.getPastDate(3),
				BcxinDateUtil.getPastDate(4),
				BcxinDateUtil.getPastDate(5), BcxinDateUtil.getPastDate(6)};

		/* 文件自动读取，只要读两天的文件就可以了 */
		if ("file".equals(mode)) {
			dateDirs = new String[]{BcxinDateUtil.getCurrentDate(), BcxinDateUtil.getPastDate(1)};
		}

		for (String dateDir : dateDirs) {
			List<File> fileList = Lists.newArrayList();
			//遍历当天的文件夹
			String[] folderPath = new String[]{upload_path, envi, dateDir, TK_FOLDER};
			String dir = BcxinFileUtil.appendFolderPath(folderPath);

			File dirFile = new File(dir);
			log.info("同步的目录：" + dir);
			if (dirFile.exists()) {
				File[] files = dirFile.listFiles();
				fileList.addAll(Arrays.asList(files));
			} else {
				dirFile.mkdirs();
			}

			if (CollectionUtil.isEmpty(fileList)) {
				log.info("文件夹是空的");
				continue;
			}

			List<String> fileNameList = Lists.newArrayList();
			for (File file : fileList) {
				String fileName = file.getName();
				//同步勘查信息（勘查图片和勘查报告以文件的形式放到文件夹，勘查信息通过路径对应）
				if (fileName.startsWith(ST)) {
					FileSyncLog fileSyncLog = logService.findRecord(fileName);
					/*如果文件名存在，证明已经同步过，则不再同步*/
					if (fileSyncLog != null) {
						if ("1".equals(fileSyncLog.getMarkRead())) {
							continue;
						}
					}
					if (fileNameList.contains(fileName)) {
						continue;
					}

					if (fileSyncLog == null) {
						fileSyncLog = new FileSyncLog();
					}
					fileSyncLog.setFileName(fileName);

					boolean flag = saveSurvey(file, dir);
					if (flag) {
						fileSyncLog.setMarkRead(DictConst.Y);
					}

					logService.mergeMark(fileSyncLog);

					fileNameList.add(fileName);
					/* 阅读完后删除文件 */
					FileUtils.delete(file);
				} else if (fileName.startsWith(STR)) {
					FileSyncLog fileSyncLog = logService.findRecord(fileName);
					/*如果文件名存在，证明已经同步过，则不再同步*/
					if (fileSyncLog != null) {
						if ("1".equals(fileSyncLog.getMarkRead())) {
							continue;
						}
					}
					if (fileNameList.contains(fileName)) {
						continue;
					}

					if (fileSyncLog == null) {
						fileSyncLog = new FileSyncLog();
					}
					fileSyncLog.setFileName(fileName);

					boolean flag = saveSurveyReform(file, dir);
					if (flag) {
						fileSyncLog.setMarkRead(DictConst.Y);
					}

					logService.mergeMark(fileSyncLog);

					fileNameList.add(fileName);
					/* 阅读完后删除文件 */
					FileUtils.delete(file);
				}
			}
		}
	}

	private boolean saveSurvey(File file, String dir) throws Exception {
		String content = BcxinFileUtil.txt2String(file);
		Map<String, String> valueMap = JSON.parseObject(content,Map.class);
		if (StringUtil.isEmpty(valueMap.get("activityNo"))) {
			return false;
		}
		String address = valueMap.get("address");
		String step = valueMap.get("step");
		Survey survey = surveyService.findByStep(valueMap.get("activityNo"), address, step);
		if (survey == null) {
			survey = new Survey();
			survey.setActivityNo(valueMap.get("activityNo"));
			survey.setAddress(address);
			survey.setStep(step);
			survey.setStartDate(isNotEmpty(valueMap.get("startDate")) ? sdf.parse(valueMap.get("startDate")) : new Date());
			survey.setEndDate(isNotEmpty(valueMap.get("endDate")) ? sdf.parse(valueMap.get("endDate")) : new Date());
			if (StringUtil.isNotEmpty(valueMap.get("sr"))) {
				survey.setReport(PACK_DOWNLOAD + dir + valueMap.get("sr"));
			}
			surveyService.save(survey);
		} else {
			/* 有踏勘报告才进行更新 */
			if (StringUtil.isNotEmpty(valueMap.get("sr"))) {
				if (!Objects.equal(survey.getReport(), PACK_DOWNLOAD + dir + valueMap.get("sr"))) {
					survey.setReport(PACK_DOWNLOAD + dir + valueMap.get("sr"));
					surveyService.update(survey);
				}
			}
		}

		if (survey.getId() != null && survey.getId() > 0) {
			surveyService.deleteInfo(survey); //先删掉原来的勘查详情，重新插入
		}

		Map<String, SurveyDetail> infoMap = Maps.newHashMap();
		Iterator<String> iterator = valueMap.keySet().iterator();
		while (iterator.hasNext()) {
			String key = iterator.next();
			if (!(key.startsWith("opinion") || key.startsWith("solve") || key.startsWith("infoId"))) {
				continue;
			}
			String digitKey = CharMatcher.DIGIT.retainFrom(key);
			SurveyDetail info = infoMap.get(digitKey);
			if (info == null) {
				info = new SurveyDetail();
				info.setSurveyId(survey.getId());
			}
			if (key.startsWith("infoId")) {
				info.setPssInfoId(Long.parseLong(valueMap.get(key)));
			} else if (key.startsWith("opinion") && !key.startsWith("opinionPic")) {
				info.setOpinion(valueMap.get(key));
			} else if (key.startsWith("solve") && !key.startsWith("solvePic")) {
				if (StringUtil.isEmpty(valueMap.get(key)) || Objects.equal(valueMap.get(key), "null")) {
					continue;
				}
				info.setSolve(valueMap.get(key));
			}
			if (key.startsWith("opinionPic")) {
				if (StringUtil.isEmpty(valueMap.get(key)) || Objects.equal(valueMap.get(key), "null")) {
					continue;
				}
				String value = valueMap.get(key);
				String[] values = StringUtil.split(value, "&&");
				StringBuffer newPath = new StringBuffer();
				for (String v : values) {
					if (StringUtil.isNotEmpty(v)) {
						newPath.append(PACK_DOWNLOAD + dir + v).append("&&");
					}
				}
				if (newPath.toString().endsWith("&&")) {
					newPath.substring(0, newPath.length() - 2);
				}
				info.setOpinionPic(newPath.toString());
			} else if (key.startsWith("solvePic")) {
				if (StringUtil.isEmpty(valueMap.get(key)) || Objects.equal(valueMap.get(key), "null")) {
					continue;
				}
				String value = valueMap.get(key);
				String[] values = StringUtil.split(value, "&&");
				StringBuilder newPath = new StringBuilder();
				for (String v : values) {
					if (StringUtil.isNotEmpty(v)) {
						newPath.append(PACK_DOWNLOAD + dir + v).append("&&");
					}
				}
				if (newPath.toString().endsWith("&&")) {
					newPath.substring(0, newPath.length() - 2);
				}
				info.setSolvePic(newPath.toString());
			}
			infoMap.put(digitKey, info);
		}
		for (String key : infoMap.keySet()) {
			surveyService.save(infoMap.get(key)); //保存
		}
		return true;
	}


	private boolean saveSurveyReform(File file, String dir) throws Exception {
		String content = BcxinFileUtil.txt2String(file);
		Map<String, String> valueMap = JSON.parseObject(content,LinkedHashMap.class);
		if (StringUtil.isEmpty(valueMap.get("activityNo"))) {
			return false;
		}
		String address = valueMap.get("address");
		String step = valueMap.get("step");
		Survey survey = surveyService.findByStep(valueMap.get("activityNo"), address, step);
		if (survey == null) {
			return false;
		}

		Map<String, SurveyDetail> infoMap = Maps.newHashMap();
		for (String key : valueMap.keySet()) {
			if (!(key.startsWith("solve") || key.startsWith("infoId"))) {
				continue;
			}
			String digitKey = CharMatcher.DIGIT.retainFrom(key);
			SurveyDetail info = infoMap.get(digitKey);
			if (info == null) {
				info = surveyDetailMapper.findByPSSInfoId(Long.parseLong(valueMap.get(key)));
				if (info == null) {
					info = new SurveyDetail();
				}
			}
			if (key.startsWith("infoId")) {
				info.setPssInfoId(Long.parseLong(valueMap.get(key)));
			} else if (key.startsWith("solve") && !key.startsWith("solvePic")) {
				if (StringUtil.isEmpty(valueMap.get(key)) || Objects.equal(valueMap.get(key), "null")) {
					continue;
				}
				info.setSolve(valueMap.get(key));
			}
			if (key.startsWith("solvePic")) {
				if (StringUtil.isEmpty(valueMap.get(key)) || Objects.equal(valueMap.get(key), "null")) {
					continue;
				}
				String value = valueMap.get(key);
				String[] values = StringUtil.split(value, "&&");
				StringBuffer newPath = new StringBuffer();
				for (String v : values) {
					if (StringUtil.isNotEmpty(v)) {
						newPath.append(PACK_DOWNLOAD + dir + v).append("&&");
					}
				}
				if (newPath.toString().endsWith("&&")) {
					newPath.substring(0, newPath.length() - 2);
				}
				info.setSolvePic(newPath.toString());
			}
			infoMap.put(digitKey, info);
		}
		for (String key : infoMap.keySet()) {
			SurveyDetail temp = infoMap.get(key);
			if (temp.getId() == null) {
				surveyService.save(temp); //保存
			} else {
				surveyService.update(temp); //修改
			}
		}
		return true;
	}

	private boolean isNotEmpty(String value) {
		if (value == null) {
			return false;
		}
		if ("".equals(value.trim())) {
			return false;
		}
		return true;
	}
}
