package com.bcxin.sp.work.util;


import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import org.apache.commons.lang3.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class BcxinDateUtil {

	public static final String FORMAT1 = "yyyy-MM-dd HH:mm:ss";
	public static final String FORMAT2 = "yyyy-MM-dd";
	public static final String FORMAT3 = "MM/dd/yyyy HH:mm:ss";
	public static final String FORMAT4 = "MM/dd/yyyy";
	public static final String FORMAT5 = "HH:mm:ss";
	public static final String FORMAT6 = "yyyy/MM/dd";
	public static final String FORMAT7 = "yyyy-MM-dd HH:mm";
	public static final String FORMAT8 = "yyyyMMddhhmmss";

	public static SimpleDateFormat systemDate = new SimpleDateFormat("yyyy-MM-dd");

	public static SimpleDateFormat systemTime = new SimpleDateFormat("yyyy-MM-dd HH:mm");

	public static SimpleDateFormat chinaDate = new SimpleDateFormat("yyyy年MM月dd日");
	
	public static SimpleDateFormat chinaTime = new SimpleDateFormat("yyyy年MM月dd日 HH时mm分");

	/** 标准日期（不含时间）格式化器 */
	private final static SimpleDateFormat NORM_DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd");
	/** 标准日期时间格式化器 */
	private final static SimpleDateFormat NORM_DATETIME_FORMAT1 = new SimpleDateFormat("yyyy-MM-dd HH:mm");
	/** 标准日期时间格式化器 */
	private final static SimpleDateFormat NORM_DATETIME_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	/**
	 * 获取当前日期
	 *
	 * @return
	 */
	public static String getCurrentDate() {
		return getCurrentDateTime(FORMAT2);
	}


	/**
	 * 根据特定格式获取当前时间
	 *
	 * @param format
	 * @return
	 */
	public static String getCurrentDateTime(String format) {
		SimpleDateFormat df = new SimpleDateFormat(format);
		return df.format(new Date());
	}

	public static String convertDateToString(Date date, String format) {
		SimpleDateFormat df = new SimpleDateFormat(format);
		return df.format(date);
	}

	public static Date addMonth(Date date, int month) {
		Calendar gc = prepare(date);
		gc.add(Calendar.MONTH, month);
		return gc.getTime();
	}

	public static Date addDay(Date date, int day) {
		Calendar gc = prepare(date);
		gc.add(Calendar.DAY_OF_MONTH, day);
		return gc.getTime();
	}

	private static Calendar prepare(Date date) {
		Calendar gc = Calendar.getInstance();
		gc.setTime(date);
		return gc;
	}

	/**
	 * 获取当前日期yyyymmddhhmmss
	 *
	 * @return
	 */
	public static String getTimestamp() {
		return getCurrentDateTime(FORMAT8);
	}

	/**
	 * 获取过去第几天的日期(- 操作) 或者 未来 第几天的日期( + 操作)
	 *
	 * @param past
	 * @return
	 */
	public static String getPastDate(int past) {
		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.DAY_OF_YEAR, calendar.get(Calendar.DAY_OF_YEAR) - past);
		Date today = calendar.getTime();
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
		String result = format.format(today);
		return result;
	}


	/**
	 * 计算两个时间的差值，并换算成中文
	 * @param beginDate
	 * @param endDate
	 * @return
	 */
	public static String diffFormatChina(Date beginDate,Date endDate) {
		long min = DateUtil.between(beginDate,endDate, DateUnit.MINUTE);
		long day = min/(24*60);
		long hour = (min-(day*(24*60)))/60;
		long mins = min - (day*(24*60) + hour*60);
		return day + "天" + hour + "小时" + mins + "分";
	}

	public static String diffFormatChina_F(Date date,String elapsed) {
		if(StringUtils.isEmpty(elapsed)){
			return DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS,date);
		}
		String day = elapsed.split("天")[0];
		String hour = elapsed.split("天")[1].split("小时")[0];
		String mins = elapsed.split("天")[1].split("小时")[1].split("分")[0];
		Calendar gc = prepare(date);
		gc.add(Calendar.DAY_OF_MONTH, Integer.parseInt(day));
		gc.add(Calendar.HOUR_OF_DAY, Integer.parseInt(hour));
		gc.add(Calendar.MINUTE, Integer.parseInt(mins));
		return convertDateToString(gc.getTime(),FORMAT1);
	}

	public static Date matchFormatByLength(String dateString) {
		try {
			int num = dateString.length();
			if(num == 16){
				return NORM_DATETIME_FORMAT1.parse(dateString);
			}else if(num == 19){
				return NORM_DATETIME_FORMAT.parse(dateString);
			}else {
				return NORM_DATE_FORMAT.parse(dateString);
			}
		} catch (ParseException e) {
		}
		return null;
	}

	public static void main(String[] args) {
		System.out.println(diffFormatChina_F(new Date(),"0天17小时4分"));
//		System.out.println(getPastDate(1));
//		System.out.println(getPastDate(2));
//		System.out.println(getPastDate(3));
//		System.out.println(getPastDate(4));
//		System.out.println(getPastDate(5));
//		System.out.println(getPastDate(6));
//		System.out.println(getPastDate(7));
	}
}
