package com.bcxin.sp.work.util;


import cn.hutool.core.map.MapUtil;
import cn.hutool.http.HttpUtil;
import com.google.common.base.Stopwatch;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.TrustStrategy;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.SSLContext;
import java.nio.charset.StandardCharsets;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

public class BcxinHttpUtil {

	private static Logger log = LoggerFactory.getLogger(HttpUtil.class);

	/**
	 * POST方式调用
	 *
	 * @param url
	 * @param params
	 *            参数为NameValuePair键值对对象
	 * @return 响应字符串
	 * @throws java.io.UnsupportedEncodingException
	 */
	public static String sendPostRequest(String url, List<NameValuePair> params) {
		log.debug("--------开始发送http请求--------");
		log.debug("请求地址:[{}]：",url);
		logParams(params);
		Stopwatch stopwatch = Stopwatch.createStarted();
		String result = "";
		try (CloseableHttpClient httpClient = createSSLClientDefault()) {
			HttpPost httpPost = new HttpPost(url);
			httpPost.setEntity(new UrlEncodedFormEntity(params, StandardCharsets.UTF_8));
			RequestConfig defaultRequestConfig = RequestConfig.custom()
					.setSocketTimeout(60000)
					.setConnectTimeout(60000)
					.setConnectionRequestTimeout(60000)
					.build();
			httpPost.setConfig(defaultRequestConfig);
			try (CloseableHttpResponse resp = httpClient.execute(httpPost)) {
				HttpEntity entity = resp.getEntity();
				result = EntityUtils.toString(entity, StandardCharsets.UTF_8);
				long nanos = stopwatch.elapsed(TimeUnit.MILLISECONDS);
				log.debug("返回结果：[{}]", result);
				log.debug("共耗时[{}]毫秒：" + nanos);
				log.debug("--------结束http请求--------");
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return result;
	}


	/**
	 * POST方式调用
	 *
	 * @param url
	 * @param params
	 *            参数为NameValuePair键值对对象
	 * @return 响应字符串
	 * @throws java.io.UnsupportedEncodingException
	 */
	public static String sendPostRequest(String url, List<NameValuePair> params,Map<String,String> headerMap) {
		log.debug("--------开始发送http请求--------");
		log.debug("请求地址:[{}]：",url);
		logParams(params);
		Stopwatch stopwatch = Stopwatch.createStarted();
		String result = "";
		try (CloseableHttpClient httpClient = createSSLClientDefault()) {
			HttpPost httpPost = new HttpPost(url);
			if (MapUtil.isNotEmpty(headerMap)) {
				for (String key : headerMap.keySet()) {
					httpPost.addHeader(key,headerMap.get(key));
				}
			}
			httpPost.setEntity(new UrlEncodedFormEntity(params, StandardCharsets.UTF_8));
			RequestConfig defaultRequestConfig = RequestConfig.custom()
					.setSocketTimeout(60000)
					.setConnectTimeout(60000)
					.setConnectionRequestTimeout(60000)
					.build();
			httpPost.setConfig(defaultRequestConfig);
			try (CloseableHttpResponse resp = httpClient.execute(httpPost)) {
				HttpEntity entity = resp.getEntity();
				result = EntityUtils.toString(entity, StandardCharsets.UTF_8);
				long nanos = stopwatch.elapsed(TimeUnit.MILLISECONDS);
				log.debug("返回结果：[{}]", result);
				log.debug("共耗时[{}]毫秒：" + nanos);
				log.debug("--------结束http请求--------");
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return result;
	}


	private static void logParams(List<NameValuePair> params){
		for(NameValuePair pair:params){
			log.debug("请求的参数名：[{}]，值：[{}]",pair.getName(),pair.getValue());
		}
	}

	//绕过ssl验证
	public static CloseableHttpClient createSSLClientDefault() {
		try {
			SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, new TrustStrategy() {
				// 信任所有
				@Override
				public boolean isTrusted(X509Certificate[] chain, String authType) throws CertificateException {
					return true;
				}
			}).build();
			SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslContext, NoopHostnameVerifier.INSTANCE);
			return HttpClients.custom().setSSLSocketFactory(sslsf).build();
		} catch (KeyManagementException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (KeyStoreException e) {
			e.printStackTrace();
		}
		return HttpClients.createDefault();
	}
}
